@@ -373,6 +373,7 @@ abstract class EntityDisplayBase extends ConfigEntityBase implements EntityDispl
   /**
    * {@inheritdoc}
    */
+  #[ActionMethod(adminLabel: new TranslatableMarkup('Hide component'), name: 'hideComponent')]
   public function removeComponent($name) {
     $this->hidden[$name] = TRUE;
     unset($this->content[$name]);
@@ -587,4 +588,20 @@ abstract class EntityDisplayBase extends ConfigEntityBase implements EntityDispl
     return \Drupal::logger('system');
   }
 
+  /**
+   * {@inheritdoc}
+   */
+  public function set($property_name, $value): static {
+    // If changing the entity ID, also update the target entity type, bundle,
+    // and view mode.
+    if ($this->isNew() && $property_name === $this->getEntityType()->getKey('id')) {
+      if (substr_count($value, '.') !== 2) {
+        throw new \InvalidArgumentException("'$value' is not a valid entity display ID.");
+      }
+      [$this->targetEntityType, $this->bundle, $this->mode] = explode('.', $value);
+    }
+    parent::set($property_name, $value);
+    return $this;
+  }
+
 }
