@@ -4,18 +4,25 @@ namespace Drupal\Core\Render;
 
 use Drupal\Component\Render\MarkupInterface;
 use Drupal\Component\Utility\Html;
+use Drupal\Component\Utility\Variable;
 use Drupal\Component\Utility\Xss;
 use Drupal\Core\Access\AccessResultInterface;
 use Drupal\Core\Cache\Cache;
 use Drupal\Core\Cache\CacheableMetadata;
 use Drupal\Core\Controller\ControllerResolverInterface;
+use Drupal\Core\Form\FormHelper;
+use Drupal\Core\Render\Element\RenderCallbackInterface;
+use Drupal\Core\Security\TrustedCallbackInterface;
+use Drupal\Core\Security\DoTrustedCallbackTrait;
 use Drupal\Core\Theme\ThemeManagerInterface;
+use Drupal\Core\Utility\CallableResolver;
 use Symfony\Component\HttpFoundation\RequestStack;
 
 /**
- * Turns a render array into a HTML string.
+ * Turns a render array into an HTML string.
  */
 class Renderer implements RendererInterface {
+  use DoTrustedCallbackTrait;
 
   /**
    * The theme manager.
@@ -25,11 +32,11 @@ class Renderer implements RendererInterface {
   protected $theme;
 
   /**
-   * The controller resolver.
+   * The callable resolver.
    *
-   * @var \Drupal\Core\Controller\ControllerResolverInterface
+   * @var \Drupal\Core\Utility\CallableResolver
    */
-  protected $controllerResolver;
+  protected CallableResolver $callableResolver;
 
   /**
    * The element info.
@@ -93,8 +100,8 @@ class Renderer implements RendererInterface {
   /**
    * Constructs a new Renderer.
    *
-   * @param \Drupal\Core\Controller\ControllerResolverInterface $controller_resolver
-   *   The controller resolver.
+   * @param \Drupal\Core\Utility\CallableResolver|\Drupal\Core\Controller\ControllerResolverInterface $callable_resolver
+   *   The callable resolver.
    * @param \Drupal\Core\Theme\ThemeManagerInterface $theme
    *   The theme manager.
    * @param \Drupal\Core\Render\ElementInfoManagerInterface $element_info
@@ -108,12 +115,19 @@ class Renderer implements RendererInterface {
    * @param array $renderer_config
    *   The renderer configuration array.
    */
-  public function __construct(ControllerResolverInterface $controller_resolver, ThemeManagerInterface $theme, ElementInfoManagerInterface $element_info, PlaceholderGeneratorInterface $placeholder_generator, RenderCacheInterface $render_cache, RequestStack $request_stack, array $renderer_config) {
-    $this->controllerResolver = $controller_resolver;
+  public function __construct(ControllerResolverInterface|CallableResolver $callable_resolver, ThemeManagerInterface $theme, ElementInfoManagerInterface $element_info, PlaceholderGeneratorInterface $placeholder_generator, RenderCacheInterface $render_cache, RequestStack $request_stack, array $renderer_config) {
+    if ($callable_resolver instanceof ControllerResolverInterface) {
+      @trigger_error('Calling ' . __METHOD__ . '() with an argument of ControllerResolverInterface is deprecated in drupal:10.2.0 and is removed in drupal:11.0.0. Use \Drupal\Core\Utility\CallableResolver instead. See https://www.drupal.org/node/3369969', E_USER_DEPRECATED);
+      $callable_resolver = \Drupal::service('callable_resolver');
+    }
+    $this->callableResolver = $callable_resolver;
     $this->theme = $theme;
     $this->elementInfo = $element_info;
     $this->placeholderGenerator = $placeholder_generator;
     $this->renderCache = $render_cache;
+    if (!isset($renderer_config['debug'])) {
+      $renderer_config['debug'] = FALSE;
+    }
     $this->rendererConfig = $renderer_config;
     $this->requestStack = $request_stack;
 
@@ -146,7 +160,7 @@ class Renderer implements RendererInterface {
   /**
    * {@inheritdoc}
    */
-  public function renderPlain(&$elements) {
+  public function renderInIsolation(&$elements) {
     return $this->executeInRenderContext(new RenderContext(), function () use (&$elements) {
       return $this->render($elements, TRUE);
     });
@@ -155,20 +169,49 @@ class Renderer implements RendererInterface {
   /**
    * {@inheritdoc}
    */
-  public function renderPlaceholder($placeholder, array $elements) {
-    // Get the render array for the given placeholder
-    $placeholder_elements = $elements['#attached']['placeholders'][$placeholder];
+  public function renderPlain(&$elements) {
+    @trigger_error('Renderer::renderPlain() is deprecated in drupal:10.3.0 and is removed from drupal:12.0.0. Instead, you should use ::renderInIsolation(). See https://www.drupal.org/node/3407994', E_USER_DEPRECATED);
+    return $this->renderInIsolation($elements);
+  }
 
+  /**
+   * Renders a placeholder into markup.
+   *
+   * @param array $placeholder_element
+   *   The placeholder element by reference.
+   *
+   * @return \Drupal\Component\Render\MarkupInterface|string
+   *   The rendered HTML.
+   */
+  protected function doRenderPlaceholder(array &$placeholder_element): MarkupInterface|string {
     // Prevent the render array from being auto-placeholdered again.
-    $placeholder_elements['#create_placeholder'] = FALSE;
+    $placeholder_element['#create_placeholder'] = FALSE;
 
     // Render the placeholder into markup.
-    $markup = $this->renderPlain($placeholder_elements);
+    $markup = $this->renderInIsolation($placeholder_element);
+    return $markup;
+  }
 
+  /**
+   * Replaces a placeholder with its markup.
+   *
+   * @param string $placeholder
+   *   The placeholder HTML.
+   * @param \Drupal\Component\Render\MarkupInterface|string $markup
+   *   The markup to replace the placeholder with.
+   * @param array $elements
+   *   The render array that the placeholder is from.
+   * @param array $placeholder_element
+   *   The placeholder element render array.
+   *
+   * @return \Drupal\Component\Render\MarkupInterface|string
+   *   The rendered HTML.
+   */
+  protected function doReplacePlaceholder(string $placeholder, string|MarkupInterface $markup, array $elements, array $placeholder_element): array {
     // Replace the placeholder with its rendered markup, and merge its
     // bubbleable metadata with the main elements'.
     $elements['#markup'] = Markup::create(str_replace($placeholder, $markup, $elements['#markup']));
-    $elements = $this->mergeBubbleableMetadata($elements, $placeholder_elements);
+    $elements = $this->mergeBubbleableMetadata($elements, $placeholder_element);
 
     // Remove the placeholder that we've just rendered.
     unset($elements['#attached']['placeholders'][$placeholder]);
@@ -176,6 +219,16 @@ class Renderer implements RendererInterface {
     return $elements;
   }
 
+  /**
+   * {@inheritdoc}
+   */
+  public function renderPlaceholder($placeholder, array $elements) {
+    // Get the render array for the given placeholder
+    $placeholder_element = $elements['#attached']['placeholders'][$placeholder];
+    $markup = $this->doRenderPlaceholder($placeholder_element);
+    return $this->doReplacePlaceholder($placeholder, $markup, $elements, $placeholder_element);
+  }
+
   /**
    * {@inheritdoc}
    */
@@ -209,20 +262,29 @@ class Renderer implements RendererInterface {
       return '';
     }
 
+    if ($this->rendererConfig['debug'] === TRUE) {
+      $render_start = microtime(TRUE);
+    }
+
     if (!isset($elements['#access']) && isset($elements['#access_callback'])) {
-      if (is_string($elements['#access_callback']) && strpos($elements['#access_callback'], '::') === FALSE) {
-        $elements['#access_callback'] = $this->controllerResolver->getControllerFromDefinition($elements['#access_callback']);
-      }
-      $elements['#access'] = call_user_func($elements['#access_callback'], $elements);
+      $elements['#access'] = $this->doCallback('#access_callback', $elements['#access_callback'], [$elements]);
     }
 
     // Early-return nothing if user does not have access.
     if (isset($elements['#access'])) {
-      // If #access is an AccessResultInterface object, we must apply it's
+      // If #access is an AccessResultInterface object, we must apply its
       // cacheability metadata to the render array.
       if ($elements['#access'] instanceof AccessResultInterface) {
         $this->addCacheableDependency($elements, $elements['#access']);
         if (!$elements['#access']->isAllowed()) {
+          // Abort, but bubble new cache metadata from the access result.
+          $context = $this->getCurrentRenderContext();
+          if (!isset($context)) {
+            throw new \LogicException("Render context is empty, because render() was called outside of a renderRoot() or renderInIsolation() call. Use renderInIsolation()/renderRoot() or #lazy_builder/#pre_render instead.");
+          }
+          $context->push(new BubbleableMetadata());
+          $context->update($elements);
+          $context->bubble();
           return '';
         }
       }
@@ -238,7 +300,7 @@ class Renderer implements RendererInterface {
 
     $context = $this->getCurrentRenderContext();
     if (!isset($context)) {
-      throw new \LogicException("Render context is empty, because render() was called outside of a renderRoot() or renderPlain() call. Use renderPlain()/renderRoot() or #lazy_builder/#pre_render instead.");
+      throw new \LogicException("Render context is empty, because render() was called outside of a renderRoot() or renderInIsolation() call. Use renderInIsolation()/renderRoot() or #lazy_builder/#pre_render instead.");
     }
     $context->push(new BubbleableMetadata());
 
@@ -273,6 +335,10 @@ class Renderer implements RendererInterface {
         if (is_string($elements['#markup'])) {
           $elements['#markup'] = Markup::create($elements['#markup']);
         }
+        // Add debug output to the renderable array on cache hit.
+        if ($this->rendererConfig['debug'] === TRUE) {
+          $elements = $this->addDebugOutput($elements, TRUE);
+        }
         // The render cache item contains all the bubbleable rendering metadata
         // for the subtree.
         $context->update($elements);
@@ -289,6 +355,7 @@ class Renderer implements RendererInterface {
     $pre_bubbling_elements = array_intersect_key($elements, [
       '#cache' => TRUE,
       '#lazy_builder' => TRUE,
+      '#lazy_builder_preview' => TRUE,
       '#create_placeholder' => TRUE,
     ]);
 
@@ -301,37 +368,26 @@ class Renderer implements RendererInterface {
     // First validate the usage of #lazy_builder; both of the next if-statements
     // use it if available.
     if (isset($elements['#lazy_builder'])) {
-      // @todo Convert to assertions once https://www.drupal.org/node/2408013
-      //   lands.
-      if (!is_array($elements['#lazy_builder'])) {
-        throw new \DomainException('The #lazy_builder property must have an array as a value.');
-      }
-      if (count($elements['#lazy_builder']) !== 2) {
-        throw new \DomainException('The #lazy_builder property must have an array as a value, containing two values: the callback, and the arguments for the callback.');
-      }
-      if (count($elements['#lazy_builder'][1]) !== count(array_filter($elements['#lazy_builder'][1], function($v) {
+      assert(is_array($elements['#lazy_builder']), 'The #lazy_builder property must have an array as a value.');
+      assert(count($elements['#lazy_builder']) === 2, 'The #lazy_builder property must have an array as a value, containing two values: the callback, and the arguments for the callback.');
+      assert(is_array($elements['#lazy_builder'][1]), 'The #lazy_builder argument for callback must have an array as a value.');
+      assert(count($elements['#lazy_builder'][1]) === count(array_filter($elements['#lazy_builder'][1], function ($v) {
         return is_null($v) || is_scalar($v);
-      }))) {
-        throw new \DomainException("A #lazy_builder callback's context may only contain scalar values or NULL.");
-      }
-      $children = Element::children($elements);
-      if ($children) {
-        throw new \DomainException(sprintf('When a #lazy_builder callback is specified, no children can exist; all children must be generated by the #lazy_builder callback. You specified the following children: %s.', implode(', ', $children)));
-      }
+      })), "A #lazy_builder callback's context may only contain scalar values or NULL.");
+      assert(!Element::children($elements), sprintf('When a #lazy_builder callback is specified, no children can exist; all children must be generated by the #lazy_builder callback. You specified the following children: %s.', implode(', ', Element::children($elements))));
       $supported_keys = [
         '#lazy_builder',
         '#cache',
         '#create_placeholder',
+        '#lazy_builder_preview',
+        '#preview',
         // The keys below are not actually supported, but these are added
         // automatically by the Renderer. Adding them as though they are
         // supported allows us to avoid throwing an exception 100% of the time.
         '#weight',
-        '#printed'
+        '#printed',
       ];
-      $unsupported_keys = array_diff(array_keys($elements), $supported_keys);
-      if (count($unsupported_keys)) {
-        throw new \DomainException(sprintf('When a #lazy_builder callback is specified, no properties can exist; all properties must be generated by the #lazy_builder callback. You specified the following properties: %s.', implode(', ', $unsupported_keys)));
-      }
+      assert(empty(array_diff(array_keys($elements), $supported_keys)), sprintf('When a #lazy_builder callback is specified, no properties can exist; all properties must be generated by the #lazy_builder callback. You specified the following properties: %s.', implode(', ', array_diff(array_keys($elements), $supported_keys))));
     }
     // Determine whether to do auto-placeholdering.
     if ($this->placeholderGenerator->canCreatePlaceholder($elements) && $this->placeholderGenerator->shouldAutomaticallyPlaceholder($elements)) {
@@ -339,9 +395,11 @@ class Renderer implements RendererInterface {
     }
     // If instructed to create a placeholder, and a #lazy_builder callback is
     // present (without such a callback, it would be impossible to replace the
-    // placeholder), replace the current element with a placeholder.
-    // @todo remove the isMethodCacheable() check when
-    //       https://www.drupal.org/node/2367555 lands.
+    // placeholder), replace the current element with a placeholder. On
+    // uncacheable requests, always skip placeholdering - if a form is inside
+    // a placeholder, which is likely, we want to render it as soon as possible,
+    // so that form submission and redirection can take over before any more
+    // content is rendered.
     if (isset($elements['#create_placeholder']) && $elements['#create_placeholder'] === TRUE && $this->requestStack->getCurrentRequest()->isMethodCacheable()) {
       if (!isset($elements['#lazy_builder'])) {
         throw new \LogicException('When #create_placeholder is set, a #lazy_builder callback must be present as well.');
@@ -350,12 +408,11 @@ class Renderer implements RendererInterface {
     }
     // Build the element if it is still empty.
     if (isset($elements['#lazy_builder'])) {
-      $callable = $elements['#lazy_builder'][0];
-      $args = $elements['#lazy_builder'][1];
-      if (is_string($callable) && strpos($callable, '::') === FALSE) {
-        $callable = $this->controllerResolver->getControllerFromDefinition($callable);
-      }
-      $new_elements = call_user_func_array($callable, $args);
+      $new_elements = $this->doCallback('#lazy_builder', $elements['#lazy_builder'][0], $elements['#lazy_builder'][1]);
+      // Throw an exception if #lazy_builder callback does not return an array;
+      // provide helpful details for troubleshooting.
+      assert(is_array($new_elements), "#lazy_builder callbacks must return a valid renderable array, got " . gettype($new_elements) . " from " . Variable::callableToString($elements['#lazy_builder'][0]));
+
       // Retain the original cacheability metadata, plus cache keys.
       CacheableMetadata::createFromRenderArray($elements)
         ->merge(CacheableMetadata::createFromRenderArray($new_elements))
@@ -372,22 +429,19 @@ class Renderer implements RendererInterface {
     // element is rendered into the final text.
     if (isset($elements['#pre_render'])) {
       foreach ($elements['#pre_render'] as $callable) {
-        if (is_string($callable) && strpos($callable, '::') === FALSE) {
-          $callable = $this->controllerResolver->getControllerFromDefinition($callable);
-        }
-        $elements = call_user_func($callable, $elements);
+        $elements = $this->doCallback('#pre_render', $callable, [$elements]);
       }
     }
 
     // All render elements support #markup and #plain_text.
-    if (!empty($elements['#markup']) || !empty($elements['#plain_text'])) {
+    if (isset($elements['#markup']) || isset($elements['#plain_text'])) {
       $elements = $this->ensureMarkupIsSafe($elements);
     }
 
     // Defaults for bubbleable rendering metadata.
-    $elements['#cache']['tags'] = isset($elements['#cache']['tags']) ? $elements['#cache']['tags'] : [];
-    $elements['#cache']['max-age'] = isset($elements['#cache']['max-age']) ? $elements['#cache']['max-age'] : Cache::PERMANENT;
-    $elements['#attached'] = isset($elements['#attached']) ? $elements['#attached'] : [];
+    $elements['#cache']['tags'] = $elements['#cache']['tags'] ?? [];
+    $elements['#cache']['max-age'] = $elements['#cache']['max-age'] ?? Cache::PERMANENT;
+    $elements['#attached'] = $elements['#attached'] ?? [];
 
     // Allow #pre_render to abort rendering.
     if (!empty($elements['#printed'])) {
@@ -401,7 +455,7 @@ class Renderer implements RendererInterface {
 
     // Add any JavaScript state information associated with the element.
     if (!empty($elements['#states'])) {
-      drupal_process_states($elements);
+      FormHelper::processStates($elements);
     }
 
     // Get the children of the element, sorted by weight.
@@ -499,21 +553,24 @@ class Renderer implements RendererInterface {
     // outputted text to be filtered.
     if (isset($elements['#post_render'])) {
       foreach ($elements['#post_render'] as $callable) {
-        if (is_string($callable) && strpos($callable, '::') === FALSE) {
-          $callable = $this->controllerResolver->getControllerFromDefinition($callable);
-        }
-        $elements['#children'] = call_user_func($callable, $elements['#children'], $elements);
+        $elements['#children'] = $this->doCallback('#post_render', $callable, [$elements['#children'], $elements]);
       }
     }
 
     // We store the resulting output in $elements['#markup'], to be consistent
     // with how render cached output gets stored. This ensures that placeholder
     // replacement logic gets the same data to work with, no matter if #cache is
-    // disabled, #cache is enabled, there is a cache hit or miss.
-    $prefix = isset($elements['#prefix']) ? $this->xssFilterAdminIfUnsafe($elements['#prefix']) : '';
-    $suffix = isset($elements['#suffix']) ? $this->xssFilterAdminIfUnsafe($elements['#suffix']) : '';
-
-    $elements['#markup'] = Markup::create($prefix . $elements['#children'] . $suffix);
+    // disabled, #cache is enabled, there is a cache hit or miss. If
+    // #render_children is set the #prefix and #suffix will have already been
+    // added.
+    if (isset($elements['#render_children'])) {
+      $elements['#markup'] = Markup::create($elements['#children']);
+    }
+    else {
+      $prefix = isset($elements['#prefix']) ? $this->xssFilterAdminIfUnsafe($elements['#prefix']) : '';
+      $suffix = isset($elements['#suffix']) ? $this->xssFilterAdminIfUnsafe($elements['#suffix']) : '';
+      $elements['#markup'] = Markup::create($prefix . $elements['#children'] . $suffix);
+    }
 
     // We've rendered this element (and its subtree!), now update the context.
     $context->update($elements);
@@ -525,6 +582,11 @@ class Renderer implements RendererInterface {
         throw new \LogicException('Cache keys may not be changed after initial setup. Use the contexts property instead to bubble additional metadata.');
       }
       $this->renderCache->set($elements, $pre_bubbling_elements);
+      // Add debug output to the renderable array on cache miss.
+      if ($this->rendererConfig['debug'] === TRUE) {
+        $render_stop = microtime(TRUE);
+        $elements = $this->addDebugOutput($elements, FALSE, $pre_bubbling_elements, $render_stop - $render_start);
+      }
       // Update the render context; the render cache implementation may update
       // the element, and it may have different bubbleable metadata now.
       // @see \Drupal\Core\Render\PlaceholderingRenderCache::set()
@@ -546,7 +608,7 @@ class Renderer implements RendererInterface {
       $this->replacePlaceholders($elements);
       // @todo remove as part of https://www.drupal.org/node/2511330.
       if ($context->count() !== 1) {
-        throw new \LogicException('A stray drupal_render() invocation with $is_root_call = TRUE is causing bubbling of attached assets to break.');
+        throw new \LogicException('A stray RendererInterface::render() invocation with $is_root_call = TRUE is causing bubbling of attached assets to break.');
       }
     }
 
@@ -574,10 +636,7 @@ class Renderer implements RendererInterface {
     // Set the provided context and call the callable, it will use that context.
     $this->setCurrentRenderContext($context);
     $result = $callable();
-    // @todo Convert to an assertion in https://www.drupal.org/node/2408013
-    if ($context->count() > 1) {
-      throw new \LogicException('Bubbling failed.');
-    }
+    assert($context->count() <= 1, 'Bubbling failed.');
 
     // Restore the original render context.
     $this->setCurrentRenderContext($previous_context);
@@ -588,12 +647,12 @@ class Renderer implements RendererInterface {
   /**
    * Returns the current render context.
    *
-   * @return \Drupal\Core\Render\RenderContext
+   * @return \Drupal\Core\Render\RenderContext|null
    *   The current render context.
    */
   protected function getCurrentRenderContext() {
     $request = $this->requestStack->getCurrentRequest();
-    return isset(static::$contextCollection[$request]) ? static::$contextCollection[$request] : NULL;
+    return static::$contextCollection[$request] ?? NULL;
   }
 
   /**
@@ -605,7 +664,7 @@ class Renderer implements RendererInterface {
    *
    * @return $this
    */
-  protected function setCurrentRenderContext(RenderContext $context = NULL) {
+  protected function setCurrentRenderContext(?RenderContext $context = NULL) {
     $request = $this->requestStack->getCurrentRequest();
     static::$contextCollection[$request] = $context;
     return $this;
@@ -628,7 +687,7 @@ class Renderer implements RendererInterface {
    *   bubbleable metadata associated with the markup that replaced the
    *   placeholders.
    *
-   * @returns bool
+   * @return bool
    *   Whether placeholders were replaced.
    *
    * @see \Drupal\Core\Render\Renderer::renderPlaceholder()
@@ -643,8 +702,8 @@ class Renderer implements RendererInterface {
     // being rendered: any code can add messages to render.
     // This violates the principle that each lazy builder must be able to render
     // itself in isolation, and therefore in any order. However, we cannot
-    // change the way drupal_set_message() works in the Drupal 8 cycle. So we
-    // have to accommodate its special needs.
+    // change the way \Drupal\Core\Messenger\Messenger works in the Drupal 8
+    // cycle. So we have to accommodate its special needs.
     // Allowing placeholders to be rendered in a particular order (in this case:
     // last) would violate this isolation principle. Thus a monopoly is granted
     // to this one special case, with this hard-coded solution.
@@ -653,14 +712,48 @@ class Renderer implements RendererInterface {
 
     // First render all placeholders except 'status messages' placeholders.
     $message_placeholders = [];
+    $fibers = [];
     foreach ($elements['#attached']['placeholders'] as $placeholder => $placeholder_element) {
       if (isset($placeholder_element['#lazy_builder']) && $placeholder_element['#lazy_builder'][0] === 'Drupal\Core\Render\Element\StatusMessages::renderMessages') {
         $message_placeholders[] = $placeholder;
       }
       else {
-        $elements = $this->renderPlaceholder($placeholder, $elements);
+        // Get the render array for the given placeholder
+        $fibers[$placeholder] = new \Fiber(function () use ($placeholder_element) {
+          return [$this->doRenderPlaceholder($placeholder_element), $placeholder_element];
+        });
       }
     }
+    $iterations = 0;
+    while (count($fibers) > 0) {
+      foreach ($fibers as $placeholder => $fiber) {
+        if (!$fiber->isStarted()) {
+          $fiber->start();
+        }
+        elseif ($fiber->isSuspended()) {
+          $fiber->resume();
+        }
+        // If the Fiber hasn't terminated by this point, move onto the next
+        // placeholder, we'll resume this fiber again when we get back here.
+        if (!$fiber->isTerminated()) {
+          // If we've gone through the placeholders once already, and they're
+          // still not finished, then start to allow code higher up the stack to
+          // get on with something else.
+          if ($iterations) {
+            $fiber = \Fiber::getCurrent();
+            if ($fiber !== NULL) {
+              $fiber->suspend();
+            }
+          }
+          continue;
+        }
+        [$markup, $placeholder_element] = $fiber->getReturn();
+
+        $elements = $this->doReplacePlaceholder($placeholder, $markup, $elements, $placeholder_element);
+        unset($fibers[$placeholder]);
+      }
+      $iterations++;
+    }
 
     // Then render 'status messages' placeholders.
     foreach ($message_placeholders as $message_placeholder) {
@@ -728,30 +821,116 @@ class Renderer implements RendererInterface {
    * @param array $elements
    *   A render array with #markup set.
    *
-   * @return \Drupal\Component\Render\MarkupInterface|string
-   *   The escaped markup wrapped in a Markup object. If $elements['#markup']
-   *   is an instance of \Drupal\Component\Render\MarkupInterface, it won't be
-   *   escaped or filtered again.
+   * @return array
+   *   The given array with the escaped markup wrapped in a Markup object.
+   *   If $elements['#markup'] is an instance of
+   *   \Drupal\Component\Render\MarkupInterface, it won't be escaped or filtered
+   *   again.
    *
    * @see \Drupal\Component\Utility\Html::escape()
    * @see \Drupal\Component\Utility\Xss::filter()
    * @see \Drupal\Component\Utility\Xss::filterAdmin()
    */
   protected function ensureMarkupIsSafe(array $elements) {
-    if (empty($elements['#markup']) && empty($elements['#plain_text'])) {
-      return $elements;
-    }
-
-    if (!empty($elements['#plain_text'])) {
+    if (isset($elements['#plain_text'])) {
       $elements['#markup'] = Markup::create(Html::escape($elements['#plain_text']));
     }
     elseif (!($elements['#markup'] instanceof MarkupInterface)) {
-      // The default behaviour is to XSS filter using the admin tag list.
-      $tags = isset($elements['#allowed_tags']) ? $elements['#allowed_tags'] : Xss::getAdminTagList();
+      // The default behavior is to XSS filter using the admin tag list.
+      $tags = $elements['#allowed_tags'] ?? Xss::getAdminTagList();
       $elements['#markup'] = Markup::create(Xss::filter($elements['#markup'], $tags));
     }
 
     return $elements;
   }
 
+  /**
+   * Performs a callback.
+   *
+   * @param string $callback_type
+   *   The type of the callback. For example, '#post_render'.
+   * @param string|callable $callback
+   *   The callback to perform.
+   * @param array $args
+   *   The arguments to pass to the callback.
+   *
+   * @return mixed
+   *   The callback's return value.
+   *
+   * @see \Drupal\Core\Security\TrustedCallbackInterface
+   */
+  protected function doCallback($callback_type, $callback, array $args) {
+    $callable = $this->callableResolver->getCallableFromDefinition($callback);
+    $message = sprintf('Render %s callbacks must be methods of a class that implements \Drupal\Core\Security\TrustedCallbackInterface or be an anonymous function. The callback was %s. See https://www.drupal.org/node/2966725', $callback_type, '%s');
+    // Add \Drupal\Core\Render\Element\RenderCallbackInterface as an extra
+    // trusted interface so that:
+    // - All public methods on Render elements are considered trusted.
+    // - Helper classes that contain only callback methods can implement this
+    //   instead of TrustedCallbackInterface.
+    return $this->doTrustedCallback($callable, $args, $message, TrustedCallbackInterface::THROW_EXCEPTION, RenderCallbackInterface::class);
+  }
+
+  /**
+   * Add cache debug information to the render array.
+   *
+   * @param array $elements
+   *   The renderable array that must be wrapped with the cache debug output.
+   * @param bool $is_cache_hit
+   *   A flag indicating that the cache is hit or miss.
+   * @param array $pre_bubbling_elements
+   *   The renderable array for pre-bubbling elements.
+   * @param float $render_time
+   *   The rendering time.
+   *
+   * @return array
+   *   The renderable array.
+   */
+  protected function addDebugOutput(array $elements, bool $is_cache_hit, array $pre_bubbling_elements = [], float $render_time = 0) {
+    if (empty($elements['#markup'])) {
+      return $elements;
+    }
+
+    $debug_items = [
+      'CACHE' => &$elements,
+      'PRE-BUBBLING CACHE' => &$pre_bubbling_elements,
+    ];
+    $prefix = "<!-- START RENDERER -->";
+    $prefix .= "\n<!-- CACHE-HIT: " . ($is_cache_hit ? 'Yes' : 'No') . " -->";
+    foreach ($debug_items as $name_prefix => $debug_item) {
+      if (!empty($debug_item['#cache']['tags'])) {
+        $prefix .= "\n<!-- " . $name_prefix . " TAGS:";
+        foreach ($debug_item['#cache']['tags'] as $tag) {
+          $prefix .= "\n   * " . $tag;
+        }
+        $prefix .= "\n-->";
+      }
+      if (!empty($debug_item['#cache']['contexts'])) {
+        $prefix .= "\n<!-- " . $name_prefix . " CONTEXTS:";
+        foreach ($debug_item['#cache']['contexts'] as $context) {
+          $prefix .= "\n   * " . $context;
+        }
+        $prefix .= "\n-->";
+      }
+      if (!empty($debug_item['#cache']['keys'])) {
+        $prefix .= "\n<!-- " . $name_prefix . " KEYS:";
+        foreach ($debug_item['#cache']['keys'] as $key) {
+          $prefix .= "\n   * " . $key;
+        }
+        $prefix .= "\n-->";
+      }
+      if (!empty($debug_item['#cache']['max-age'])) {
+        $prefix .= "\n<!-- " . $name_prefix . " MAX-AGE: " . $debug_item['#cache']['max-age'] . " -->";
+      }
+    }
+
+    if (!empty($render_time)) {
+      $prefix .= "\n<!-- RENDERING TIME: " . number_format($render_time, 9) . " -->";
+    }
+    $suffix = "<!-- END RENDERER -->";
+
+    $elements['#markup'] = Markup::create("$prefix\n" . $elements['#markup'] . "\n$suffix");
+
+    return $elements;
+  }
+
 }
