@@ -1,8 +1,9 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\book\Functional;
 
-use Drupal\Component\Render\FormattableMarkup;
 use Drupal\Core\Cache\Cache;
 use Drupal\Tests\BrowserTestBase;
 use Drupal\user\RoleInterface;
@@ -11,6 +12,8 @@ use Drupal\user\RoleInterface;
  * Create a book, add pages, and test book interface.
  *
  * @group book
+ * @group legacy
+ * @group #slow
  */
 class BookTest extends BrowserTestBase {
 
@@ -32,19 +35,19 @@ class BookTest extends BrowserTestBase {
   /**
    * {@inheritdoc}
    */
-  protected $defaultTheme = 'classy';
+  protected $defaultTheme = 'stark';
 
   /**
    * A user with permission to view a book and access printer-friendly version.
    *
-   * @var object
+   * @var \Drupal\user\UserInterface
    */
   protected $webUser;
 
   /**
    * A user with permission to create and edit books and to administer blocks.
    *
-   * @var object
+   * @var \Drupal\user\UserInterface
    */
   protected $adminUser;
 
@@ -102,7 +105,7 @@ class BookTest extends BrowserTestBase {
    *
    * @see \Drupal\book\Cache\BookNavigationCacheContext
    */
-  public function testBookNavigationCacheContext() {
+  public function testBookNavigationCacheContext(): void {
     // Create a page node.
     $this->drupalCreateContentType(['type' => 'page']);
     $page = $this->drupalCreateNode();
@@ -118,29 +121,29 @@ class BookTest extends BrowserTestBase {
 
     // On non-node route.
     $this->drupalGet($this->adminUser->toUrl());
-    $this->assertRaw('[route.book_navigation]=book.none');
+    $this->assertSession()->responseContains('[route.book_navigation]=book.none');
 
     // On non-book node route.
     $this->drupalGet($page->toUrl());
-    $this->assertRaw('[route.book_navigation]=book.none');
+    $this->assertSession()->responseContains('[route.book_navigation]=book.none');
 
     // On book node route.
     $this->drupalGet($book_nodes[0]->toUrl());
-    $this->assertRaw('[route.book_navigation]=0|2|3');
+    $this->assertSession()->responseContains('[route.book_navigation]=0|2|3');
     $this->drupalGet($book_nodes[1]->toUrl());
-    $this->assertRaw('[route.book_navigation]=0|2|3|4');
+    $this->assertSession()->responseContains('[route.book_navigation]=0|2|3|4');
     $this->drupalGet($book_nodes[2]->toUrl());
-    $this->assertRaw('[route.book_navigation]=0|2|3|5');
+    $this->assertSession()->responseContains('[route.book_navigation]=0|2|3|5');
     $this->drupalGet($book_nodes[3]->toUrl());
-    $this->assertRaw('[route.book_navigation]=0|2|6');
+    $this->assertSession()->responseContains('[route.book_navigation]=0|2|6');
     $this->drupalGet($book_nodes[4]->toUrl());
-    $this->assertRaw('[route.book_navigation]=0|2|7');
+    $this->assertSession()->responseContains('[route.book_navigation]=0|2|7');
   }
 
   /**
    * Tests saving the book outline on an empty book.
    */
-  public function testEmptyBook() {
+  public function testEmptyBook(): void {
     // Create a new empty book.
     $this->drupalLogin($this->bookAuthor);
     $book = $this->createBookNode('new');
@@ -148,14 +151,15 @@ class BookTest extends BrowserTestBase {
 
     // Log in as a user with access to the book outline and save the form.
     $this->drupalLogin($this->adminUser);
-    $this->drupalPostForm('admin/structure/book/' . $book->id(), [], t('Save book pages'));
-    $this->assertText(t('Updated book @book.', ['@book' => $book->label()]));
+    $this->drupalGet('admin/structure/book/' . $book->id());
+    $this->submitForm([], 'Save book pages');
+    $this->assertSession()->pageTextContains('Updated book ' . $book->label() . '.');
   }
 
   /**
    * Tests book functionality through node interfaces.
    */
-  public function testBook() {
+  public function testBook(): void {
     // Create new book.
     $nodes = $this->createBook();
     $book = $this->book;
@@ -176,7 +180,7 @@ class BookTest extends BrowserTestBase {
 
     // Check the presence of expected cache tags.
     $this->drupalGet('node/add/book');
-    $this->assertCacheTag('config:book.settings');
+    $this->assertSession()->responseHeaderContains('X-Drupal-Cache-Tags', 'config:book.settings');
 
     /*
      * Add Node 5 under Node 3.
@@ -201,7 +205,8 @@ class BookTest extends BrowserTestBase {
     $other_book = $this->createBookNode('new');
     $node = $this->createBookNode($book->id());
     $edit = ['book[bid]' => $other_book->id()];
-    $this->drupalPostForm('node/' . $node->id() . '/edit', $edit, t('Save'));
+    $this->drupalGet('node/' . $node->id() . '/edit');
+    $this->submitForm($edit, 'Save');
 
     $this->drupalLogout();
     $this->drupalLogin($this->webUser);
@@ -217,31 +222,39 @@ class BookTest extends BrowserTestBase {
     $this->drupalLogin($this->bookAuthor);
     $book = $this->createBookNode('new');
     $book->save();
+
+    // Confirm that an unpublished book page has the 'Add child page' link.
+    $this->drupalGet('node/' . $nodes[4]->id());
+    $this->assertSession()->linkExists('Add child page');
+    $nodes[4]->setUnPublished();
+    $nodes[4]->save();
+    $this->drupalGet('node/' . $nodes[4]->id());
+    $this->assertSession()->linkExists('Add child page');
   }
 
   /**
    * Tests book export ("printer-friendly version") functionality.
    */
-  public function testBookExport() {
+  public function testBookExport(): void {
     // Create a book.
     $nodes = $this->createBook();
 
     // Log in as web user and view printer-friendly version.
     $this->drupalLogin($this->webUser);
     $this->drupalGet('node/' . $this->book->id());
-    $this->clickLink(t('Printer-friendly version'));
+    $this->clickLink('Printer-friendly version');
 
     // Make sure each part of the book is there.
     foreach ($nodes as $node) {
-      $this->assertText($node->label(), 'Node title found in printer friendly version.');
-      $this->assertRaw($node->body->processed, 'Node body found in printer friendly version.');
+      $this->assertSession()->pageTextContains($node->label());
+      $this->assertSession()->responseContains($node->body->processed);
     }
 
     // Make sure we can't export an unsupported format.
     $this->drupalGet('book/export/foobar/' . $this->book->id());
     $this->assertSession()->statusCodeEquals(404);
 
-    // Make sure we get a 404 on a not existing book node.
+    // Make sure we get a 404 on a non-existent book node.
     $this->drupalGet('book/export/html/123');
     $this->assertSession()->statusCodeEquals(404);
 
@@ -250,7 +263,7 @@ class BookTest extends BrowserTestBase {
 
     // Load the book and verify there is no printer-friendly version link.
     $this->drupalGet('node/' . $this->book->id());
-    $this->assertSession()->linkNotExists(t('Printer-friendly version'), 'Anonymous user is not shown link to printer-friendly version.');
+    $this->assertSession()->linkNotExists('Printer-friendly version', 'Anonymous user is not shown link to printer-friendly version.');
 
     // Try getting the URL directly, and verify it fails.
     $this->drupalGet('book/export/html/' . $this->book->id());
@@ -267,7 +280,7 @@ class BookTest extends BrowserTestBase {
   /**
    * Tests the functionality of the book navigation block.
    */
-  public function testBookNavigationBlock() {
+  public function testBookNavigationBlock(): void {
     $this->drupalLogin($this->adminUser);
 
     // Enable the block.
@@ -276,21 +289,50 @@ class BookTest extends BrowserTestBase {
     // Give anonymous users the permission 'node test view'.
     $edit = [];
     $edit[RoleInterface::ANONYMOUS_ID . '[node test view]'] = TRUE;
-    $this->drupalPostForm('admin/people/permissions/' . RoleInterface::ANONYMOUS_ID, $edit, t('Save permissions'));
-    $this->assertText(t('The changes have been saved.'), "Permission 'node test view' successfully assigned to anonymous users.");
+    $this->drupalGet('admin/people/permissions/' . RoleInterface::ANONYMOUS_ID);
+    $this->submitForm($edit, 'Save permissions');
+    $this->assertSession()->pageTextContains('The changes have been saved.');
 
     // Test correct display of the block.
     $nodes = $this->createBook();
     $this->drupalGet('<front>');
-    $this->assertText($block->label(), 'Book navigation block is displayed.');
-    $this->assertText($this->book->label(), new FormattableMarkup('Link to book root (@title) is displayed.', ['@title' => $nodes[0]->label()]));
-    $this->assertNoText($nodes[0]->label(), 'No links to individual book pages are displayed.');
+    // Book navigation block.
+    $this->assertSession()->pageTextContains($block->label());
+    // Link to book root.
+    $this->assertSession()->pageTextContains($this->book->label());
+    // No links to individual book pages.
+    $this->assertSession()->pageTextNotContains($nodes[0]->label());
+
+    // Ensure that an unpublished node does not appear in the navigation for a
+    // user without access. By unpublishing a parent page, child pages should
+    // not appear in the navigation. The node_access_test module is disabled
+    // since it interferes with this logic.
+
+    /** @var \Drupal\Core\Extension\ModuleInstaller $installer */
+    $installer = \Drupal::service('module_installer');
+    $installer->uninstall(['node_access_test']);
+    node_access_rebuild();
+
+    $nodes[0]->setUnPublished();
+    $nodes[0]->save();
+
+    // Verify the user does not have access to the unpublished node.
+    $this->assertFalse($nodes[0]->access('view', $this->webUser));
+
+    // Verify the unpublished book page does not appear in the navigation.
+    $this->drupalLogin($this->webUser);
+    $this->drupalGet($nodes[0]->toUrl());
+    $this->assertSession()->statusCodeEquals(403);
+    $this->drupalGet($this->book->toUrl());
+    $this->assertSession()->responseNotContains($nodes[0]->getTitle());
+    $this->assertSession()->responseNotContains($nodes[1]->getTitle());
+    $this->assertSession()->responseNotContains($nodes[2]->getTitle());
   }
 
   /**
    * Tests BookManager::getTableOfContents().
    */
-  public function testGetTableOfContents() {
+  public function testGetTableOfContents(): void {
     // Create new book.
     $nodes = $this->createBook();
     $book = $this->book;
@@ -320,39 +362,38 @@ class BookTest extends BrowserTestBase {
     $this->drupalGet($nodes[0]->toUrl('edit-form'));
     // Since Node 0 has children 2 levels deep, nodes 10 and 11 should not
     // appear in the selector.
-    $this->assertNoOption('edit-book-pid', $nodes[10]->id());
-    $this->assertNoOption('edit-book-pid', $nodes[11]->id());
+    $this->assertSession()->optionNotExists('edit-book-pid', $nodes[10]->id());
+    $this->assertSession()->optionNotExists('edit-book-pid', $nodes[11]->id());
     // Node 9 should be available as an option.
-    $this->assertOption('edit-book-pid', $nodes[9]->id());
+    $this->assertSession()->optionExists('edit-book-pid', $nodes[9]->id());
 
     // Get a shallow set of options.
     /** @var \Drupal\book\BookManagerInterface $manager */
     $manager = $this->container->get('book.manager');
     $options = $manager->getTableOfContents($book->id(), 3);
+    // Verify that all expected option keys are present.
     $expected_nids = [$book->id(), $nodes[0]->id(), $nodes[1]->id(), $nodes[2]->id(), $nodes[3]->id(), $nodes[6]->id(), $nodes[4]->id()];
-    $this->assertEqual(count($options), count($expected_nids));
-    $diff = array_diff($expected_nids, array_keys($options));
-    $this->assertTrue(empty($diff), 'Found all expected option keys');
+    $this->assertEquals($expected_nids, array_keys($options));
     // Exclude Node 3.
     $options = $manager->getTableOfContents($book->id(), 3, [$nodes[3]->id()]);
+    // Verify that expected option keys are present after excluding Node 3.
     $expected_nids = [$book->id(), $nodes[0]->id(), $nodes[1]->id(), $nodes[2]->id(), $nodes[4]->id()];
-    $this->assertEqual(count($options), count($expected_nids));
-    $diff = array_diff($expected_nids, array_keys($options));
-    $this->assertTrue(empty($diff), 'Found all expected option keys after excluding Node 3');
+    $this->assertEquals($expected_nids, array_keys($options));
   }
 
   /**
    * Tests the book navigation block when an access module is installed.
    */
-  public function testNavigationBlockOnAccessModuleInstalled() {
+  public function testNavigationBlockOnAccessModuleInstalled(): void {
     $this->drupalLogin($this->adminUser);
     $block = $this->drupalPlaceBlock('book_navigation', ['block_mode' => 'book pages']);
 
     // Give anonymous users the permission 'node test view'.
     $edit = [];
     $edit[RoleInterface::ANONYMOUS_ID . '[node test view]'] = TRUE;
-    $this->drupalPostForm('admin/people/permissions/' . RoleInterface::ANONYMOUS_ID, $edit, t('Save permissions'));
-    $this->assertText(t('The changes have been saved.'), "Permission 'node test view' successfully assigned to anonymous users.");
+    $this->drupalGet('admin/people/permissions/' . RoleInterface::ANONYMOUS_ID);
+    $this->submitForm($edit, 'Save permissions');
+    $this->assertSession()->pageTextContains('The changes have been saved.');
 
     // Create a book.
     $this->createBook();
@@ -360,22 +401,22 @@ class BookTest extends BrowserTestBase {
     // Test correct display of the block to registered users.
     $this->drupalLogin($this->webUser);
     $this->drupalGet('node/' . $this->book->id());
-    $this->assertText($block->label(), 'Book navigation block is displayed to registered users.');
+    $this->assertSession()->pageTextContains($block->label());
     $this->drupalLogout();
 
     // Test correct display of the block to anonymous users.
     $this->drupalGet('node/' . $this->book->id());
-    $this->assertText($block->label(), 'Book navigation block is displayed to anonymous users.');
+    $this->assertSession()->pageTextContains($block->label());
 
     // Test the 'book pages' block_mode setting.
     $this->drupalGet('<front>');
-    $this->assertNoText($block->label(), 'Book navigation block is not shown on non-book pages.');
+    $this->assertSession()->pageTextNotContains($block->label());
   }
 
   /**
    * Tests the access for deleting top-level book nodes.
    */
-  public function testBookDelete() {
+  public function testBookDelete(): void {
     $node_storage = $this->container->get('entity_type.manager')->getStorage('node');
     $nodes = $this->createBook();
     $this->drupalLogin($this->adminUser);
@@ -386,10 +427,11 @@ class BookTest extends BrowserTestBase {
     $this->assertSession()->statusCodeEquals(403);
 
     // Ensure that a child book node can be deleted.
-    $this->drupalPostForm('node/' . $nodes[4]->id() . '/outline/remove', $edit, t('Remove'));
+    $this->drupalGet('node/' . $nodes[4]->id() . '/outline/remove');
+    $this->submitForm($edit, 'Remove');
     $node_storage->resetCache([$nodes[4]->id()]);
     $node4 = $node_storage->load($nodes[4]->id());
-    $this->assertTrue(empty($node4->book), 'Deleting child book node properly allowed.');
+    $this->assertEmpty($node4->book, 'Deleting child book node properly allowed.');
 
     // $nodes[4] is stale, trying to delete it directly will cause an error.
     $node4->delete();
@@ -398,58 +440,62 @@ class BookTest extends BrowserTestBase {
     // Delete all child book nodes and retest top-level node deletion.
     $node_storage->delete($nodes);
 
-    $this->drupalPostForm('node/' . $this->book->id() . '/outline/remove', $edit, t('Remove'));
+    $this->drupalGet('node/' . $this->book->id() . '/outline/remove');
+    $this->submitForm($edit, 'Remove');
     $node_storage->resetCache([$this->book->id()]);
     $node = $node_storage->load($this->book->id());
-    $this->assertTrue(empty($node->book), 'Deleting childless top-level book node properly allowed.');
+    $this->assertEmpty($node->book, 'Deleting childless top-level book node properly allowed.');
 
     // Tests directly deleting a book parent.
     $nodes = $this->createBook();
     $this->drupalLogin($this->adminUser);
     $this->drupalGet($this->book->toUrl('delete-form'));
-    $this->assertRaw(t('%title is part of a book outline, and has associated child pages. If you proceed with deletion, the child pages will be relocated automatically.', ['%title' => $this->book->label()]));
+    $this->assertSession()->pageTextContains($this->book->label() . ' is part of a book outline, and has associated child pages. If you proceed with deletion, the child pages will be relocated automatically.');
     // Delete parent, and visit a child page.
-    $this->drupalPostForm($this->book->toUrl('delete-form'), [], t('Delete'));
+    $this->drupalGet($this->book->toUrl('delete-form'));
+    $this->submitForm([], 'Delete');
     $this->drupalGet($nodes[0]->toUrl());
     $this->assertSession()->statusCodeEquals(200);
-    $this->assertText($nodes[0]->label());
+    $this->assertSession()->pageTextContains($nodes[0]->label());
     // The book parents should be updated.
     $node_storage = \Drupal::entityTypeManager()->getStorage('node');
     $node_storage->resetCache();
     $child = $node_storage->load($nodes[0]->id());
-    $this->assertEqual($child->id(), $child->book['bid'], 'Child node book ID updated when parent is deleted.');
+    $this->assertEquals($child->id(), $child->book['bid'], 'Child node book ID updated when parent is deleted.');
     // 3rd-level children should now be 2nd-level.
     $second = $node_storage->load($nodes[1]->id());
-    $this->assertEqual($child->id(), $second->book['bid'], '3rd-level child node is now second level when top-level node is deleted.');
+    $this->assertEquals($child->id(), $second->book['bid'], '3rd-level child node is now second level when top-level node is deleted.');
   }
 
   /**
    * Tests outline of a book.
    */
-  public function testBookOutline() {
+  public function testBookOutline(): void {
     $this->drupalLogin($this->bookAuthor);
 
     // Create new node not yet a book.
     $empty_book = $this->drupalCreateNode(['type' => 'book']);
     $this->drupalGet('node/' . $empty_book->id() . '/outline');
-    $this->assertSession()->linkNotExists(t('Book outline'), 'Book Author is not allowed to outline');
+    $this->assertSession()->linkNotExists('Book outline', 'Book Author is not allowed to outline');
 
     $this->drupalLogin($this->adminUser);
     $this->drupalGet('node/' . $empty_book->id() . '/outline');
-    $this->assertRaw(t('Book outline'));
-    $this->assertOptionSelected('edit-book-bid', 0, 'Node does not belong to a book');
-    $this->assertSession()->linkNotExists(t('Remove from book outline'));
+    $this->assertSession()->pageTextContains('Book outline');
+    // Verify that the node does not belong to a book.
+    $this->assertTrue($this->assertSession()->optionExists('edit-book-bid', 0)->isSelected());
+    $this->assertSession()->linkNotExists('Remove from book outline');
 
     $edit = [];
     $edit['book[bid]'] = '1';
-    $this->drupalPostForm('node/' . $empty_book->id() . '/outline', $edit, t('Add to book outline'));
+    $this->drupalGet('node/' . $empty_book->id() . '/outline');
+    $this->submitForm($edit, 'Add to book outline');
     $node = \Drupal::entityTypeManager()->getStorage('node')->load($empty_book->id());
     // Test the book array.
-    $this->assertEqual($node->book['nid'], $empty_book->id());
-    $this->assertEqual($node->book['bid'], $empty_book->id());
-    $this->assertEqual($node->book['depth'], 1);
-    $this->assertEqual($node->book['p1'], $empty_book->id());
-    $this->assertEqual($node->book['pid'], '0');
+    $this->assertEquals($empty_book->id(), $node->book['nid']);
+    $this->assertEquals($empty_book->id(), $node->book['bid']);
+    $this->assertEquals(1, $node->book['depth']);
+    $this->assertEquals($empty_book->id(), $node->book['p1']);
+    $this->assertEquals('0', $node->book['pid']);
 
     // Create new book.
     $this->drupalLogin($this->bookAuthor);
@@ -457,53 +503,54 @@ class BookTest extends BrowserTestBase {
 
     $this->drupalLogin($this->adminUser);
     $this->drupalGet('node/' . $book->id() . '/outline');
-    $this->assertRaw(t('Book outline'));
-    $this->clickLink(t('Remove from book outline'));
-    $this->assertRaw(t('Are you sure you want to remove %title from the book hierarchy?', ['%title' => $book->label()]));
+    $this->assertSession()->pageTextContains('Book outline');
+    $this->clickLink('Remove from book outline');
+    $this->assertSession()->pageTextContains('Are you sure you want to remove ' . $book->label() . ' from the book hierarchy?');
 
     // Create a new node and set the book after the node was created.
     $node = $this->drupalCreateNode(['type' => 'book']);
     $edit = [];
     $edit['book[bid]'] = $node->id();
-    $this->drupalPostForm('node/' . $node->id() . '/edit', $edit, t('Save'));
+    $this->drupalGet('node/' . $node->id() . '/edit');
+    $this->submitForm($edit, 'Save');
     $node = \Drupal::entityTypeManager()->getStorage('node')->load($node->id());
 
     // Test the book array.
-    $this->assertEqual($node->book['nid'], $node->id());
-    $this->assertEqual($node->book['bid'], $node->id());
-    $this->assertEqual($node->book['depth'], 1);
-    $this->assertEqual($node->book['p1'], $node->id());
-    $this->assertEqual($node->book['pid'], '0');
+    $this->assertEquals($node->id(), $node->book['nid']);
+    $this->assertEquals($node->id(), $node->book['bid']);
+    $this->assertEquals(1, $node->book['depth']);
+    $this->assertEquals($node->id(), $node->book['p1']);
+    $this->assertEquals('0', $node->book['pid']);
 
     // Test the form itself.
     $this->drupalGet('node/' . $node->id() . '/edit');
-    $this->assertOptionSelected('edit-book-bid', $node->id());
+    $this->assertTrue($this->assertSession()->optionExists('edit-book-bid', $node->id())->isSelected());
   }
 
   /**
    * Tests that saveBookLink() returns something.
    */
-  public function testSaveBookLink() {
+  public function testSaveBookLink(): void {
     $book_manager = \Drupal::service('book.manager');
 
     // Mock a link for a new book.
-    $link = ['nid' => 1, 'has_children' => 0, 'original_bid' => 0, 'parent_depth_limit' => 8, 'pid' => 0, 'weight' => 0, 'bid' => 1];
+    $link = ['nid' => 1, 'has_children' => 0, 'original_bid' => 0, 'pid' => 0, 'weight' => 0, 'bid' => 0];
     $new = TRUE;
 
     // Save the link.
     $return = $book_manager->saveBookLink($link, $new);
 
     // Add the link defaults to $link so we have something to compare to the return from saveBookLink().
-    $link += $book_manager->getLinkDefaults($link['nid']);
+    $link = $book_manager->getLinkDefaults($link['nid']);
 
     // Test the return from saveBookLink.
-    $this->assertEqual($return, $link);
+    $this->assertEquals($return, $link);
   }
 
   /**
    * Tests the listing of all books.
    */
-  public function testBookListing() {
+  public function testBookListing(): void {
     // Uninstall 'node_access_test' as this interferes with the test.
     \Drupal::service('module_installer')->uninstall(['node_access_test']);
 
@@ -512,8 +559,7 @@ class BookTest extends BrowserTestBase {
 
     // Load the book page and assert the created book title is displayed.
     $this->drupalGet('book');
-
-    $this->assertText($this->book->label(), 'The book title is displayed on the book listing page.');
+    $this->assertSession()->pageTextContains($this->book->label());
 
     // Unpublish the top book page and confirm that the created book title is
     // not displayed for anonymous.
@@ -557,37 +603,60 @@ class BookTest extends BrowserTestBase {
   /**
    * Tests the administrative listing of all books.
    */
-  public function testAdminBookListing() {
+  public function testAdminBookListing(): void {
     // Create a new book.
-    $this->createBook();
+    $nodes = $this->createBook();
 
     // Load the book page and assert the created book title is displayed.
     $this->drupalLogin($this->adminUser);
     $this->drupalGet('admin/structure/book');
-    $this->assertText($this->book->label(), 'The book title is displayed on the administrative book listing page.');
+    $this->assertSession()->pageTextContains($this->book->label());
   }
 
   /**
    * Tests the administrative listing of all book pages in a book.
    */
-  public function testAdminBookNodeListing() {
+  public function testAdminBookNodeListing(): void {
     // Create a new book.
-    $this->createBook();
+    $nodes = $this->createBook();
     $this->drupalLogin($this->adminUser);
 
     // Load the book page list and assert the created book title is displayed
     // and action links are shown on list items.
     $this->drupalGet('admin/structure/book/' . $this->book->id());
-    $this->assertText($this->book->label(), 'The book title is displayed on the administrative book listing page.');
+    $this->assertSession()->pageTextContains($this->book->label());
 
-    $elements = $this->xpath('//table//ul[@class="dropbutton"]/li/a');
-    $this->assertEqual($elements[0]->getText(), 'View', 'View link is found from the list.');
+    // Test that the view link is found from the list.
+    $this->assertSession()->elementTextEquals('xpath', '//table//ul[@class="dropbutton"]/li/a', 'View');
+
+    // Test that all the book pages are displayed on the book outline page.
+    $this->assertSession()->elementsCount('xpath', '//table//ul[@class="dropbutton"]/li/a', count($nodes));
+
+    // Unpublish a book in the hierarchy.
+    $nodes[0]->setUnPublished();
+    $nodes[0]->save();
+
+    // Node should still appear on the outline for admins.
+    $this->drupalGet('admin/structure/book/' . $this->book->id());
+    $this->assertSession()->elementsCount('xpath', '//table//ul[@class="dropbutton"]/li/a', count($nodes));
+
+    // Saving a book page not as the current version shouldn't effect the book.
+    $old_title = $nodes[1]->getTitle();
+    $new_title = $this->getRandomGenerator()->name();
+    $nodes[1]->isDefaultRevision(FALSE);
+    $nodes[1]->setNewRevision(TRUE);
+    $nodes[1]->setTitle($new_title);
+    $nodes[1]->save();
+    $this->drupalGet('admin/structure/book/' . $this->book->id());
+    $this->assertSession()->elementsCount('xpath', '//table//ul[@class="dropbutton"]/li/a', count($nodes));
+    $this->assertSession()->responseNotContains($new_title);
+    $this->assertSession()->responseContains($old_title);
   }
 
   /**
    * Ensure the loaded book in hook_node_load() does not depend on the user.
    */
-  public function testHookNodeLoadAccess() {
+  public function testHookNodeLoadAccess(): void {
     \Drupal::service('module_installer')->install(['node_access_test']);
 
     // Ensure that the loaded book in hook_node_load() does NOT depend on the
@@ -603,16 +672,16 @@ class BookTest extends BrowserTestBase {
     // @see node_access_test_node_grants().
     $this->drupalLogin($this->webUserWithoutNodeAccess);
     $book_node = $node_storage->load($this->book->id());
-    $this->assertTrue(!empty($book_node->book));
-    $this->assertEqual($book_node->book['bid'], $this->book->id());
+    $this->assertNotEmpty($book_node->book);
+    $this->assertEquals($this->book->id(), $book_node->book['bid']);
 
     // Reset the internal cache to retrigger the hook_node_load() call.
     $node_storage->resetCache();
 
     $this->drupalLogin($this->webUser);
     $book_node = $node_storage->load($this->book->id());
-    $this->assertTrue(!empty($book_node->book));
-    $this->assertEqual($book_node->book['bid'], $this->book->id());
+    $this->assertNotEmpty($book_node->book);
+    $this->assertEquals($this->book->id(), $book_node->book['bid']);
   }
 
   /**
@@ -620,7 +689,7 @@ class BookTest extends BrowserTestBase {
    *
    * There was a fatal error with "Show block only on book pages" block mode.
    */
-  public function testBookNavigationBlockOnUnpublishedBook() {
+  public function testBookNavigationBlockOnUnpublishedBook(): void {
     // Create a new book.
     $this->createBook();
 
@@ -637,19 +706,23 @@ class BookTest extends BrowserTestBase {
 
     // Unpublish book node.
     $edit = ['status[value]' => FALSE];
-    $this->drupalPostForm('node/' . $this->book->id() . '/edit', $edit, t('Save'));
+    $this->drupalGet('node/' . $this->book->id() . '/edit');
+    $this->submitForm($edit, 'Save');
 
     // Test node page.
     $this->drupalGet('node/' . $this->book->id());
-    $this->assertText($this->book->label(), 'Unpublished book with "Show block only on book pages" book navigation settings.');
+    // Unpublished book with "Show block only on book pages" book navigation
+    // settings.
+    $this->assertSession()->pageTextContains($this->book->label());
   }
 
   /**
    * Tests that the book settings form can be saved without error.
    */
-  public function testSettingsForm() {
+  public function testSettingsForm(): void {
     $this->drupalLogin($this->adminUser);
-    $this->drupalPostForm('admin/structure/book/settings', [], 'Save configuration');
+    $this->drupalGet('admin/structure/book/settings');
+    $this->submitForm([], 'Save configuration');
   }
 
 }
