@@ -1,15 +1,18 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\ckeditor5\FunctionalJavascript;
 
+use Drupal\Core\Entity\Entity\EntityViewMode;
 use Drupal\editor\Entity\Editor;
 use Drupal\filter\Entity\FilterFormat;
 use Symfony\Component\Yaml\Yaml;
 
-// cspell:ignore esque imageUpload nofilter noeditor sourceediting Editing's
+// cspell:ignore esque imageUpload sourceediting Editing's
 
 /**
- * Tests for CKEditor5.
+ * Tests for CKEditor 5.
  *
  * @group ckeditor5
  * @internal
@@ -21,10 +24,11 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
    */
   protected static $modules = [
     'node',
-    'ckeditor',
+    'editor_test',
     'ckeditor5',
     'media',
     'media_library',
+    'ckeditor5_incompatible_filter_test',
   ];
 
   /**
@@ -35,74 +39,59 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
   protected $allowedElements = '<br> <p> <h2> <h3> <h4> <h5> <h6> <strong> <em>';
 
   /**
-   * The element that must be allowed when media embed is enabled.
+   * The default allowed elements for filter_html's "allowed_html" setting.
+   *
+   * @see \Drupal\filter\Plugin\Filter\FilterHtml
    *
    * @var string
    */
-  protected $mediaElement = '<drupal-media data-entity-type data-entity-uuid alt>';
+  protected $defaultElementsWhenUpdatingNotCkeditor5 = "<a href hreflang> <em> <strong> <cite> <blockquote cite> <code> <ul type> <ol start type='1 A I'> <li> <dl> <dt> <dd> <h2 id='jump-*'> <h3 id> <h4 id> <h5 id> <h6 id>";
 
   /**
-   * The default allowed elements when updating a non-CKEditor 5 editor.
+   * The expected allowed elements after updating to CKEditor 5.
    *
    * @var string
    */
-  protected $defaultElementsWhenUpdatingNotCkeditor5 = '<a href hreflang> <em> <strong> <cite> <blockquote cite> <code> <ul type> <ol start type> <li> <dl> <dt> <dd> <h2 id> <h3 id> <h4 id> <h5 id> <h6 id> <img src alt data-entity-type data-entity-uuid>';
+  protected $defaultElementsAfterUpdatingToCkeditor5 = '<br> <p> <h2 id="jump-*"> <h3 id> <h4 id> <h5 id> <h6 id> <cite> <dl> <dt> <dd> <a hreflang href> <blockquote cite> <ul type> <ol type="1 A I" start> <strong> <em> <code> <li>';
 
   /**
    * Test enabling CKEditor 5 in a way that triggers validation.
    */
-  public function testEnablingToVersion5Validation() {
+  public function testEnablingToVersion5Validation(): void {
     $page = $this->getSession()->getPage();
     $assert_session = $this->assertSession();
 
-    $this->createNewTextFormat($page, $assert_session, 'ckeditor');
-    $page->selectFieldOption('editor[editor]', 'ckeditor');
-    $assert_session->assertWaitOnAjaxRequest();
-    $page->checkField('filters[filter_html][status]');
-    $assert_session->assertWaitOnAjaxRequest();
-    $page->selectFieldOption('editor[editor]', 'ckeditor5');
-    $assert_session->assertWaitOnAjaxRequest();
-    $assert_session->pageTextContains('CKEditor 5 needs at least the <p> and <br> tags to be allowed to be able to function. They are not allowed by the "Limit allowed HTML tags and correct faulty HTML" (filter_html) filter.');
+    $incompatible_filter_name = 'filters[filter_incompatible][status]';
+    $filter_warning = 'CKEditor 5 only works with HTML-based text formats. The "A TYPE_MARKUP_LANGUAGE filter incompatible with CKEditor 5" (filter_incompatible) filter implies this text format is not HTML anymore.';
 
-    // Add the tags that must be included in the html filter for CKEditor 5
-    // defaults.
-    $allowed_html_field = $assert_session->fieldExists('filters[filter_html][settings][allowed_html]');
-    $allowed_html_field->setValue('<p> <br>');
+    $this->createNewTextFormat($page, $assert_session, 'unicorn');
+    $page->checkField('filters[filter_html][status]');
+    $page->checkField($incompatible_filter_name);
+    $page->selectFieldOption('editor[editor]', 'ckeditor5');
+    $assert_session->assertExpectedAjaxRequest(2);
+    $assert_session->pageTextContains($filter_warning);
+
+    // Disable the incompatible filter.
+    $page->uncheckField($incompatible_filter_name);
 
     // Confirm there are no longer any warnings.
     $assert_session->waitForElementRemoved('css', '[data-drupal-messages] [role="alert"]');
 
-    $page->pressButton('update-ckeditor5-allowed-tags');
-    $assert_session->assertWaitOnAjaxRequest();
+    // Confirm the text format can be saved.
     $this->saveNewTextFormat($page, $assert_session);
   }
 
   /**
-   * Tests that when image uploads are enabled in CKEditor 4, they remain in 5.
+   * Tests that when image uploads were enabled, they remain enabled.
    */
   public function testImageUploadsRemainEnabled(): void {
     FilterFormat::create([
-      'format' => 'cke4_image_uploads',
-      'name' => 'CKEditor 4, image uploads',
+      'format' => 'editor_with_image_uploads',
+      'name' => 'Text Editor with image uploads enabled',
     ])->save();
     Editor::create([
-      'format' => 'cke4_image_uploads',
-      'editor' => 'ckeditor',
-      'settings' => [
-        'toolbar' => [
-          'rows' => [
-            0 => [
-              [
-                'name' => 'Media',
-                'items' => [
-                  'DrupalImage',
-                ],
-              ],
-            ],
-          ],
-        ],
-        'plugins' => [],
-      ],
+      'format' => 'editor_with_image_uploads',
+      'editor' => 'unicorn',
       'image_upload' => [
         'status' => TRUE,
         'scheme' => 'public',
@@ -119,13 +108,19 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     $assert_session = $this->assertSession();
 
     // Assert that image uploads are enabled initially.
-    $this->drupalGet('admin/config/content/formats/manage/cke4_image_uploads');
+    $this->drupalGet('admin/config/content/formats/manage/editor_with_image_uploads');
     $this->assertTrue($page->hasCheckedField('Enable image uploads'));
 
     // Switch the text format to CKEditor 5.
     $page->selectFieldOption('editor[editor]', 'ckeditor5');
     $assert_session->assertWaitOnAjaxRequest();
 
+    // Enable the image toolbar item. This does NOT enable image uploads: it
+    // triggers the image upload settings form to become visible, to allow the
+    // image upload status to be checked.
+    $this->triggerKeyUp('.ckeditor5-toolbar-item-drupalInsertImage', 'ArrowDown');
+    $assert_session->assertWaitOnAjaxRequest();
+
     // Assert that image uploads are still enabled.
     $this->assertTrue($page->hasCheckedField('Enable image uploads'));
   }
@@ -133,65 +128,45 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
   /**
    * Confirm that switching to CKEditor 5 from another editor updates tags.
    */
-  public function testSwitchToVersion5() {
+  public function testSwitchToVersion5(): void {
     $page = $this->getSession()->getPage();
     $assert_session = $this->assertSession();
 
-    $this->createNewTextFormat($page, $assert_session, 'ckeditor');
-    $assert_session->assertWaitOnAjaxRequest();
+    $this->createNewTextFormat($page, $assert_session, 'unicorn');
 
     // Enable the HTML filter.
     $this->assertTrue($page->hasUncheckedField('filters[filter_html][status]'));
     $page->checkField('filters[filter_html][status]');
-    $assert_session->assertWaitOnAjaxRequest();
 
-    // Confirm the allowed HTML tags are the defaults for non-Ckeditor5 editors.
+    // Confirm the allowed HTML tags are the defaults initially.
     $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $this->defaultElementsWhenUpdatingNotCkeditor5);
 
     $this->saveNewTextFormat($page, $assert_session);
-    $assert_session->pageTextContains('Added text format ckeditor');
+    $assert_session->pageTextContains('Added text format unicorn');
 
     // Return to the config form to confirm that switching text editors on
     // existing formats will properly switch allowed tags.
-    $this->drupalGet('admin/config/content/formats/manage/ckeditor');
-    $assert_session->assertWaitOnAjaxRequest();
+    $this->drupalGet('admin/config/content/formats/manage/unicorn');
     $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $this->defaultElementsWhenUpdatingNotCkeditor5);
 
     $page->selectFieldOption('editor[editor]', 'ckeditor5');
     $assert_session->assertWaitOnAjaxRequest();
+    $assert_session->pageTextContains('The <br>, <p> tags were added because they are required by CKEditor 5');
+    $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $this->defaultElementsAfterUpdatingToCkeditor5);
 
-    $this->assertTrue($page->find('css', '[name="editor[editor]"]')->hasClass('error'));
-    $this->assertNotEmpty($assert_session->waitForElement('css', '[data-ckeditor5-allowed-tags-info]'));
-    $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $this->defaultElementsWhenUpdatingNotCkeditor5);
-
-    $allowed_html_field = $assert_session->fieldExists('filters[filter_html][settings][allowed_html]');
-    $allowed_html_field->setValue('<p> <br>');
-
-    $page->pressButton('update-ckeditor5-allowed-tags');
-    $assert_session->assertWaitOnAjaxRequest();
-    $this->assertFalse($page->find('css', '[name="editor[editor]"]')->hasClass('error'));
     $page->pressButton('Save configuration');
 
-    $assert_session->pageTextContains('The Image upload toolbar item requires image uploads to be enabled.');
-    $page->clickLink('Image Upload');
-    $assert_session->waitForText('Enable image uploads');
-    $this->assertTrue($page->hasUncheckedField('editor[settings][plugins][ckeditor5_imageUpload][status]'));
-    $page->checkField('editor[settings][plugins][ckeditor5_imageUpload][status]');
-    $assert_session->assertWaitOnAjaxRequest();
-    $page->pressButton('Save configuration');
-
-    $assert_session->pageTextContains('The text format ckeditor has been updated');
+    $assert_session->pageTextContains('The text format unicorn has been updated');
   }
 
   /**
    * Tests that the img tag is added after enabling image uploads.
    */
-  public function testImgAddedViaUploadPlugin() {
+  public function testImgAddedViaUploadPlugin(): void {
     $page = $this->getSession()->getPage();
     $assert_session = $this->assertSession();
 
     $this->createNewTextFormat($page, $assert_session);
-    $assert_session->assertWaitOnAjaxRequest();
 
     $allowed_html_field = $assert_session->fieldExists('filters[filter_html][settings][allowed_html]');
     $this->assertTrue($allowed_html_field->hasAttribute('readonly'));
@@ -202,62 +177,68 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     // The image upload settings form should not be present.
     $assert_session->elementNotExists('css', '[data-drupal-selector="edit-editor-settings-plugins-ckeditor5-imageupload"]');
 
-    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-item-uploadImage'));
-    $this->triggerKeyUp('.ckeditor5-toolbar-item-uploadImage', 'ArrowDown');
+    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-item-drupalInsertImage'));
+    $this->triggerKeyUp('.ckeditor5-toolbar-item-drupalInsertImage', 'ArrowDown');
     $assert_session->assertWaitOnAjaxRequest();
 
     // The image upload settings form should now be present.
-    $assert_session->elementExists('css', '[data-drupal-selector="edit-editor-settings-plugins-ckeditor5-imageupload"]');
+    $assert_session->elementExists('css', '[data-drupal-selector="edit-editor-settings-plugins-ckeditor5-image"]');
 
-    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-active .ckeditor5-toolbar-item-uploadImage'));
+    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-active .ckeditor5-toolbar-item-drupalInsertImage'));
 
-    // The image upload plugin is enabled, but <img> not yet allowed.
-    $this->assertEquals($this->allowedElements, $allowed_html_field->getValue());
+    // The image insert plugin is enabled and inserting <img> is allowed.
+    $this->assertEquals($this->allowedElements . ' <img src alt height width>', $allowed_html_field->getValue());
 
-    $page->clickLink('Image Upload');
+    $page->clickLink('Image');
     $assert_session->waitForText('Enable image uploads');
-    $this->assertTrue($page->hasUncheckedField('editor[settings][plugins][ckeditor5_imageUpload][status]'));
-    $page->checkField('editor[settings][plugins][ckeditor5_imageUpload][status]');
+    $this->assertTrue($page->hasUncheckedField('editor[settings][plugins][ckeditor5_image][status]'));
+    $page->checkField('editor[settings][plugins][ckeditor5_image][status]');
     $assert_session->assertWaitOnAjaxRequest();
 
     // Enabling image uploads adds <img> with several attributes to allowed
     // tags.
-    $this->assertEquals($this->allowedElements . ' <img src alt data-entity-uuid data-entity-type height width>', $allowed_html_field->getValue());
+    $this->assertEquals($this->allowedElements . ' <img src alt height width data-entity-uuid data-entity-type>', $allowed_html_field->getValue());
 
     // Also enabling the caption filter will add the data-caption attribute to
     // <img>.
     $this->assertTrue($page->hasUncheckedField('filters[filter_caption][status]'));
     $page->checkField('filters[filter_caption][status]');
     $assert_session->assertWaitOnAjaxRequest();
-    $this->assertEquals($this->allowedElements . ' <img src alt data-entity-uuid data-entity-type height width data-caption>', $allowed_html_field->getValue());
+    $this->assertEquals($this->allowedElements . ' <img src alt height width data-entity-uuid data-entity-type data-caption>', $allowed_html_field->getValue());
 
     // Also enabling the alignment filter will add the data-align attribute to
     // <img>.
     $this->assertTrue($page->hasUncheckedField('filters[filter_align][status]'));
     $page->checkField('filters[filter_align][status]');
     $assert_session->assertWaitOnAjaxRequest();
-    $this->assertEquals($this->allowedElements . ' <img src alt data-entity-uuid data-entity-type height width data-caption data-align>', $allowed_html_field->getValue());
+    $this->assertEquals($this->allowedElements . ' <img src alt height width data-entity-uuid data-entity-type data-caption data-align>', $allowed_html_field->getValue());
 
     // Disable image upload.
-    $page->clickLink('Image Upload');
+    $page->clickLink('Image');
     $assert_session->waitForText('Enable image uploads');
-    $this->assertTrue($page->hasCheckedField('editor[settings][plugins][ckeditor5_imageUpload][status]'));
-    $page->uncheckField('editor[settings][plugins][ckeditor5_imageUpload][status]');
+    $this->assertTrue($page->hasCheckedField('editor[settings][plugins][ckeditor5_image][status]'));
+    $page->uncheckField('editor[settings][plugins][ckeditor5_image][status]');
     $assert_session->assertWaitOnAjaxRequest();
 
-    // Confirm <img> is no longer an allowed tag, once image upload disabled.
+    // The image insert is still allowed when image uploads are disabled.
+    $this->assertEquals($this->allowedElements . ' <img src alt height width data-caption data-align>', $allowed_html_field->getValue());
+
+    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-item-drupalInsertImage'));
+    $this->triggerKeyUp('.ckeditor5-toolbar-item-drupalInsertImage', 'ArrowUp');
+    $assert_session->assertWaitOnAjaxRequest();
+
+    // Confirm <img> is no longer an allowed tag, once image insert is disabled.
     $this->assertEquals($this->allowedElements, $allowed_html_field->getValue());
   }
 
   /**
    * Test filter_html allowed tags.
    */
-  public function testAllowedTags() {
+  public function testAllowedTags(): void {
     $page = $this->getSession()->getPage();
     $assert_session = $this->assertSession();
 
     $this->createNewTextFormat($page, $assert_session);
-    $assert_session->assertWaitOnAjaxRequest();
 
     // Confirm the "allowed tags" field is  read only, and the value
     // matches the tags required by CKEditor.
@@ -275,7 +256,6 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     $this->assertSame($this->allowedElements, FilterFormat::load('ckeditor5')->filters('filter_html')->getConfiguration()['settings']['allowed_html']);
 
     $page->find('css', '[data-drupal-selector="edit-formats-ckeditor5"]')->clickLink('Configure');
-    $assert_session->assertWaitOnAjaxRequest();
 
     // Add the block quote plugin to the CKEditor 5 toolbar.
     $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-item-blockQuote'));
@@ -303,7 +283,7 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $allowed_with_blockquote);
 
     // Add the source editing plugin to the CKEditor 5 toolbar.
-    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-item-sourceEditing'));
+    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-available .ckeditor5-toolbar-item-sourceEditing'));
     $this->triggerKeyUp('.ckeditor5-toolbar-item-sourceEditing', 'ArrowDown');
     $assert_session->assertWaitOnAjaxRequest();
 
@@ -311,7 +291,6 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     // filter_html to include those additional tags.
     $assert_session->waitForText('Source editing');
     $page->find('css', '[href^="#edit-editor-settings-plugins-ckeditor5-sourceediting"]')->click();
-    $assert_session->assertWaitOnAjaxRequest();
     $assert_session->waitForText('Manually editable HTML tags');
     $source_edit_tags_field = $assert_session->fieldExists('editor[settings][plugins][ckeditor5_sourceEditing][allowed_tags]');
     $source_edit_tags_field->setValue('<aside>');
@@ -359,10 +338,25 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
   /**
    * Test that <drupal-media> is added to allowed tags when media embed enabled.
    */
-  public function testMediaElementAllowedTags() {
+  public function testMediaElementAllowedTags(): void {
     $page = $this->getSession()->getPage();
     $assert_session = $this->assertSession();
 
+    EntityViewMode::create([
+      'id' => 'media.view_mode_1',
+      'targetEntityType' => 'media',
+      'status' => TRUE,
+      'enabled' => TRUE,
+      'label' => 'View Mode 1',
+    ])->save();
+    EntityViewMode::create([
+      'id' => 'media.view_mode_2',
+      'targetEntityType' => 'media',
+      'status' => TRUE,
+      'enabled' => TRUE,
+      'label' => 'View Mode 2',
+    ])->save();
+
     $this->createNewTextFormat($page, $assert_session);
 
     // Allowed HTML field is readonly and its wrapper has a form-disabled class.
@@ -375,12 +369,22 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
 
     // Enable media embed.
     $this->assertTrue($page->hasUncheckedField('filters[media_embed][status]'));
+    $this->assertNull($assert_session->waitForElementVisible('css', '[data-drupal-selector=edit-filters-media-embed-settings]', 0));
     $page->checkField('filters[media_embed][status]');
-    $assert_session->assertWaitOnAjaxRequest();
-    $assert_session->responseContains('Media types selectable in the Media Library');
+    $assert_session->assertExpectedAjaxRequest(2);
+    $this->assertNotNull($assert_session->waitForElementVisible('css', '[data-drupal-selector=edit-filters-media-embed-settings]', 0));
 
-    $allowed_with_media = $this->allowedElements . ' ' . $this->mediaElement;
-    $assert_session->responseContains('Media types selectable in the Media Library');
+    $page->clickLink('Embed media');
+    $page->checkField('filters[media_embed][settings][allowed_view_modes][view_mode_1]');
+    $page->checkField('filters[media_embed][settings][allowed_view_modes][view_mode_2]');
+
+    $allowed_with_media = $this->allowedElements . ' <drupal-media data-entity-type data-entity-uuid alt data-view-mode>';
+    $allowed_with_media_without_view_mode = $this->allowedElements . ' <drupal-media data-entity-type data-entity-uuid alt>';
+    $page->clickLink('Media');
+    $this->assertTrue($page->hasUncheckedField('editor[settings][plugins][media_media][allow_view_mode_override]'));
+    $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $allowed_with_media_without_view_mode);
+    $page->checkField('editor[settings][plugins][media_media][allow_view_mode_override]');
+    $assert_session->assertExpectedAjaxRequest(3);
     $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $allowed_with_media);
     $this->saveNewTextFormat($page, $assert_session);
     $assert_session->pageTextContains('Added text format ckeditor5.');
@@ -395,34 +399,25 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     // field.
     $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $allowed_with_media);
 
+    // Ensure that data-align attribute is added to <drupal-media> when
+    // filter_align is enabled.
+    $page->checkField('filters[filter_align][status]');
+    $assert_session->assertExpectedAjaxRequest(1);
+    $this->assertEquals($this->allowedElements . ' <drupal-media data-entity-type data-entity-uuid alt data-view-mode data-align>', $allowed_html_field->getValue());
+
     // Disable media embed.
     $this->assertTrue($page->hasCheckedField('filters[media_embed][status]'));
     $page->uncheckField('filters[media_embed][status]');
-    $assert_session->assertWaitOnAjaxRequest();
 
+    $assert_session->assertExpectedAjaxRequest(2);
     // Confirm allowed tags no longer has <drupal-media>.
     $this->assertHtmlEsqueFieldValueEquals('filters[filter_html][settings][allowed_html]', $this->allowedElements);
   }
 
-  /**
-   * Tests the presence of the IE warning when CKEditor 5 is selected.
-   */
-  public function testInternetExplorerWarning() {
-    $page = $this->getSession()->getPage();
-    $assert_session = $this->assertSession();
-    $warning_text = 'CKEditor 5 is not compatible with Internet Explorer. Text fields using CKEditor 5 will fall back to plain HTML editing without CKEditor for users of Internet Explorer.';
-    $this->createNewTextFormat($page, $assert_session);
-    $assert_session->waitForText($warning_text);
-    $page->selectFieldOption('editor[editor]', 'None');
-    $this->getSession()->getDriver()->executeScript("document.querySelector('#drupal-live-announce').innerHTML = ''");
-    $assert_session->assertNoElementAfterWait('css', '.messages--warning');
-    $assert_session->pageTextNotContains($warning_text);
-  }
-
   /**
    * Tests full HTML text format.
    */
-  public function testFullHtml() {
+  public function testFullHtml(): void {
     FilterFormat::create(
       Yaml::parseFile('core/profiles/standard/config/install/filter.format.full_html.yml')
     )->save();
@@ -436,7 +431,7 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     // Add a node with text rendered via the Plain Text format.
     $this->drupalGet('node/add');
     $page->fillField('title[0][value]', 'My test content');
-    $page->fillField('body[0][value]', '<p><a style="color:#ff0000;" foo="bar" hreflang="en" href="https://example.com"><abbr title="National Aeronautics and Space Administration">NASA</abbr> is an acronym.</a></p>');
+    $page->fillField('body[0][value]', '<foo bar="baz">⬅️✌️➡️</foo><p><a style="color:#ff0000;" foo="bar" hreflang="en" href="https://example.com"><abbr title="National Aeronautics and Space Administration">NASA</abbr> is an acronym.</a></p>');
     $page->pressButton('Save');
 
     // Configure Full HTML text format to use CKEditor 5.
@@ -449,6 +444,13 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
 
     // Change the node's text format to Full HTML.
     $this->drupalGet('node/1/edit');
+    $filter_tips = $page->find('css', '[data-drupal-format-id="basic_html"]');
+    $this->assertTrue($filter_tips->isVisible());
+    $page->selectFieldOption('body[0][format]', 'full_html');
+    $this->assertNotEmpty($assert_session->waitForText('Change text format?'));
+    // Check the visibility of "Filter tips" by clicking the "Cancel" button.
+    $page->pressButton('Cancel');
+    $this->assertTrue($filter_tips->isVisible());
     $page->selectFieldOption('body[0][format]', 'full_html');
     $this->assertNotEmpty($assert_session->waitForText('Change text format?'));
     $page->pressButton('Continue');
@@ -459,7 +461,7 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
 
     // But note that the `style` attribute was stripped by
     // \Drupal\editor\EditorXssFilter\Standard.
-    $assert_session->responseContains('<p><a href="https://example.com" hreflang="en" foo="bar"><abbr title="National Aeronautics and Space Administration">NASA</abbr> is an acronym.</a></p>');
+    $assert_session->responseContains('<foo bar="baz">⬅️✌️➡️</foo><p><a foo="bar" hreflang="en" href="https://example.com"><abbr title="National Aeronautics and Space Administration">NASA</abbr> is an acronym.</a></p>');
 
     // Ensure attributes are retained after enabling link plugin.
     $this->drupalGet('admin/config/content/formats/manage/full_html');
@@ -471,15 +473,15 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     $this->drupalGet('node/1/edit');
     $page->pressButton('Save');
 
-    $assert_session->responseContains('<p><a href="https://example.com" hreflang="en" foo="bar"><abbr title="National Aeronautics and Space Administration">NASA</abbr> is an acronym.</a></p>');
+    $assert_session->responseContains('<p><a foo="bar" hreflang="en" href="https://example.com"><abbr title="National Aeronautics and Space Administration">NASA</abbr> is an acronym.</a></p>');
 
     // Configure Basic HTML text format to use CKE5 and enable the link plugin.
     $this->drupalGet('admin/config/content/formats/manage/basic_html');
     $page->checkField('roles[authenticated]');
     $page->selectFieldOption('editor[editor]', 'ckeditor5');
     $assert_session->assertWaitOnAjaxRequest();
-    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-item-link'));
-    $this->triggerKeyUp('.ckeditor5-toolbar-item-link', 'ArrowDown');
+    $this->assertNotEmpty($assert_session->waitForElement('css', '.ckeditor5-toolbar-available .ckeditor5-toolbar-item-underline'));
+    $this->triggerKeyUp('.ckeditor5-toolbar-item-underline', 'ArrowDown');
     $assert_session->assertWaitOnAjaxRequest();
     $page->pressButton('Save configuration');
     $this->assertTrue($assert_session->waitForText('The text format Basic HTML has been updated.'));
@@ -493,8 +495,8 @@ class CKEditor5AllowedTagsTest extends CKEditor5TestBase {
     $page->pressButton('Save');
 
     // The `style` and foo` attributes should have been removed, as should the
-    // `<abbr>` tag.
-    $assert_session->responseContains('<p><a href="https://example.com" hreflang="en">NASA is an acronym.</a></p>');
+    // `<abbr>` and `<foo>` tags.
+    $assert_session->responseContains('<p>⬅️✌️➡️</p><p><a href="https://example.com" hreflang="en">NASA is an acronym.</a></p>');
   }
 
 }
