@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\comment\Functional\Views;
 
 /**
@@ -22,13 +24,11 @@ class CommentRowTest extends CommentTestBase {
   public static $testViews = ['test_comment_row'];
 
   /**
-   * Test comment row.
+   * Tests comment row.
    */
-  public function testCommentRow() {
+  public function testCommentRow(): void {
     $this->drupalGet('test-comment-row');
-
-    $result = $this->xpath('//article[contains(@class, "comment")]');
-    $this->assertCount(1, $result, 'One rendered comment found.');
+    $this->assertSession()->elementsCount('xpath', '//article[contains(@class, "comment")]', 1);
   }
 
 }
