@@ -8,7 +8,6 @@
 use Drupal\node\NodeInterface;
 use Drupal\Component\Utility\Html;
 use Drupal\Component\Utility\Xss;
-use Drupal\Core\Access\AccessResult;
 
 /**
  * @addtogroup hooks
@@ -23,11 +22,8 @@ use Drupal\Core\Access\AccessResult;
  * "realms". In hook_node_access_records(), the realms and grant IDs are
  * associated with permission to view, edit, and delete individual nodes.
  *
- * The realms and grant IDs can be arbitrarily defined by your node access
- * module; it is common to use role IDs as grant IDs, but that is not required.
- * Your module could instead maintain its own list of users, where each list has
- * an ID. In that case, the return value of this hook would be an array of the
- * list IDs that this user is a member of.
+ * Grant IDs can be arbitrarily defined by a node access module using a list of
+ * integer IDs associated with users.
  *
  * A node access module may implement as many realms as necessary to properly
  * define the access privileges for the nodes. Note that the system makes no
@@ -43,20 +39,20 @@ use Drupal\Core\Access\AccessResult;
  * access modules can also grant "view all" permission on their custom realms;
  * for example, a module could create a record in {node_access} with:
  * @code
- * $record = array(
+ * $record = [
  *   'nid' => 0,
  *   'gid' => 888,
  *   'realm' => 'example_realm',
  *   'grant_view' => 1,
  *   'grant_update' => 0,
  *   'grant_delete' => 0,
- * );
+ * ];
  * \Drupal::database()->insert('node_access')->fields($record)->execute();
  * @endcode
  * And then in its hook_node_grants() implementation, it would need to return:
  * @code
  * if ($op == 'view') {
- *   $grants['example_realm'] = array(888);
+ *   $grants['example_realm'] = [888];
  * }
  * @endcode
  * If you decide to do this, be aware that the node_access_rebuild() function
@@ -64,11 +60,9 @@ use Drupal\Core\Access\AccessResult;
  * sure to restore your {node_access} record after node_access_rebuild() is
  * called.
  *
- * For a detailed example, see node_access_example.module.
- *
  * @param \Drupal\Core\Session\AccountInterface $account
  *   The account object whose grants are requested.
- * @param string $op
+ * @param string $operation
  *   The node operation to be performed, such as 'view', 'update', or 'delete'.
  *
  * @return array
@@ -79,7 +73,7 @@ use Drupal\Core\Access\AccessResult;
  * @see node_access_rebuild()
  * @ingroup node_access
  */
-function hook_node_grants(\Drupal\Core\Session\AccountInterface $account, $op) {
+function hook_node_grants(\Drupal\Core\Session\AccountInterface $account, $operation) {
   if ($account->hasPermission('access private content')) {
     $grants['example'] = [1];
   }
@@ -130,14 +124,14 @@ function hook_node_grants(\Drupal\Core\Session\AccountInterface $account, $op) {
  * A "deny all" grant may be used to deny all access to a particular node or
  * node translation:
  * @code
- * $grants[] = array(
+ * $grants[] = [
  *   'realm' => 'all',
  *   'gid' => 0,
  *   'grant_view' => 0,
  *   'grant_update' => 0,
  *   'grant_delete' => 0,
  *   'langcode' => 'ca',
- * );
+ * ];
  * @endcode
  * Note that another module node access module could override this by granting
  * access to one or more nodes, since grants are additive. To enforce that
@@ -148,7 +142,7 @@ function hook_node_grants(\Drupal\Core\Session\AccountInterface $account, $op) {
  * @param \Drupal\node\NodeInterface $node
  *   The node that has just been saved.
  *
- * @return
+ * @return array|null
  *   An array of grants as defined above.
  *
  * @see hook_node_access_records_alter()
@@ -174,8 +168,8 @@ function hook_node_access_records(\Drupal\node\NodeInterface $node) {
     }
     // For the example_author array, the GID is equivalent to a UID, which
     // means there are many groups of just 1 user.
-    // Note that an author can always view his or her nodes, even if they
-    // have status unpublished.
+    // Note that an author can always view nodes they own, even if they have
+    // status unpublished.
     if ($node->getOwnerId()) {
       $grants[] = [
         'realm' => 'example_author',
@@ -222,7 +216,7 @@ function hook_node_access_records(\Drupal\node\NodeInterface $node) {
  * @see hook_node_grants_alter()
  * @ingroup node_access
  */
-function hook_node_access_records_alter(&$grants, Drupal\node\NodeInterface $node) {
+function hook_node_access_records_alter(&$grants, \Drupal\node\NodeInterface $node) {
   // Our module allows editors to mark specific articles with the 'is_preview'
   // field. If the node being saved has a TRUE value for that field, then only
   // our grants are retained, and other grants are removed. Doing so ensures
@@ -259,7 +253,7 @@ function hook_node_access_records_alter(&$grants, Drupal\node\NodeInterface $nod
  *   The $grants array returned by hook_node_grants().
  * @param \Drupal\Core\Session\AccountInterface $account
  *   The account requesting access to content.
- * @param string $op
+ * @param string $operation
  *   The operation being performed, 'view', 'update' or 'delete'.
  *
  * @see hook_node_grants()
@@ -267,7 +261,7 @@ function hook_node_access_records_alter(&$grants, Drupal\node\NodeInterface $nod
  * @see hook_node_access_records_alter()
  * @ingroup node_access
  */
-function hook_node_grants_alter(&$grants, \Drupal\Core\Session\AccountInterface $account, $op) {
+function hook_node_grants_alter(&$grants, \Drupal\Core\Session\AccountInterface $account, $operation) {
   // Our sample module never allows certain roles to edit or delete
   // content. Since some other node access modules might allow this
   // permission, we expressly remove it by returning an empty $grants
@@ -276,7 +270,7 @@ function hook_node_grants_alter(&$grants, \Drupal\Core\Session\AccountInterface
   // Get our list of banned roles.
   $restricted = \Drupal::config('example.settings')->get('restricted_roles');
 
-  if ($op != 'view' && !empty($restricted)) {
+  if ($operation != 'view' && !empty($restricted)) {
     // Now check the roles for this account against the restrictions.
     foreach ($account->getRoles() as $rid) {
       if (in_array($rid, $restricted)) {
@@ -286,79 +280,6 @@ function hook_node_grants_alter(&$grants, \Drupal\Core\Session\AccountInterface
   }
 }
 
-/**
- * Controls access to a node.
- *
- * Modules may implement this hook if they want to have a say in whether or not
- * a given user has access to perform a given operation on a node.
- *
- * The administrative account (user ID #1) always passes any access check, so
- * this hook is not called in that case. Users with the "bypass node access"
- * permission may always view and edit content through the administrative
- * interface.
- *
- * The access to a node can be influenced in several ways:
- * - To explicitly allow access, return an AccessResultInterface object with
- * isAllowed() returning TRUE. Other modules can override this access by
- * returning TRUE for isForbidden().
- * - To explicitly forbid access, return an AccessResultInterface object with
- * isForbidden() returning TRUE. Access will be forbidden even if your module
- * (or another module) also returns TRUE for isNeutral() or isAllowed().
- * - To neither allow nor explicitly forbid access, return an
- * AccessResultInterface object with isNeutral() returning TRUE.
- * - If your module does not return an AccessResultInterface object, neutral
- * access will be assumed.
- *
- * Also note that this function isn't called for node listings (e.g., RSS feeds,
- * the default home page at path 'node', a recent content block, etc.) See
- * @link node_access Node access rights @endlink for a full explanation.
- *
- * @param \Drupal\node\NodeInterface|string $node
- *   Either a node entity or the machine name of the content type on which to
- *   perform the access check.
- * @param string $op
- *   The operation to be performed. Possible values:
- *   - "create"
- *   - "delete"
- *   - "update"
- *   - "view"
- * @param \Drupal\Core\Session\AccountInterface $account
- *   The user object to perform the access check operation on.
- *
- * @return \Drupal\Core\Access\AccessResultInterface
- *   The access result.
- *
- * @ingroup node_access
- */
-function hook_node_access(\Drupal\node\NodeInterface $node, $op, \Drupal\Core\Session\AccountInterface $account) {
-  $type = $node->bundle();
-
-  switch ($op) {
-    case 'create':
-      return AccessResult::allowedIfHasPermission($account, 'create ' . $type . ' content');
-
-    case 'update':
-      if ($account->hasPermission('edit any ' . $type . ' content')) {
-        return AccessResult::allowed()->cachePerPermissions();
-      }
-      else {
-        return AccessResult::allowedIf($account->hasPermission('edit own ' . $type . ' content') && ($account->id() == $node->getOwnerId()))->cachePerPermissions()->cachePerUser()->addCacheableDependency($node);
-      }
-
-    case 'delete':
-      if ($account->hasPermission('delete any ' . $type . ' content')) {
-        return AccessResult::allowed()->cachePerPermissions();
-      }
-      else {
-        return AccessResult::allowedIf($account->hasPermission('delete own ' . $type . ' content') && ($account->id() == $node->getOwnerId()))->cachePerPermissions()->cachePerUser()->addCacheableDependency($node);
-      }
-
-    default:
-      // No opinion.
-      return AccessResult::neutral();
-  }
-}
-
 /**
  * Act on a node being displayed as a search result.
  *
@@ -380,7 +301,7 @@ function hook_node_access(\Drupal\node\NodeInterface $node, $op, \Drupal\Core\Se
  * @ingroup entity_crud
  */
 function hook_node_search_result(\Drupal\node\NodeInterface $node) {
-  $rating = db_query('SELECT SUM(points) FROM {my_rating} WHERE nid = :nid', ['nid' => $node->id()])->fetchField();
+  $rating = \Drupal::database()->query('SELECT SUM([points]) FROM {my_rating} WHERE [nid] = :nid', ['nid' => $node->id()])->fetchField();
   return ['rating' => \Drupal::translation()->formatPlural($rating, '1 point', '@count points')];
 }
 
@@ -400,7 +321,7 @@ function hook_node_search_result(\Drupal\node\NodeInterface $node) {
  */
 function hook_node_update_index(\Drupal\node\NodeInterface $node) {
   $text = '';
-  $ratings = db_query('SELECT title, description FROM {my_ratings} WHERE nid = :nid', [':nid' => $node->id()]);
+  $ratings = \Drupal::database()->query('SELECT [title], [description] FROM {my_ratings} WHERE [nid] = :nid', [':nid' => $node->id()]);
   foreach ($ratings as $rating) {
     $text .= '<h2>' . Html::escape($rating->title) . '</h2>' . Xss::filter($rating->description);
   }
@@ -492,8 +413,8 @@ function hook_ranking() {
  * @see entity_crud
  */
 function hook_node_links_alter(array &$links, NodeInterface $entity, array &$context) {
-  $links['mymodule'] = [
-    '#theme' => 'links__node__mymodule',
+  $links['my_module'] = [
+    '#theme' => 'links__node__my_module',
     '#attributes' => ['class' => ['links', 'inline']],
     '#links' => [
       'node-report' => [
