@@ -5,31 +5,34 @@ namespace Drupal\system\Plugin\ImageToolkit;
 use Drupal\Component\Utility\Color;
 use Drupal\Core\Config\ConfigFactoryInterface;
 use Drupal\Core\File\Exception\FileException;
+use Drupal\Core\File\FileExists;
 use Drupal\Core\File\FileSystemInterface;
 use Drupal\Core\Form\FormStateInterface;
+use Drupal\Core\ImageToolkit\Attribute\ImageToolkit;
 use Drupal\Core\ImageToolkit\ImageToolkitBase;
 use Drupal\Core\ImageToolkit\ImageToolkitOperationManagerInterface;
 use Drupal\Core\StreamWrapper\StreamWrapperInterface;
+use Drupal\Core\StreamWrapper\StreamWrapperManager;
 use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
+use Drupal\Core\StringTranslation\TranslatableMarkup;
 use Psr\Log\LoggerInterface;
 use Symfony\Component\DependencyInjection\ContainerInterface;
 
+// cspell:ignore imagecreatefrom rrggbb
+
 /**
  * Defines the GD2 toolkit for image manipulation within Drupal.
- *
- * @ImageToolkit(
- *   id = "gd",
- *   title = @Translation("GD2 image manipulation toolkit")
- * )
  */
+#[ImageToolkit(
+  id: "gd",
+  title: new TranslatableMarkup("GD2 image manipulation toolkit"),
+)]
 class GDToolkit extends ImageToolkitBase {
 
   /**
-   * A GD image resource.
-   *
-   * @var resource|null
+   * A GD image.
    */
-  protected $resource = NULL;
+  protected ?\GdImage $image = NULL;
 
   /**
    * Image type represented by a PHP IMAGETYPE_* constant (e.g. IMAGETYPE_JPEG).
@@ -39,16 +42,16 @@ class GDToolkit extends ImageToolkitBase {
   protected $type;
 
   /**
-   * Image information from a file, available prior to loading the GD resource.
+   * Image information from a file, available prior to loading the GD object.
    *
    * This contains a copy of the array returned by executing getimagesize()
    * on the image file when the image object is instantiated. It gets reset
-   * to NULL as soon as the GD resource is loaded.
+   * to NULL as soon as the GD object is loaded.
    *
    * @var array|null
    *
    * @see \Drupal\system\Plugin\ImageToolkit\GDToolkit::parseFile()
-   * @see \Drupal\system\Plugin\ImageToolkit\GDToolkit::setResource()
+   * @see \Drupal\system\Plugin\ImageToolkit\GDToolkit::setImage()
    * @see http://php.net/manual/function.getimagesize.php
    */
   protected $preLoadInfo = NULL;
@@ -73,7 +76,7 @@ class GDToolkit extends ImageToolkitBase {
    * @param array $configuration
    *   A configuration array containing information about the plugin instance.
    * @param string $plugin_id
-   *   The plugin_id for the plugin instance.
+   *   The plugin ID for the plugin instance.
    * @param array $plugin_definition
    *   The plugin implementation definition.
    * @param \Drupal\Core\ImageToolkit\ImageToolkitOperationManagerInterface $operation_manager
@@ -87,27 +90,12 @@ class GDToolkit extends ImageToolkitBase {
    * @param \Drupal\Core\File\FileSystemInterface $file_system
    *   The file system.
    */
-  public function __construct(array $configuration, $plugin_id, array $plugin_definition, ImageToolkitOperationManagerInterface $operation_manager, LoggerInterface $logger, ConfigFactoryInterface $config_factory, StreamWrapperManagerInterface $stream_wrapper_manager, FileSystemInterface $file_system = NULL) {
+  public function __construct(array $configuration, $plugin_id, array $plugin_definition, ImageToolkitOperationManagerInterface $operation_manager, LoggerInterface $logger, ConfigFactoryInterface $config_factory, StreamWrapperManagerInterface $stream_wrapper_manager, FileSystemInterface $file_system) {
     parent::__construct($configuration, $plugin_id, $plugin_definition, $operation_manager, $logger, $config_factory);
     $this->streamWrapperManager = $stream_wrapper_manager;
-    if (!$file_system) {
-      @trigger_error('The file_system service must be passed to GDToolkit::__construct(), it is required before Drupal 9.0.0. See https://www.drupal.org/node/3006851.', E_USER_DEPRECATED);
-      $file_system = \Drupal::service('file_system');
-    }
     $this->fileSystem = $file_system;
   }
 
-  /**
-   * Destructs a GDToolkit object.
-   *
-   * Frees memory associated with a GD image resource.
-   */
-  public function __destruct() {
-    if (is_resource($this->resource)) {
-      imagedestroy($this->resource);
-    }
-  }
-
   /**
    * {@inheritdoc}
    */
@@ -124,35 +112,111 @@ class GDToolkit extends ImageToolkitBase {
     );
   }
 
+  /**
+   * {@inheritdoc}
+   */
+  public function __get(string $name) {
+    if ($name === 'resource') {
+      @trigger_error('Accessing the \Drupal\system\Plugin\ImageToolkit\GDToolkit::resource property is deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use \Drupal\system\Plugin\ImageToolkit\GDToolkit::image instead. See https://www.drupal.org/node/3265963', E_USER_DEPRECATED);
+      return $this->image;
+    }
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function __set(string $name, mixed $value): void {
+    if ($name === 'resource') {
+      @trigger_error('Setting the \Drupal\system\Plugin\ImageToolkit\GDToolkit::resource property is deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use \Drupal\system\Plugin\ImageToolkit\GDToolkit::image instead. See https://www.drupal.org/node/3265963', E_USER_DEPRECATED);
+      $this->image = $value;
+    }
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function __isset(string $name): bool {
+    if ($name === 'resource') {
+      @trigger_error('Checking the \Drupal\system\Plugin\ImageToolkit\GDToolkit::resource property is deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use \Drupal\system\Plugin\ImageToolkit\GDToolkit::image instead. See https://www.drupal.org/node/3265963', E_USER_DEPRECATED);
+      return isset($this->image);
+    }
+    return FALSE;
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public function __unset(string $name): void {
+    if ($name === 'resource') {
+      @trigger_error('Unsetting the \Drupal\system\Plugin\ImageToolkit\GDToolkit::resource property is deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use \Drupal\system\Plugin\ImageToolkit\GDToolkit::image instead. See https://www.drupal.org/node/3265963', E_USER_DEPRECATED);
+      unset($this->image);
+    }
+  }
+
   /**
    * Sets the GD image resource.
    *
-   * @param resource $resource
+   * @param \GdImage $resource
    *   The GD image resource.
    *
-   * @return \Drupal\system\Plugin\ImageToolkit\GDToolkit
+   * @return $this
    *   An instance of the current toolkit object.
+   *
+   * @deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use
+   *   \Drupal\system\Plugin\ImageToolkit\GDToolkit::setImage() instead.
+   *
+   * @see https://www.drupal.org/node/3265963
    */
   public function setResource($resource) {
-    if (!is_resource($resource) || get_resource_type($resource) != 'gd') {
+    @trigger_error(__METHOD__ . '() is deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use \Drupal\system\Plugin\ImageToolkit\GDToolkit::setImage() instead. See https://www.drupal.org/node/3265963', E_USER_DEPRECATED);
+    if (!$resource instanceof \GdImage) {
       throw new \InvalidArgumentException('Invalid resource argument');
     }
-    $this->preLoadInfo = NULL;
-    $this->resource = $resource;
-    return $this;
+    return $this->setImage($resource);
   }
 
   /**
    * Retrieves the GD image resource.
    *
-   * @return resource|null
+   * @return \GdImage|null
    *   The GD image resource, or NULL if not available.
+   *
+   * @deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use
+   *   \Drupal\system\Plugin\ImageToolkit\GDToolkit::getImage() instead.
+   *
+   * @see https://www.drupal.org/node/3265963
    */
   public function getResource() {
-    if (!is_resource($this->resource)) {
+    @trigger_error(__METHOD__ . '() is deprecated in drupal:10.2.0 and is removed from drupal:11.0.0. Use \Drupal\system\Plugin\ImageToolkit\GDToolkit::getImage() instead. See https://www.drupal.org/node/3265963', E_USER_DEPRECATED);
+    return $this->getImage();
+  }
+
+  /**
+   * Sets an image or resets existing one.
+   *
+   * @param \GdImage|null $image
+   *   The GD image object or NULL.
+   *
+   * @return $this
+   *   An instance of the current toolkit object.
+   */
+  public function setImage(?\GdImage $image): static {
+    $this->preLoadInfo = NULL;
+    $this->image = $image;
+    return $this;
+  }
+
+  /**
+   * Retrieves the image.
+   *
+   * @return \GdImage|null
+   *   The GD image object, or NULL if not available.
+   */
+  public function getImage(): ?\GdImage {
+    if (!$this->image) {
       $this->load();
     }
-    return $this->resource;
+    return $this->image;
   }
 
   /**
@@ -161,12 +225,12 @@ class GDToolkit extends ImageToolkitBase {
   public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
     $form['image_jpeg_quality'] = [
       '#type' => 'number',
-      '#title' => t('JPEG quality'),
-      '#description' => t('Define the image quality for JPEG manipulations. Ranges from 0 to 100. Higher values mean better image quality but bigger files.'),
+      '#title' => $this->t('JPEG quality'),
+      '#description' => $this->t('Define the image quality for JPEG manipulations. Ranges from 0 to 100. Higher values mean better image quality but bigger files.'),
       '#min' => 0,
       '#max' => 100,
       '#default_value' => $this->configFactory->getEditable('system.image.gd')->get('jpeg_quality', FALSE),
-      '#field_suffix' => t('%'),
+      '#field_suffix' => $this->t('%'),
     ];
     return $form;
   }
@@ -181,7 +245,7 @@ class GDToolkit extends ImageToolkitBase {
   }
 
   /**
-   * Loads a GD resource from a file.
+   * Loads an image from a file.
    *
    * @return bool
    *   TRUE or FALSE, based on success.
@@ -192,47 +256,69 @@ class GDToolkit extends ImageToolkitBase {
       return FALSE;
     }
 
+    // Invalidate the image object and return if there's no function to load the
+    // image file.
     $function = 'imagecreatefrom' . image_type_to_extension($this->getType(), FALSE);
-    if (function_exists($function) && $resource = $function($this->getSource())) {
-      $this->setResource($resource);
-      if (imageistruecolor($resource)) {
-        return TRUE;
-      }
-      else {
-        // Convert indexed images to truecolor, copying the image to a new
-        // truecolor resource, so that filters work correctly and don't result
-        // in unnecessary dither.
-        $data = [
-          'width' => imagesx($resource),
-          'height' => imagesy($resource),
-          'extension' => image_type_to_extension($this->getType(), FALSE),
-          'transparent_color' => $this->getTransparentColor(),
-          'is_temp' => TRUE,
-        ];
-        if ($this->apply('create_new', $data)) {
-          imagecopy($this->getResource(), $resource, 0, 0, 0, 0, imagesx($resource), imagesy($resource));
-          imagedestroy($resource);
-        }
-      }
-      return (bool) $this->getResource();
+    if (!function_exists($function)) {
+      $this->logger->error("The image toolkit '@toolkit' can not process image '@image'.", [
+        '@toolkit' => $this->getPluginId(),
+        '@image' => $this->getSource(),
+      ]);
+      $this->preLoadInfo = NULL;
+      return FALSE;
     }
-    return FALSE;
+
+    // Invalidate the image object and return if the load fails.
+    try {
+      $image = $function($this->getSource());
+    }
+    catch (\Throwable $t) {
+      $this->logger->error("The image toolkit '@toolkit' failed loading image '@image'. Reported error: @class - @message", [
+        '@toolkit' => $this->getPluginId(),
+        '@image' => $this->getSource(),
+        '@class' => get_class($t),
+        '@message' => $t->getMessage(),
+      ]);
+      $this->preLoadInfo = NULL;
+      return FALSE;
+    }
+
+    $this->setImage($image);
+    if (imageistruecolor($image)) {
+      return TRUE;
+    }
+    else {
+      // Convert indexed images to truecolor, copying the image to a new
+      // truecolor image, so that filters work correctly and don't result
+      // in unnecessary dither.
+      $data = [
+        'width' => imagesx($image),
+        'height' => imagesy($image),
+        'extension' => image_type_to_extension($this->getType(), FALSE),
+        'transparent_color' => $this->getTransparentColor(),
+        'is_temp' => TRUE,
+      ];
+      if ($this->apply('create_new', $data)) {
+        imagecopy($this->getImage(), $image, 0, 0, 0, 0, imagesx($image), imagesy($image));
+      }
+    }
+    return (bool) $this->getImage();
   }
 
   /**
    * {@inheritdoc}
    */
   public function isValid() {
-    return ((bool) $this->preLoadInfo || (bool) $this->resource);
+    return ((bool) $this->preLoadInfo || isset($this->image));
   }
 
   /**
    * {@inheritdoc}
    */
   public function save($destination) {
-    $scheme = file_uri_scheme($destination);
+    $scheme = StreamWrapperManager::getScheme($destination);
     // Work around lack of stream wrapper support in imagejpeg() and imagepng().
-    if ($scheme && file_stream_wrapper_valid_scheme($scheme)) {
+    if ($scheme && $this->streamWrapperManager->isValidScheme($scheme)) {
       // If destination is not local, save image to temporary local file.
       $local_wrappers = $this->streamWrapperManager->getWrappers(StreamWrapperInterface::LOCAL);
       if (!isset($local_wrappers[$scheme])) {
@@ -248,20 +334,42 @@ class GDToolkit extends ImageToolkitBase {
       return FALSE;
     }
     if ($this->getType() == IMAGETYPE_JPEG) {
-      $success = $function($this->getResource(), $destination, $this->configFactory->get('system.image.gd')->get('jpeg_quality'));
+      try {
+        $success = $function($this->getImage(), $destination, $this->configFactory->get('system.image.gd')->get('jpeg_quality'));
+      }
+      catch (\Throwable $t) {
+        $this->logger->error("The image toolkit '@toolkit' failed saving image '@image'. Reported error: @class - @message", [
+          '@toolkit' => $this->getPluginId(),
+          '@image' => $destination,
+          '@class' => get_class($t),
+          '@message' => $t->getMessage(),
+        ]);
+        $success = FALSE;
+      }
     }
     else {
-      // Always save PNG images with full transparency.
-      if ($this->getType() == IMAGETYPE_PNG) {
-        imagealphablending($this->getResource(), FALSE);
-        imagesavealpha($this->getResource(), TRUE);
+      // Image types that support alpha need to be saved accordingly.
+      if (in_array($this->getType(), [IMAGETYPE_PNG, IMAGETYPE_WEBP], TRUE)) {
+        imagealphablending($this->getImage(), FALSE);
+        imagesavealpha($this->getImage(), TRUE);
+      }
+      try {
+        $success = $function($this->getImage(), $destination);
+      }
+      catch (\Throwable $t) {
+        $this->logger->error("The image toolkit '@toolkit' failed saving image '@image'. Reported error: @class - @message", [
+          '@toolkit' => $this->getPluginId(),
+          '@image' => $destination,
+          '@class' => get_class($t),
+          '@message' => $t->getMessage(),
+        ]);
+        $success = FALSE;
       }
-      $success = $function($this->getResource(), $destination);
     }
     // Move temporary local file to remote destination.
     if (isset($permanent_destination) && $success) {
       try {
-        $this->fileSystem->move($destination, $permanent_destination, FileSystemInterface::EXISTS_REPLACE);
+        $this->fileSystem->move($destination, $permanent_destination, FileExists::Replace);
         return TRUE;
       }
       catch (FileException $e) {
@@ -291,16 +399,16 @@ class GDToolkit extends ImageToolkitBase {
    *   A color string like '#rrggbb', or NULL if not set or not relevant.
    */
   public function getTransparentColor() {
-    if (!$this->getResource() || $this->getType() != IMAGETYPE_GIF) {
+    if (!$this->getImage() || $this->getType() != IMAGETYPE_GIF) {
       return NULL;
     }
     // Find out if a transparent color is set, will return -1 if no
     // transparent color has been defined in the image.
-    $transparent = imagecolortransparent($this->getResource());
+    $transparent = imagecolortransparent($this->getImage());
     if ($transparent >= 0) {
       // Find out the number of colors in the image palette. It will be 0 for
       // truecolor images.
-      $palette_size = imagecolorstotal($this->getResource());
+      $palette_size = imagecolorstotal($this->getImage());
       if ($palette_size == 0 || $transparent < $palette_size) {
         // Return the transparent color, either if it is a truecolor image
         // or if the transparent color is part of the palette.
@@ -308,7 +416,7 @@ class GDToolkit extends ImageToolkitBase {
         // image rather than of the palette, it is possible that an image
         // could be created with this index set outside the palette size.
         // (see http://stackoverflow.com/a/3898007).
-        $rgb = imagecolorsforindex($this->getResource(), $transparent);
+        $rgb = imagecolorsforindex($this->getImage(), $transparent);
         unset($rgb['alpha']);
         return Color::rgbToHex($rgb);
       }
@@ -323,7 +431,7 @@ class GDToolkit extends ImageToolkitBase {
     if ($this->preLoadInfo) {
       return $this->preLoadInfo[0];
     }
-    elseif ($res = $this->getResource()) {
+    elseif ($res = $this->getImage()) {
       return imagesx($res);
     }
     else {
@@ -338,7 +446,7 @@ class GDToolkit extends ImageToolkitBase {
     if ($this->preLoadInfo) {
       return $this->preLoadInfo[1];
     }
-    elseif ($res = $this->getResource()) {
+    elseif ($res = $this->getImage()) {
       return imagesy($res);
     }
     else {
@@ -388,14 +496,59 @@ class GDToolkit extends ImageToolkitBase {
 
     $info = gd_info();
     $requirements['version'] = [
-      'title' => t('GD library'),
+      'title' => $this->t('GD library'),
       'value' => $info['GD Version'],
     ];
 
+    // Check if toolkit supported image formats can be actually processed by the
+    // GD library installed with PHP.
+    $check_formats = [
+      IMG_GIF => 'GIF',
+      IMG_JPG => 'JPEG',
+      IMG_PNG => 'PNG',
+      IMG_WEBP => 'WEBP',
+    ];
+    $supported_formats = array_filter($check_formats, fn($type) => imagetypes() & $type, ARRAY_FILTER_USE_KEY);
+    $unsupported_formats = array_diff_key($check_formats, $supported_formats);
+
+    $descriptions = [];
+    if ($supported_formats) {
+      $descriptions[] = $this->formatPlural(
+        count($supported_formats),
+        'Supported image file format: %formats.',
+        'Supported image file formats: %formats.',
+        ['%formats' => implode(', ', $supported_formats)]
+      );
+    }
+    if ($unsupported_formats) {
+      $requirements['version']['severity'] = REQUIREMENT_WARNING;
+      $unsupported = $this->formatPlural(
+        count($unsupported_formats),
+        'Unsupported image file format: %formats.',
+        'Unsupported image file formats: %formats.',
+        ['%formats' => implode(', ', $unsupported_formats)]
+      );
+      $fix_info = $this->t('Check the <a href="https://www.php.net/manual/en/image.installation.php">PHP GD installation documentation</a> if you want to add support.');
+      $descriptions[] = $this->t('@unsupported<br>@ref', [
+        '@unsupported' => $unsupported,
+        '@ref' => $fix_info,
+      ]);
+    }
+
     // Check for filter and rotate support.
     if (!function_exists('imagefilter') || !function_exists('imagerotate')) {
       $requirements['version']['severity'] = REQUIREMENT_WARNING;
-      $requirements['version']['description'] = t('The GD Library for PHP is enabled, but was compiled without support for functions used by the rotate and desaturate effects. It was probably compiled using the official GD libraries from http://www.libgd.org instead of the GD library bundled with PHP. You should recompile PHP --with-gd using the bundled GD library. See <a href="http://php.net/manual/book.image.php">the PHP manual</a>.');
+      $descriptions[] = $this->t('The GD Library for PHP is enabled, but was compiled without support for functions used by the rotate and desaturate effects. It was probably compiled using the official GD libraries from the <a href="https://libgd.github.io/">gdLibrary site</a> instead of the GD library bundled with PHP. You should recompile PHP --with-gd using the bundled GD library. See <a href="https://www.php.net/manual/book.image.php">the PHP manual</a>.');
+    }
+
+    if (count($descriptions) > 1) {
+      $requirements['version']['description'] = [
+        '#theme' => 'item_list',
+        '#items' => $descriptions,
+      ];
+    }
+    else {
+      $requirements['version']['description'] = $descriptions[0];
     }
 
     return $requirements;
@@ -462,7 +615,7 @@ class GDToolkit extends ImageToolkitBase {
    *   IMAGETYPE_* constant (e.g. IMAGETYPE_JPEG, IMAGETYPE_PNG, etc.).
    */
   protected static function supportedTypes() {
-    return [IMAGETYPE_PNG, IMAGETYPE_JPEG, IMAGETYPE_GIF];
+    return [IMAGETYPE_PNG, IMAGETYPE_JPEG, IMAGETYPE_GIF, IMAGETYPE_WEBP];
   }
 
 }
