@@ -9,7 +9,7 @@ namespace Drupal\FunctionalTests\Installer;
  *
  * @group Installer
  */
-class InstallerExistingConfigNoSystemSiteTest extends InstallerExistingConfigTestBase {
+class InstallerExistingConfigNoSystemSiteTest extends InstallerConfigDirectoryTestBase {
 
   /**
    * {@inheritdoc}
@@ -44,8 +44,8 @@ class InstallerExistingConfigNoSystemSiteTest extends InstallerExistingConfigTes
   /**
    * {@inheritdoc}
    */
-  protected function getConfigTarball() {
-    return __DIR__ . '/../../../fixtures/config_install/testing_config_install.tar.gz';
+  protected function getConfigLocation(): string {
+    return __DIR__ . '/../../../fixtures/config_install/testing_config_install';
   }
 
 }
