@@ -1,7 +1,10 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\KernelTests\Core\Entity;
 
+use Drupal\Core\Config\Entity\ConfigEntityStorageInterface;
 use Drupal\Core\Config\Entity\Query\QueryFactory;
 use Drupal\config_test\Entity\ConfigQueryTest;
 use Drupal\KernelTests\KernelTestBase;
@@ -15,11 +18,9 @@ use Drupal\KernelTests\KernelTestBase;
 class ConfigEntityQueryTest extends KernelTestBase {
 
   /**
-   * Modules to enable.
-   *
-   * @var array
+   * {@inheritdoc}
    */
-  public static $modules = ['config_test'];
+  protected static $modules = ['config_test'];
 
   /**
    * Stores the search results for alter comparison.
@@ -36,11 +37,9 @@ class ConfigEntityQueryTest extends KernelTestBase {
   protected $factory;
 
   /**
-   * The entity storage used for testing.
-   *
-   * @var \Drupal\Core\Entity\EntityStorageInterface
+   * The config entity storage used for testing.
    */
-  protected $entityStorage;
+  protected ConfigEntityStorageInterface $entityStorage;
 
   /**
    * Stores all config entities created for the test.
@@ -49,7 +48,10 @@ class ConfigEntityQueryTest extends KernelTestBase {
    */
   protected $entities;
 
-  protected function setUp() {
+  /**
+   * {@inheritdoc}
+   */
+  protected function setUp(): void {
     parent::setUp();
 
     $this->entities = [];
@@ -62,7 +64,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
     // The tests match array.level1.level2.
     $array['level1']['level2'] = 1;
     $entity = ConfigQueryTest::create([
-      'label' => $this->randomMachineName(),
+      'label' => 'entity_1',
       'id' => '1',
       'number' => 31,
       'array' => $array,
@@ -73,7 +75,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
 
     $array['level1']['level2'] = 2;
     $entity = ConfigQueryTest::create([
-      'label' => $this->randomMachineName(),
+      'label' => 'entity_2',
       'id' => '2',
       'number' => 41,
       'array' => $array,
@@ -84,7 +86,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
 
     $array['level1']['level2'] = 1;
     $entity = ConfigQueryTest::create([
-      'label' => 'test_prefix_' . $this->randomMachineName(),
+      'label' => 'test_prefix_entity_3',
       'id' => '3',
       'number' => 59,
       'array' => $array,
@@ -95,7 +97,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
 
     $array['level1']['level2'] = 2;
     $entity = ConfigQueryTest::create([
-      'label' => $this->randomMachineName() . '_test_suffix',
+      'label' => 'entity_4_test_suffix',
       'id' => '4',
       'number' => 26,
       'array' => $array,
@@ -106,7 +108,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
 
     $array['level1']['level2'] = 3;
     $entity = ConfigQueryTest::create([
-      'label' => $this->randomMachineName() . '_TEST_contains_' . $this->randomMachineName(),
+      'label' => 'entity_5_TEST_contains_entity_5',
       'id' => '5',
       'number' => 53,
       'array' => $array,
@@ -114,20 +116,41 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $this->entities[] = $entity;
     $entity->enforceIsNew();
     $entity->save();
+
+    $array['level1'] = [];
+    $entity = ConfigQueryTest::create([
+      'label' => $this->randomMachineName(),
+      'id' => '6',
+      'array' => $array,
+    ]);
+    $this->entities[] = $entity;
+    $entity->enforceIsNew();
+    $entity->save();
+
+    $array['level1']['level2'] = 4;
+    $entity = ConfigQueryTest::create([
+      'label' => $this->randomMachineName(),
+      'id' => '7',
+      'number' => 70,
+      'array' => $array,
+    ]);
+    $this->entities[] = $entity;
+    $entity->enforceIsNew();
+    $entity->save();
   }
 
   /**
    * Tests basic functionality.
    */
-  public function testConfigEntityQuery() {
+  public function testConfigEntityQuery(): void {
     // Run a test without any condition.
     $this->queryResults = $this->entityStorage->getQuery()
       ->execute();
-    $this->assertResults(['1', '2', '3', '4', '5']);
+    $this->assertResults(['1', '2', '3', '4', '5', '6', '7']);
     // No conditions, OR.
     $this->queryResults = $this->entityStorage->getQuery('OR')
       ->execute();
-    $this->assertResults(['1', '2', '3', '4', '5']);
+    $this->assertResults(['1', '2', '3', '4', '5', '6', '7']);
 
     // Filter by ID with equality.
     $this->queryResults = $this->entityStorage->getQuery()
@@ -169,19 +192,19 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $this->queryResults = $this->entityStorage->getQuery()
       ->condition('id', '3', '>')
       ->execute();
-    $this->assertResults(['4', '5']);
+    $this->assertResults(['4', '5', '6', '7']);
 
     // Filter by ID with the >= operator.
     $this->queryResults = $this->entityStorage->getQuery()
       ->condition('id', '3', '>=')
       ->execute();
-    $this->assertResults(['3', '4', '5']);
+    $this->assertResults(['3', '4', '5', '6', '7']);
 
     // Filter by ID with the <> operator.
     $this->queryResults = $this->entityStorage->getQuery()
       ->condition('id', '3', '<>')
       ->execute();
-    $this->assertResults(['1', '2', '4', '5']);
+    $this->assertResults(['1', '2', '4', '5', '6', '7']);
 
     // Filter by ID with the < operator.
     $this->queryResults = $this->entityStorage->getQuery()
@@ -226,7 +249,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
       ->condition('number', 10, '>=')
       ->condition('number', 50, '>=')
       ->execute();
-    $this->assertResults(['3', '5']);
+    $this->assertResults(['3', '5', '7']);
 
     // Filter with an OR condition group.
     $this->queryResults = $this->entityStorage->getQuery('OR')
@@ -249,7 +272,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $this->queryResults = $this->entityStorage->getQuery()
       ->condition('id', ['1', '2'], 'NOT IN')
       ->execute();
-    $this->assertResults(['3', '4', '5']);
+    $this->assertResults(['3', '4', '5', '6', '7']);
 
     // Filter with an OR condition group on different fields.
     $this->queryResults = $this->entityStorage->getQuery('OR')
@@ -328,7 +351,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $this->queryResults = $this->entityStorage->getQuery()
       ->exists('id')
       ->execute();
-    $this->assertResults(['1', '2', '3', '4', '5']);
+    $this->assertResults(['1', '2', '3', '4', '5', '6', '7']);
 
     $this->queryResults = $this->entityStorage->getQuery()
       ->exists('non-existent')
@@ -343,16 +366,16 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $this->queryResults = $this->entityStorage->getQuery()
       ->notExists('non-existent')
       ->execute();
-    $this->assertResults(['1', '2', '3', '4', '5']);
+    $this->assertResults(['1', '2', '3', '4', '5', '6', '7']);
   }
 
   /**
    * Tests ID conditions.
    */
-  public function testStringIdConditions() {
+  public function testStringIdConditions(): void {
     // We need an entity with a non-numeric ID.
     $entity = ConfigQueryTest::create([
-      'label' => $this->randomMachineName(),
+      'label' => 'entity_1',
       'id' => 'foo.bar',
     ]);
     $this->entities[] = $entity;
@@ -405,12 +428,12 @@ class ConfigEntityQueryTest extends KernelTestBase {
   /**
    * Tests count query.
    */
-  public function testCount() {
+  public function testCount(): void {
     // Test count on no conditions.
     $count = $this->entityStorage->getQuery()
       ->count()
       ->execute();
-    $this->assertIdentical($count, count($this->entities));
+    $this->assertCount($count, $this->entities);
 
     // Test count on a complex query.
     $query = $this->entityStorage->getQuery('OR');
@@ -425,72 +448,106 @@ class ConfigEntityQueryTest extends KernelTestBase {
       ->condition($and_condition_2)
       ->count()
       ->execute();
-    $this->assertIdentical($count, 2);
+    $this->assertSame(2, $count);
   }
 
   /**
    * Tests sorting and range on config entity queries.
    */
-  public function testSortRange() {
+  public function testSortRange(): void {
     // Sort by simple ascending/descending.
     $this->queryResults = $this->entityStorage->getQuery()
       ->sort('number', 'DESC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['3', '5', '2', '1', '4']);
+    $this->assertSame(['7', '3', '5', '2', '1', '4', '6'], array_values($this->queryResults));
 
     $this->queryResults = $this->entityStorage->getQuery()
       ->sort('number', 'ASC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['4', '1', '2', '5', '3']);
+    $this->assertSame(['6', '4', '1', '2', '5', '3', '7'], array_values($this->queryResults));
 
     // Apply some filters and sort.
     $this->queryResults = $this->entityStorage->getQuery()
       ->condition('id', '3', '>')
       ->sort('number', 'DESC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['5', '4']);
+    $this->assertSame(['7', '5', '4', '6'], array_values($this->queryResults));
 
     $this->queryResults = $this->entityStorage->getQuery()
       ->condition('id', '3', '>')
       ->sort('number', 'ASC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['4', '5']);
+    $this->assertSame(['6', '4', '5', '7'], array_values($this->queryResults));
 
     // Apply a pager and sort.
     $this->queryResults = $this->entityStorage->getQuery()
       ->sort('number', 'DESC')
       ->range('2', '2')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['2', '1']);
+    $this->assertSame(['5', '2'], array_values($this->queryResults));
 
     $this->queryResults = $this->entityStorage->getQuery()
       ->sort('number', 'ASC')
       ->range('2', '2')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['2', '5']);
+    $this->assertSame(['1', '2'], array_values($this->queryResults));
 
     // Add a range to a query without a start parameter.
     $this->queryResults = $this->entityStorage->getQuery()
       ->range(0, '3')
       ->sort('id', 'ASC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['1', '2', '3']);
+    $this->assertSame(['1', '2', '3'], array_values($this->queryResults));
+
+    // Omit optional parameters for the range and sort.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->range()
+      ->sort('id')
+      ->execute();
+    $this->assertSame(['1', '2', '3', '4', '5', '6', '7'], array_values($this->queryResults));
+
+    // Explicitly pass NULL for the range and sort.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->range(NULL, NULL)
+      ->sort('id')
+      ->execute();
+    $this->assertSame(['1', '2', '3', '4', '5', '6', '7'], array_values($this->queryResults));
+
+    // Omit the optional start parameter for the range.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->range(NULL, 1)
+      ->sort('id')
+      ->execute();
+    $this->assertSame(['1'], array_values($this->queryResults));
+
+    // Omit the optional length parameter for the range.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->range(4)
+      ->sort('id')
+      ->execute();
+    $this->assertSame(['5', '6', '7'], array_values($this->queryResults));
+
+    // Request an empty range.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->range(0, 0)
+      ->execute();
+    $this->assertEmpty($this->queryResults);
 
     // Apply a pager with limit 4.
     $this->queryResults = $this->entityStorage->getQuery()
       ->pager('4', 0)
       ->sort('id', 'ASC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['1', '2', '3', '4']);
+    $this->assertSame(['1', '2', '3', '4'], array_values($this->queryResults));
   }
 
   /**
    * Tests sorting with tableSort on config entity queries.
    */
-  public function testTableSort() {
+  public function testTableSort(): void {
     $header = [
-      ['data' => t('ID'), 'specifier' => 'id'],
-      ['data' => t('Number'), 'specifier' => 'number'],
+      ['data' => 'ID', 'specifier' => 'id'],
+      ['data' => 'Number', 'specifier' => 'number'],
     ];
 
     // Sort key: id
@@ -499,28 +556,28 @@ class ConfigEntityQueryTest extends KernelTestBase {
       ->tableSort($header)
       ->sort('id', 'DESC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['5', '4', '3', '2', '1']);
+    $this->assertSame(['7', '6', '5', '4', '3', '2', '1'], array_values($this->queryResults));
 
     // Sorting with 'ASC' upper case
     $this->queryResults = $this->entityStorage->getQuery()
       ->tableSort($header)
       ->sort('id', 'ASC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['1', '2', '3', '4', '5']);
+    $this->assertSame(['1', '2', '3', '4', '5', '6', '7'], array_values($this->queryResults));
 
     // Sorting with 'desc' lower case
     $this->queryResults = $this->entityStorage->getQuery()
       ->tableSort($header)
       ->sort('id', 'desc')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['5', '4', '3', '2', '1']);
+    $this->assertSame(['7', '6', '5', '4', '3', '2', '1'], array_values($this->queryResults));
 
     // Sorting with 'asc' lower case
     $this->queryResults = $this->entityStorage->getQuery()
       ->tableSort($header)
       ->sort('id', 'asc')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['1', '2', '3', '4', '5']);
+    $this->assertSame(['1', '2', '3', '4', '5', '6', '7'], array_values($this->queryResults));
 
     // Sort key: number
     // Sorting with 'DeSc' mixed upper and lower case
@@ -528,34 +585,34 @@ class ConfigEntityQueryTest extends KernelTestBase {
       ->tableSort($header)
       ->sort('number', 'DeSc')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['3', '5', '2', '1', '4']);
+    $this->assertSame(['7', '3', '5', '2', '1', '4', '6'], array_values($this->queryResults));
 
     // Sorting with 'AsC' mixed upper and lower case
     $this->queryResults = $this->entityStorage->getQuery()
       ->tableSort($header)
       ->sort('number', 'AsC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['4', '1', '2', '5', '3']);
+    $this->assertSame(['6', '4', '1', '2', '5', '3', '7'], array_values($this->queryResults));
 
     // Sorting with 'dEsC' mixed upper and lower case
     $this->queryResults = $this->entityStorage->getQuery()
       ->tableSort($header)
       ->sort('number', 'dEsC')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['3', '5', '2', '1', '4']);
+    $this->assertSame(['7', '3', '5', '2', '1', '4', '6'], array_values($this->queryResults));
 
     // Sorting with 'aSc' mixed upper and lower case
     $this->queryResults = $this->entityStorage->getQuery()
       ->tableSort($header)
       ->sort('number', 'aSc')
       ->execute();
-    $this->assertIdentical(array_values($this->queryResults), ['4', '1', '2', '5', '3']);
+    $this->assertSame(['6', '4', '1', '2', '5', '3', '7'], array_values($this->queryResults));
   }
 
   /**
    * Tests dotted path matching.
    */
-  public function testDotted() {
+  public function testDotted(): void {
     $this->queryResults = $this->entityStorage->getQuery()
       ->condition('array.level1.*', 1)
       ->execute();
@@ -572,6 +629,15 @@ class ConfigEntityQueryTest extends KernelTestBase {
       ->condition('array.level1.level2', 3)
       ->execute();
     $this->assertResults(['5']);
+    // Test dotted sorting.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->sort('array.level1.level2')
+      ->execute();
+    $this->assertResults(['6', '1', '3', '2', '4', '5', '7']);
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->sort('array.level1.level2', 'DESC')
+      ->execute();
+    $this->assertResults(['7', '5', '2', '4', '1', '3', '6']);
     // Make sure that values on the wildcard level do not match if there are
     // sub-keys defined. This must not find anything even if entity 2 has a
     // top-level key number with value 41.
@@ -581,23 +647,22 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $this->assertResults([]);
     // Make sure that "IS NULL" and "IS NOT NULL" work correctly with
     // array-valued fields/keys.
-    $all = ['1', '2', '3', '4', '5'];
     $this->queryResults = $this->entityStorage->getQuery()
       ->exists('array.level1.level2')
       ->execute();
-    $this->assertResults($all);
+    $this->assertResults(['1', '2', '3', '4', '5', '7']);
     $this->queryResults = $this->entityStorage->getQuery()
       ->exists('array.level1')
       ->execute();
-    $this->assertResults($all);
+    $this->assertResults(['1', '2', '3', '4', '5', '6', '7']);
     $this->queryResults = $this->entityStorage->getQuery()
       ->exists('array')
       ->execute();
-    $this->assertResults($all);
+    $this->assertResults(['1', '2', '3', '4', '5', '6', '7']);
     $this->queryResults = $this->entityStorage->getQuery()
       ->notExists('array.level1.level2')
       ->execute();
-    $this->assertResults([]);
+    $this->assertResults(['6']);
     $this->queryResults = $this->entityStorage->getQuery()
       ->notExists('array.level1')
       ->execute();
@@ -606,12 +671,22 @@ class ConfigEntityQueryTest extends KernelTestBase {
       ->notExists('array')
       ->execute();
     $this->assertResults([]);
+    // Make sure that "IS NULL" and "IS NOT NULL" work correctly when the dotted
+    // path cannot be fully followed.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->exists('does.not.exist')
+      ->execute();
+    $this->assertResults([]);
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->notExists('does.not.exist')
+      ->execute();
+    $this->assertResults(['1', '2', '3', '4', '5', '6', '7']);
   }
 
   /**
    * Tests case sensitivity.
    */
-  public function testCaseSensitivity() {
+  public function testCaseSensitivity(): void {
     // Filter by label with a known containing case-sensitive word.
     $this->queryResults = $this->entityStorage->getQuery()
       ->condition('label', 'TEST', 'CONTAINS')
@@ -627,7 +702,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
   /**
    * Tests lookup keys are added to the key value store.
    */
-  public function testLookupKeys() {
+  public function testLookupKeys(): void {
     \Drupal::service('state')->set('config_test.lookup_keys', TRUE);
     \Drupal::entityTypeManager()->clearCachedDefinitions();
     $key_value = $this->container->get('keyvalue')->get(QueryFactory::CONFIG_LOOKUP_PREFIX . 'config_test');
@@ -635,7 +710,7 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $test_entities = [];
     $storage = \Drupal::entityTypeManager()->getStorage('config_test');
     $entity = $storage->create([
-      'label' => $this->randomMachineName(),
+      'label' => 'entity_1',
       'id' => '1',
       'style' => 'test',
     ]);
@@ -644,10 +719,10 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $entity->save();
 
     $expected[] = $entity->getConfigDependencyName();
-    $this->assertEqual($expected, $key_value->get('style:test'));
+    $this->assertEquals($expected, $key_value->get('style:test'));
 
     $entity = $storage->create([
-      'label' => $this->randomMachineName(),
+      'label' => 'entity_2',
       'id' => '2',
       'style' => 'test',
     ]);
@@ -655,10 +730,10 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $entity->enforceIsNew();
     $entity->save();
     $expected[] = $entity->getConfigDependencyName();
-    $this->assertEqual($expected, $key_value->get('style:test'));
+    $this->assertEquals($expected, $key_value->get('style:test'));
 
     $entity = $storage->create([
-      'label' => $this->randomMachineName(),
+      'label' => 'entity_3',
       'id' => '3',
       'style' => 'blah',
     ]);
@@ -666,20 +741,38 @@ class ConfigEntityQueryTest extends KernelTestBase {
     $entity->save();
     // Do not add this entity to the list of expected result as it has a
     // different value.
-    $this->assertEqual($expected, $key_value->get('style:test'));
-    $this->assertEqual([$entity->getConfigDependencyName()], $key_value->get('style:blah'));
+    $this->assertEquals($expected, $key_value->get('style:test'));
+    $this->assertEquals([$entity->getConfigDependencyName()], $key_value->get('style:blah'));
 
     // Ensure that a delete clears a key.
     $entity->delete();
-    $this->assertEqual(NULL, $key_value->get('style:blah'));
+    $this->assertNull($key_value->get('style:blah'));
 
     // Ensure that delete only clears one key.
     $entity_id = array_pop($expected);
     $test_entities[$entity_id]->delete();
-    $this->assertEqual($expected, $key_value->get('style:test'));
+    $this->assertEquals($expected, $key_value->get('style:test'));
     $entity_id = array_pop($expected);
     $test_entities[$entity_id]->delete();
-    $this->assertEqual(NULL, $key_value->get('style:test'));
+    $this->assertNull($key_value->get('style:test'));
+  }
+
+  /**
+   * Test the entity query alter hooks are invoked.
+   *
+   * @see config_test_entity_query_tag__config_query_test__config_entity_query_alter_hook_test_alter()
+   */
+  public function testAlterHook(): void {
+    // Run a test without any condition.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->execute();
+    $this->assertResults(['1', '2', '3', '4', '5', '6', '7']);
+
+    // config_test alter hook removes the entity with id '7'.
+    $this->queryResults = $this->entityStorage->getQuery()
+      ->addTag('config_entity_query_alter_hook_test')
+      ->execute();
+    $this->assertResults(['1', '2', '3', '4', '5', '6']);
   }
 
   /**
@@ -687,12 +780,15 @@ class ConfigEntityQueryTest extends KernelTestBase {
    *
    * @param array $expected
    *   Array of expected entity IDs.
+   *
+   * @internal
    */
-  protected function assertResults($expected) {
-    $this->assertIdentical(count($this->queryResults), count($expected));
+  protected function assertResults(array $expected): void {
+    $expected_count = count($expected);
+    $this->assertCount($expected_count, $this->queryResults);
     foreach ($expected as $value) {
       // This also tests whether $this->queryResults[$value] is even set at all.
-      $this->assertIdentical($this->queryResults[$value], $value);
+      $this->assertSame($value, $this->queryResults[$value]);
     }
   }
 
