@@ -1,9 +1,6 @@
 <?php
 
-/**
- * @file
- * Contains \Drupal\Tests\Core\Entity\EntityFieldManagerTest.
- */
+declare(strict_types=1);
 
 namespace Drupal\Tests\Core\Entity;
 
@@ -15,7 +12,6 @@ use Drupal\Core\Entity\ContentEntityInterface;
 use Drupal\Core\Entity\ContentEntityTypeInterface;
 use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
 use Drupal\Core\Entity\EntityFieldManager;
-use Drupal\Core\Entity\EntityInterface;
 use Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface;
 use Drupal\Core\Entity\EntityStorageInterface;
 use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
@@ -195,16 +191,12 @@ class EntityFieldManagerTest extends UnitTestCase {
    *   (optional) An array of entity type definitions.
    */
   protected function setUpEntityTypeDefinitions($definitions = []) {
-    $class = $this->getMockClass(EntityInterface::class);
     foreach ($definitions as $key => $entity_type) {
       // \Drupal\Core\Entity\EntityTypeInterface::getLinkTemplates() is called
       // by \Drupal\Core\Entity\EntityTypeManager::processDefinition() so it must
       // always be mocked.
       $entity_type->getLinkTemplates()->willReturn([]);
 
-      // Give the entity type a legitimate class to return.
-      $entity_type->getClass()->willReturn($class);
-
       $definitions[$key] = $entity_type->reveal();
     }
 
@@ -231,7 +223,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    * @covers ::getBaseFieldDefinitions
    * @covers ::buildBaseFieldDefinitions
    */
-  public function testGetBaseFieldDefinitions() {
+  public function testGetBaseFieldDefinitions(): void {
     $field_definition = $this->setUpEntityWithFieldDefinition();
 
     $expected = ['id' => $field_definition];
@@ -244,7 +236,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    * @covers ::getFieldDefinitions
    * @covers ::buildBundleFieldDefinitions
    */
-  public function testGetFieldDefinitions() {
+  public function testGetFieldDefinitions(): void {
     $field_definition = $this->setUpEntityWithFieldDefinition();
 
     $bundle_field_definition = $this->prophesize()
@@ -303,7 +295,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    * @covers ::getFieldStorageDefinitions
    * @covers ::buildFieldStorageDefinitions
    */
-  public function testGetFieldStorageDefinitions() {
+  public function testGetFieldStorageDefinitions(): void {
     $field_definition = $this->setUpEntityWithFieldDefinition(TRUE);
     $field_storage_definition = $this->prophesize(FieldStorageDefinitionInterface::class);
     $field_storage_definition->getName()->willReturn('field_storage');
@@ -347,7 +339,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    *
    * @dataProvider providerTestGetBaseFieldDefinitionsTranslatableEntityTypeDefaultLangcode
    */
-  public function testGetBaseFieldDefinitionsTranslatableEntityTypeDefaultLangcode($default_langcode_key) {
+  public function testGetBaseFieldDefinitionsTranslatableEntityTypeDefaultLangcode($default_langcode_key): void {
     $this->setUpEntityWithFieldDefinition(FALSE, 'id', ['langcode' => 'langcode', 'default_langcode' => $default_langcode_key]);
 
     $field_definition = $this->prophesize()->willImplement(FieldDefinitionInterface::class)->willImplement(FieldStorageDefinitionInterface::class);
@@ -369,7 +361,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    * @return array
    *   Test data.
    */
-  public function providerTestGetBaseFieldDefinitionsTranslatableEntityTypeDefaultLangcode() {
+  public static function providerTestGetBaseFieldDefinitionsTranslatableEntityTypeDefaultLangcode() {
     return [
       ['default_langcode'],
       ['custom_default_langcode_key'],
@@ -384,7 +376,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    *
    * @dataProvider providerTestGetBaseFieldDefinitionsTranslatableEntityTypeLangcode
    */
-  public function testGetBaseFieldDefinitionsTranslatableEntityTypeLangcode($provide_key, $provide_field, $translatable) {
+  public function testGetBaseFieldDefinitionsTranslatableEntityTypeLangcode($provide_key, $provide_field, $translatable): void {
     $keys = $provide_key ? ['langcode' => 'langcode'] : [];
     $this->setUpEntityWithFieldDefinition(FALSE, 'id', $keys);
 
@@ -413,7 +405,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    * @return array
    *   Test data.
    */
-  public function providerTestGetBaseFieldDefinitionsTranslatableEntityTypeLangcode() {
+  public static function providerTestGetBaseFieldDefinitionsTranslatableEntityTypeLangcode() {
     return [
       [FALSE, TRUE, TRUE],
       [TRUE, FALSE, TRUE],
@@ -426,18 +418,19 @@ class EntityFieldManagerTest extends UnitTestCase {
    *
    * @covers ::getBaseFieldDefinitions
    */
-  public function testGetBaseFieldDefinitionsWithCaching() {
+  public function testGetBaseFieldDefinitionsWithCaching(): void {
     $field_definition = $this->setUpEntityWithFieldDefinition();
 
     $expected = ['id' => $field_definition];
 
+    $cacheBackend = $this->cacheBackend;
     $this->cacheBackend->get('entity_base_field_definitions:test_entity_type:en')
       ->willReturn(FALSE)
       ->shouldBeCalled();
     $this->cacheBackend->set('entity_base_field_definitions:test_entity_type:en', Argument::any(), Cache::PERMANENT, ['entity_types', 'entity_field_info'])
-      ->will(function ($args) {
+      ->will(function (array $args) use ($cacheBackend) {
         $data = (object) ['data' => $args[1]];
-        $this->get('entity_base_field_definitions:test_entity_type:en')
+        $cacheBackend->get('entity_base_field_definitions:test_entity_type:en')
           ->willReturn($data)
           ->shouldBeCalled();
       })
@@ -453,11 +446,12 @@ class EntityFieldManagerTest extends UnitTestCase {
    *
    * @covers ::getFieldDefinitions
    */
-  public function testGetFieldDefinitionsWithCaching() {
+  public function testGetFieldDefinitionsWithCaching(): void {
     $field_definition = $this->setUpEntityWithFieldDefinition(FALSE, 'id');
 
     $expected = ['id' => $field_definition];
 
+    $cacheBackend = $this->cacheBackend;
     $this->cacheBackend->get('entity_base_field_definitions:test_entity_type:en')
       ->willReturn((object) ['data' => $expected])
       ->shouldBeCalledTimes(2);
@@ -465,9 +459,9 @@ class EntityFieldManagerTest extends UnitTestCase {
       ->willReturn(FALSE)
       ->shouldBeCalledTimes(1);
     $this->cacheBackend->set('entity_bundle_field_definitions:test_entity_type:test_bundle:en', Argument::any(), Cache::PERMANENT, ['entity_types', 'entity_field_info'])
-      ->will(function ($args) {
+      ->will(function (array $args) use ($cacheBackend) {
         $data = (object) ['data' => $args[1]];
-        $this->get('entity_bundle_field_definitions:test_entity_type:test_bundle:en')
+        $cacheBackend->get('entity_bundle_field_definitions:test_entity_type:test_bundle:en')
           ->willReturn($data)
           ->shouldBeCalled();
       })
@@ -483,7 +477,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    *
    * @covers ::getFieldStorageDefinitions
    */
-  public function testGetFieldStorageDefinitionsWithCaching() {
+  public function testGetFieldStorageDefinitionsWithCaching(): void {
     $field_definition = $this->setUpEntityWithFieldDefinition(TRUE, 'id');
     $field_storage_definition = $this->prophesize(FieldStorageDefinitionInterface::class);
     $field_storage_definition->getName()->willReturn('field_storage');
@@ -507,14 +501,15 @@ class EntityFieldManagerTest extends UnitTestCase {
       'field_storage' => $field_storage_definition->reveal(),
     ];
 
+    $cacheBackend = $this->cacheBackend;
     $this->cacheBackend->get('entity_base_field_definitions:test_entity_type:en')
       ->willReturn((object) ['data' => ['id' => $expected['id']]])
       ->shouldBeCalledTimes(2);
     $this->cacheBackend->get('entity_field_storage_definitions:test_entity_type:en')->willReturn(FALSE);
 
     $this->cacheBackend->set('entity_field_storage_definitions:test_entity_type:en', Argument::any(), Cache::PERMANENT, ['entity_types', 'entity_field_info'])
-      ->will(function () use ($expected) {
-        $this->get('entity_field_storage_definitions:test_entity_type:en')
+      ->will(function () use ($expected, $cacheBackend) {
+        $cacheBackend->get('entity_field_storage_definitions:test_entity_type:en')
           ->willReturn((object) ['data' => $expected])
           ->shouldBeCalled();
       })
@@ -531,7 +526,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    * @covers ::getBaseFieldDefinitions
    * @covers ::buildBaseFieldDefinitions
    */
-  public function testGetBaseFieldDefinitionsInvalidDefinition() {
+  public function testGetBaseFieldDefinitionsInvalidDefinition(): void {
     $this->setUpEntityWithFieldDefinition(FALSE, 'langcode', ['langcode' => 'langcode']);
 
     $this->entityType->isTranslatable()->willReturn(TRUE);
@@ -547,13 +542,13 @@ class EntityFieldManagerTest extends UnitTestCase {
    * @covers ::getFieldDefinitions
    * @covers ::buildBundleFieldDefinitions
    */
-  public function testGetFieldDefinitionsProvider() {
+  public function testGetFieldDefinitionsProvider(): void {
     $this->setUpEntityWithFieldDefinition(TRUE);
 
     $module = 'entity_field_manager_test_module';
 
     // @todo Mock FieldDefinitionInterface once it exposes a proper provider
-    //   setter. See https://www.drupal.org/node/2225961.
+    //   setter. See https://www.drupal.org/node/2346329.
     $field_definition = $this->prophesize(BaseFieldDefinition::class);
 
     // We expect two calls as the field definition will be returned from both
@@ -651,7 +646,7 @@ class EntityFieldManagerTest extends UnitTestCase {
    *
    * @covers ::clearCachedFieldDefinitions
    */
-  public function testClearCachedFieldDefinitions() {
+  public function testClearCachedFieldDefinitions(): void {
     $this->setUpEntityTypeDefinitions();
 
     $this->cacheTagsInvalidator->invalidateTags(['entity_field_info'])->shouldBeCalled();
@@ -665,7 +660,7 @@ class EntityFieldManagerTest extends UnitTestCase {
   /**
    * @covers ::getExtraFields
    */
-  public function testGetExtraFields() {
+  public function testGetExtraFields(): void {
     $this->setUpEntityTypeDefinitions();
 
     $entity_type_id = $this->randomMachineName();
@@ -686,7 +681,7 @@ class EntityFieldManagerTest extends UnitTestCase {
     $processed_hook_bundle_extra_fields[$entity_type_id][$bundle] += [
       'display' => [],
     ];
-    $cache_id = 'entity_bundle_extra_fields:' . $entity_type_id . ':' . $bundle . ':' . $language_code;
+    $cache_id = 'entity_extra_field_info:' . $language_code;
 
     $language = new Language(['id' => $language_code]);
     $this->languageManager->getCurrentLanguage()
@@ -698,7 +693,7 @@ class EntityFieldManagerTest extends UnitTestCase {
     $this->moduleHandler->invokeAll('entity_extra_field_info')->willReturn($hook_bundle_extra_fields);
     $this->moduleHandler->alter('entity_extra_field_info', $hook_bundle_extra_fields)->shouldBeCalled();
 
-    $this->cacheBackend->set($cache_id, $processed_hook_bundle_extra_fields[$entity_type_id][$bundle], Cache::PERMANENT, ['entity_field_info'])->shouldBeCalled();
+    $this->cacheBackend->set($cache_id, $processed_hook_bundle_extra_fields, Cache::PERMANENT, ['entity_field_info'])->shouldBeCalled();
 
     $this->assertSame($processed_hook_bundle_extra_fields[$entity_type_id][$bundle], $this->entityFieldManager->getExtraFields($entity_type_id, $bundle));
   }
@@ -706,7 +701,7 @@ class EntityFieldManagerTest extends UnitTestCase {
   /**
    * @covers ::getFieldMap
    */
-  public function testGetFieldMap() {
+  public function testGetFieldMap(): void {
     $this->entityTypeBundleInfo->getBundleInfo('test_entity_type')->willReturn([])->shouldBeCalled();
 
     // Set up a content entity type.
@@ -782,7 +777,7 @@ class EntityFieldManagerTest extends UnitTestCase {
   /**
    * @covers ::getFieldMap
    */
-  public function testGetFieldMapFromCache() {
+  public function testGetFieldMapFromCache(): void {
     $expected = [
       'test_entity_type' => [
         'id' => [
@@ -806,7 +801,7 @@ class EntityFieldManagerTest extends UnitTestCase {
   /**
    * @covers ::getFieldMapByFieldType
    */
-  public function testGetFieldMapByFieldType() {
+  public function testGetFieldMapByFieldType(): void {
     // Set up a content entity type.
     $entity_type = $this->prophesize(ContentEntityTypeInterface::class);
     $entity_class = EntityTypeManagerTestEntity::class;
@@ -876,7 +871,7 @@ class TestEntityFieldManager extends EntityFieldManager {
   /**
    * Allows the static caches to be cleared.
    */
-  public function testClearEntityFieldInfo() {
+  public function testClearEntityFieldInfo(): void {
     $this->baseFieldDefinitions = [];
     $this->fieldDefinitions = [];
     $this->fieldStorageDefinitions = [];
@@ -916,7 +911,7 @@ abstract class EntityTypeManagerTestEntity implements \Iterator, ContentEntityIn
    * {@inheritdoc}
    */
   public static function bundleFieldDefinitions(EntityTypeInterface $entity_type, $bundle, array $base_field_definitions) {
-    return isset(static::$bundleFieldDefinitions[$entity_type->id()][$bundle]) ? static::$bundleFieldDefinitions[$entity_type->id()][$bundle] : [];
+    return static::$bundleFieldDefinitions[$entity_type->id()][$bundle] ?? [];
   }
 
 }
