@@ -1,14 +1,17 @@
 <?php
 
+declare(strict_types=1);
+
 namespace Drupal\Tests\Core\Entity\KeyValueStore;
 
+use Drupal\Component\Datetime\Time;
 use Drupal\Core\Cache\MemoryCache\MemoryCache;
 use Drupal\Core\Config\Entity\ConfigEntityInterface;
 use Drupal\Core\DependencyInjection\ContainerBuilder;
+use Drupal\Core\Entity\EntityBase;
 use Drupal\Core\Entity\EntityFieldManagerInterface;
 use Drupal\Core\Entity\EntityInterface;
 use Drupal\Core\Entity\EntityMalformedException;
-use Drupal\Core\Entity\EntityManager;
 use Drupal\Core\Entity\EntityStorageException;
 use Drupal\Core\Entity\EntityTypeManagerInterface;
 use Drupal\Core\Language\Language;
@@ -24,35 +27,35 @@ class KeyValueEntityStorageTest extends UnitTestCase {
   /**
    * The entity type.
    *
-   * @var \Drupal\Core\Entity\EntityTypeInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @var \Drupal\Core\Entity\EntityTypeInterface|\PHPUnit\Framework\MockObject\MockObject
    */
   protected $entityType;
 
   /**
    * The key value store.
    *
-   * @var \Drupal\Core\KeyValueStore\KeyValueStoreInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @var \Drupal\Core\KeyValueStore\KeyValueStoreInterface|\PHPUnit\Framework\MockObject\MockObject
    */
   protected $keyValueStore;
 
   /**
    * The module handler.
    *
-   * @var \Drupal\Core\Extension\ModuleHandlerInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @var \Drupal\Core\Extension\ModuleHandlerInterface|\PHPUnit\Framework\MockObject\MockObject
    */
   protected $moduleHandler;
 
   /**
    * The UUID service.
    *
-   * @var \Drupal\Component\Uuid\UuidInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @var \Drupal\Component\Uuid\UuidInterface|\PHPUnit\Framework\MockObject\MockObject
    */
   protected $uuidService;
 
   /**
    * The language manager.
    *
-   * @var \Drupal\Core\Language\LanguageManagerInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @var \Drupal\Core\Language\LanguageManagerInterface|\PHPUnit\Framework\MockObject\MockObject
    */
   protected $languageManager;
 
@@ -61,40 +64,33 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    */
   protected $entityStorage;
 
-  /**
-   * The entity manager.
-   *
-   * @var \Drupal\Core\Entity\EntityManagerInterface|\PHPUnit_Framework_MockObject_MockObject
-   */
-  protected $entityManager;
-
   /**
    * The mocked entity type manager.
    *
-   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\PHPUnit\Framework\MockObject\MockObject
    */
   protected $entityTypeManager;
 
   /**
    * The mocked entity field manager.
    *
-   * @var \Drupal\Core\Entity\EntityFieldManagerInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @var \Drupal\Core\Entity\EntityFieldManagerInterface|\PHPUnit\Framework\MockObject\MockObject
    */
   protected $entityFieldManager;
 
   /**
    * The mocked cache tags invalidator.
    *
-   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface|\PHPUnit\Framework\MockObject\MockObject
    */
   protected $cacheTagsInvalidator;
 
   /**
    * {@inheritdoc}
    */
-  protected function setUp() {
+  protected function setUp(): void {
     parent::setUp();
-    $this->entityType = $this->getMock('Drupal\Core\Entity\EntityTypeInterface');
+    $this->entityType = $this->createMock('Drupal\Core\Entity\EntityTypeInterface');
   }
 
   /**
@@ -108,54 +104,48 @@ class KeyValueEntityStorageTest extends UnitTestCase {
   protected function setUpKeyValueEntityStorage($uuid_key = 'uuid') {
     $this->entityType->expects($this->atLeastOnce())
       ->method('getKey')
-      ->will($this->returnValueMap([
+      ->willReturnMap([
         ['id', 'id'],
         ['uuid', $uuid_key],
         ['langcode', 'langcode'],
-      ]));
+      ]);
     $this->entityType->expects($this->atLeastOnce())
       ->method('id')
-      ->will($this->returnValue('test_entity_type'));
+      ->willReturn('test_entity_type');
     $this->entityType->expects($this->any())
       ->method('getListCacheTags')
       ->willReturn(['test_entity_type_list']);
 
-    $this->entityManager = new EntityManager();
-
-    $this->entityTypeManager = $this->getMock(EntityTypeManagerInterface::class);
+    $this->entityTypeManager = $this->createMock(EntityTypeManagerInterface::class);
     $this->entityTypeManager->expects($this->any())
       ->method('getDefinition')
       ->with('test_entity_type')
-      ->will($this->returnValue($this->entityType));
+      ->willReturn($this->entityType);
 
-    $this->entityFieldManager = $this->getMock(EntityFieldManagerInterface::class);
+    $this->entityFieldManager = $this->createMock(EntityFieldManagerInterface::class);
 
-    $this->cacheTagsInvalidator = $this->getMock('Drupal\Core\Cache\CacheTagsInvalidatorInterface');
+    $this->cacheTagsInvalidator = $this->createMock('Drupal\Core\Cache\CacheTagsInvalidatorInterface');
 
-    $this->keyValueStore = $this->getMock('Drupal\Core\KeyValueStore\KeyValueStoreInterface');
-    $this->moduleHandler = $this->getMock('Drupal\Core\Extension\ModuleHandlerInterface');
-    $this->uuidService = $this->getMock('Drupal\Component\Uuid\UuidInterface');
-    $this->languageManager = $this->getMock('Drupal\Core\Language\LanguageManagerInterface');
+    $this->keyValueStore = $this->createMock('Drupal\Core\KeyValueStore\KeyValueStoreInterface');
+    $this->moduleHandler = $this->createMock('Drupal\Core\Extension\ModuleHandlerInterface');
+    $this->uuidService = $this->createMock('Drupal\Component\Uuid\UuidInterface');
+    $this->languageManager = $this->createMock('Drupal\Core\Language\LanguageManagerInterface');
     $language = new Language(['langcode' => 'en']);
     $this->languageManager->expects($this->any())
       ->method('getDefaultLanguage')
-      ->will($this->returnValue($language));
+      ->willReturn($language);
     $this->languageManager->expects($this->any())
       ->method('getCurrentLanguage')
-      ->will($this->returnValue($language));
+      ->willReturn($language);
 
-    $this->entityStorage = new KeyValueEntityStorage($this->entityType, $this->keyValueStore, $this->uuidService, $this->languageManager, new MemoryCache());
+    $this->entityStorage = new KeyValueEntityStorage($this->entityType, $this->keyValueStore, $this->uuidService, $this->languageManager, new MemoryCache(new Time()));
     $this->entityStorage->setModuleHandler($this->moduleHandler);
 
     $container = new ContainerBuilder();
-    $container->set('entity.manager', $this->entityManager);
     $container->set('entity_field.manager', $this->entityFieldManager);
     $container->set('entity_type.manager', $this->entityTypeManager);
     $container->set('language_manager', $this->languageManager);
     $container->set('cache_tags.invalidator', $this->cacheTagsInvalidator);
-    // Inject the container into entity.manager so it can defer to
-    // entity_type.manager and other services.
-    $this->entityManager->setContainer($container);
     \Drupal::setContainer($container);
   }
 
@@ -163,18 +153,18 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::create
    * @covers ::doCreate
    */
-  public function testCreateWithPredefinedUuid() {
+  public function testCreateWithPredefinedUuid(): void {
     $this->entityType->expects($this->once())
       ->method('getClass')
-      ->will($this->returnValue(get_class($this->getMockEntity())));
+      ->willReturn(get_class($this->getMockEntity()));
     $this->setUpKeyValueEntityStorage();
 
-    $this->moduleHandler->expects($this->at(0))
+    $hooks = ['test_entity_type_create', 'entity_create'];
+    $this->moduleHandler->expects($this->exactly(count($hooks)))
       ->method('invokeAll')
-      ->with('test_entity_type_create');
-    $this->moduleHandler->expects($this->at(1))
-      ->method('invokeAll')
-      ->with('entity_create');
+      ->with($this->callback(function (string $hook) use (&$hooks): bool {
+        return array_shift($hooks) === $hook;
+      }));
     $this->uuidService->expects($this->never())
       ->method('generate');
 
@@ -188,19 +178,19 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::create
    * @covers ::doCreate
    */
-  public function testCreateWithoutUuidKey() {
+  public function testCreateWithoutUuidKey(): void {
     // Set up the entity storage to expect no UUID key.
     $this->entityType->expects($this->once())
       ->method('getClass')
-      ->will($this->returnValue(get_class($this->getMockEntity())));
+      ->willReturn(get_class($this->getMockEntity()));
     $this->setUpKeyValueEntityStorage(NULL);
 
-    $this->moduleHandler->expects($this->at(0))
+    $hooks = ['test_entity_type_create', 'entity_create'];
+    $this->moduleHandler->expects($this->exactly(count($hooks)))
       ->method('invokeAll')
-      ->with('test_entity_type_create');
-    $this->moduleHandler->expects($this->at(1))
-      ->method('invokeAll')
-      ->with('entity_create');
+      ->with($this->callback(function (string $hook) use (&$hooks): bool {
+        return array_shift($hooks) === $hook;
+      }));
     $this->uuidService->expects($this->never())
       ->method('generate');
 
@@ -217,21 +207,21 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @return \Drupal\Core\Entity\EntityInterface
    */
   public function testCreate() {
-    $entity = $this->getMockEntity('Drupal\Core\Entity\EntityBase', [], ['toArray']);
+    $entity = $this->getMockEntity(EntityBaseTest::class, [], ['toArray']);
     $this->entityType->expects($this->once())
       ->method('getClass')
-      ->will($this->returnValue(get_class($entity)));
+      ->willReturn(get_class($entity));
     $this->setUpKeyValueEntityStorage();
 
-    $this->moduleHandler->expects($this->at(0))
+    $hooks = ['test_entity_type_create', 'entity_create'];
+    $this->moduleHandler->expects($this->exactly(count($hooks)))
       ->method('invokeAll')
-      ->with('test_entity_type_create');
-    $this->moduleHandler->expects($this->at(1))
-      ->method('invokeAll')
-      ->with('entity_create');
+      ->with($this->callback(function (string $hook) use (&$hooks): bool {
+        return array_shift($hooks) === $hook;
+      }));
     $this->uuidService->expects($this->once())
       ->method('generate')
-      ->will($this->returnValue('bar'));
+      ->willReturn('bar');
 
     $entity = $this->entityStorage->create(['id' => 'foo']);
     $this->assertInstanceOf('Drupal\Core\Entity\EntityInterface', $entity);
@@ -245,22 +235,20 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::doSave
    *
    * @param \Drupal\Core\Entity\EntityInterface $entity
+   *   The entity.
    *
    * @return \Drupal\Core\Entity\EntityInterface
    *
    * @depends testCreate
    */
   public function testSaveInsert(EntityInterface $entity) {
-    $this->entityType->expects($this->once())
-      ->method('getClass')
-      ->will($this->returnValue(get_class($entity)));
     $this->setUpKeyValueEntityStorage();
 
     $expected = ['id' => 'foo'];
     $this->keyValueStore->expects($this->exactly(2))
       ->method('has')
       ->with('foo')
-      ->will($this->returnValue(FALSE));
+      ->willReturn(FALSE);
     $this->keyValueStore->expects($this->never())
       ->method('getMultiple');
     $this->keyValueStore->expects($this->never())
@@ -268,20 +256,15 @@ class KeyValueEntityStorageTest extends UnitTestCase {
 
     $entity->expects($this->atLeastOnce())
       ->method('toArray')
-      ->will($this->returnValue($expected));
+      ->willReturn($expected);
 
-    $this->moduleHandler->expects($this->at(0))
+    $hooks = ['test_entity_type_presave', 'entity_presave', 'test_entity_type_insert', 'entity_insert'];
+    $this->moduleHandler->expects($this->exactly(count($hooks)))
       ->method('invokeAll')
-      ->with('test_entity_type_presave');
-    $this->moduleHandler->expects($this->at(1))
-      ->method('invokeAll')
-      ->with('entity_presave');
-    $this->moduleHandler->expects($this->at(2))
-      ->method('invokeAll')
-      ->with('test_entity_type_insert');
-    $this->moduleHandler->expects($this->at(3))
-      ->method('invokeAll')
-      ->with('entity_insert');
+      ->with($this->callback(function (string $hook) use (&$hooks): bool {
+        return array_shift($hooks) === $hook;
+      }));
+
     $this->keyValueStore->expects($this->once())
       ->method('set')
       ->with('foo', $expected);
@@ -295,6 +278,7 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::doSave
    *
    * @param \Drupal\Core\Entity\EntityInterface $entity
+   *   The entity.
    *
    * @return \Drupal\Core\Entity\EntityInterface
    *
@@ -303,41 +287,28 @@ class KeyValueEntityStorageTest extends UnitTestCase {
   public function testSaveUpdate(EntityInterface $entity) {
     $this->entityType->expects($this->once())
       ->method('getClass')
-      ->will($this->returnValue(get_class($entity)));
+      ->willReturn(get_class($entity));
     $this->setUpKeyValueEntityStorage();
 
     $expected = ['id' => 'foo'];
     $this->keyValueStore->expects($this->exactly(2))
       ->method('has')
       ->with('foo')
-      ->will($this->returnValue(TRUE));
+      ->willReturn(TRUE);
     $this->keyValueStore->expects($this->once())
       ->method('getMultiple')
       ->with(['foo'])
-      ->will($this->returnValue([['id' => 'foo']]));
+      ->willReturn([['id' => 'foo']]);
     $this->keyValueStore->expects($this->never())
       ->method('delete');
 
-    $this->moduleHandler->expects($this->at(0))
-      ->method('getImplementations')
-      ->with('entity_load')
-      ->will($this->returnValue([]));
-    $this->moduleHandler->expects($this->at(1))
-      ->method('getImplementations')
-      ->with('test_entity_type_load')
-      ->will($this->returnValue([]));
-    $this->moduleHandler->expects($this->at(2))
+    $hooks = ['test_entity_type_presave', 'entity_presave', 'test_entity_type_update', 'entity_update'];
+    $this->moduleHandler->expects($this->exactly(count($hooks)))
       ->method('invokeAll')
-      ->with('test_entity_type_presave');
-    $this->moduleHandler->expects($this->at(3))
-      ->method('invokeAll')
-      ->with('entity_presave');
-    $this->moduleHandler->expects($this->at(4))
-      ->method('invokeAll')
-      ->with('test_entity_type_update');
-    $this->moduleHandler->expects($this->at(5))
-      ->method('invokeAll')
-      ->with('entity_update');
+      ->with($this->callback(function (string $hook) use (&$hooks): bool {
+        return array_shift($hooks) === $hook;
+      }));
+
     $this->keyValueStore->expects($this->once())
       ->method('set')
       ->with('foo', $expected);
@@ -364,12 +335,12 @@ class KeyValueEntityStorageTest extends UnitTestCase {
     $expected = ['id' => 'foo'];
     $entity->expects($this->atLeastOnce())
       ->method('toArray')
-      ->will($this->returnValue($expected));
+      ->willReturn($expected);
 
     $this->keyValueStore->expects($this->exactly(2))
       ->method('has')
       ->with('foo')
-      ->will($this->returnValue(FALSE));
+      ->willReturn(FALSE);
     $this->keyValueStore->expects($this->once())
       ->method('set')
       ->with('foo', $expected);
@@ -387,32 +358,24 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    *
    * @depends testSaveConfigEntity
    */
-  public function testSaveRenameConfigEntity(ConfigEntityInterface $entity) {
+  public function testSaveRenameConfigEntity(ConfigEntityInterface $entity): void {
     $this->entityType->expects($this->once())
       ->method('getClass')
-      ->will($this->returnValue(get_class($entity)));
+      ->willReturn(get_class($entity));
     $this->setUpKeyValueEntityStorage();
 
-    $this->moduleHandler->expects($this->at(0))
-      ->method('getImplementations')
-      ->with('entity_load')
-      ->will($this->returnValue([]));
-    $this->moduleHandler->expects($this->at(1))
-      ->method('getImplementations')
-      ->with('test_entity_type_load')
-      ->will($this->returnValue([]));
     $expected = ['id' => 'foo'];
     $entity->expects($this->once())
       ->method('toArray')
-      ->will($this->returnValue($expected));
+      ->willReturn($expected);
     $this->keyValueStore->expects($this->exactly(2))
       ->method('has')
       ->with('foo')
-      ->will($this->returnValue(TRUE));
+      ->willReturn(TRUE);
     $this->keyValueStore->expects($this->once())
       ->method('getMultiple')
       ->with(['foo'])
-      ->will($this->returnValue([['id' => 'foo']]));
+      ->willReturn([['id' => 'foo']]);
     $this->keyValueStore->expects($this->once())
       ->method('delete')
       ->with('foo');
@@ -434,19 +397,19 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::save
    * @covers ::doSave
    */
-  public function testSaveContentEntity() {
+  public function testSaveContentEntity(): void {
     $this->entityType->expects($this->any())
       ->method('getKeys')
-      ->will($this->returnValue([
+      ->willReturn([
         'id' => 'id',
-      ]));
+      ]);
     $this->setUpKeyValueEntityStorage();
 
     $expected = ['id' => 'foo'];
     $this->keyValueStore->expects($this->exactly(2))
       ->method('has')
       ->with('foo')
-      ->will($this->returnValue(FALSE));
+      ->willReturn(FALSE);
     $this->keyValueStore->expects($this->once())
       ->method('set')
       ->with('foo', $expected);
@@ -458,10 +421,10 @@ class KeyValueEntityStorageTest extends UnitTestCase {
     ]);
     $entity->expects($this->atLeastOnce())
       ->method('id')
-      ->will($this->returnValue('foo'));
+      ->willReturn('foo');
     $entity->expects($this->once())
       ->method('toArray')
-      ->will($this->returnValue($expected));
+      ->willReturn($expected);
     $this->entityStorage->save($entity);
   }
 
@@ -469,7 +432,7 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::save
    * @covers ::doSave
    */
-  public function testSaveInvalid() {
+  public function testSaveInvalid(): void {
     $this->setUpKeyValueEntityStorage();
 
     $entity = $this->getMockEntity('Drupal\Core\Config\Entity\ConfigEntityBase');
@@ -479,7 +442,8 @@ class KeyValueEntityStorageTest extends UnitTestCase {
       ->method('set');
     $this->keyValueStore->expects($this->never())
       ->method('delete');
-    $this->setExpectedException(EntityMalformedException::class, 'The entity does not have an ID.');
+    $this->expectException(EntityMalformedException::class);
+    $this->expectExceptionMessage('The entity does not have an ID.');
     $this->entityStorage->save($entity);
   }
 
@@ -487,19 +451,20 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::save
    * @covers ::doSave
    */
-  public function testSaveDuplicate() {
+  public function testSaveDuplicate(): void {
     $this->setUpKeyValueEntityStorage();
 
-    $entity = $this->getMockEntity('Drupal\Core\Entity\EntityBase', [['id' => 'foo']]);
+    $entity = $this->getMockEntity(EntityBaseTest::class, [['id' => 'foo']]);
     $entity->enforceIsNew();
     $this->keyValueStore->expects($this->once())
       ->method('has')
-      ->will($this->returnValue(TRUE));
+      ->willReturn(TRUE);
     $this->keyValueStore->expects($this->never())
       ->method('set');
     $this->keyValueStore->expects($this->never())
       ->method('delete');
-    $this->setExpectedException(EntityStorageException::class, "'test_entity_type' entity with ID 'foo' already exists");
+    $this->expectException(EntityStorageException::class);
+    $this->expectExceptionMessage("'test_entity_type' entity with ID 'foo' already exists");
     $this->entityStorage->save($entity);
   }
 
@@ -507,25 +472,17 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::load
    * @covers ::postLoad
    */
-  public function testLoad() {
+  public function testLoad(): void {
     $entity = $this->getMockEntity();
     $this->entityType->expects($this->once())
       ->method('getClass')
-      ->will($this->returnValue(get_class($entity)));
+      ->willReturn(get_class($entity));
     $this->setUpKeyValueEntityStorage();
 
     $this->keyValueStore->expects($this->once())
       ->method('getMultiple')
       ->with(['foo'])
-      ->will($this->returnValue([['id' => 'foo']]));
-    $this->moduleHandler->expects($this->at(0))
-      ->method('getImplementations')
-      ->with('entity_load')
-      ->will($this->returnValue([]));
-    $this->moduleHandler->expects($this->at(1))
-      ->method('getImplementations')
-      ->with('test_entity_type_load')
-      ->will($this->returnValue([]));
+      ->willReturn([['id' => 'foo']]);
     $entity = $this->entityStorage->load('foo');
     $this->assertInstanceOf('Drupal\Core\Entity\EntityInterface', $entity);
     $this->assertSame('foo', $entity->id());
@@ -534,17 +491,13 @@ class KeyValueEntityStorageTest extends UnitTestCase {
   /**
    * @covers ::load
    */
-  public function testLoadMissingEntity() {
-    $this->entityType->expects($this->once())
-      ->method('getClass');
+  public function testLoadMissingEntity(): void {
     $this->setUpKeyValueEntityStorage();
 
     $this->keyValueStore->expects($this->once())
       ->method('getMultiple')
       ->with(['foo'])
-      ->will($this->returnValue([]));
-    $this->moduleHandler->expects($this->never())
-      ->method('getImplementations');
+      ->willReturn([]);
     $entity = $this->entityStorage->load('foo');
     $this->assertNull($entity);
   }
@@ -555,25 +508,17 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::mapFromStorageRecords
    * @covers ::doLoadMultiple
    */
-  public function testLoadMultipleAll() {
-    $expected['foo'] = $this->getMockEntity('Drupal\Core\Entity\EntityBase', [['id' => 'foo']]);
-    $expected['bar'] = $this->getMockEntity('Drupal\Core\Entity\EntityBase', [['id' => 'bar']]);
+  public function testLoadMultipleAll(): void {
+    $expected['foo'] = $this->getMockEntity(EntityBaseTest::class, [['id' => 'foo']]);
+    $expected['bar'] = $this->getMockEntity(EntityBaseTest::class, [['id' => 'bar']]);
     $this->entityType->expects($this->once())
       ->method('getClass')
-      ->will($this->returnValue(get_class(reset($expected))));
+      ->willReturn(get_class(reset($expected)));
     $this->setUpKeyValueEntityStorage();
 
     $this->keyValueStore->expects($this->once())
       ->method('getAll')
-      ->will($this->returnValue([['id' => 'foo'], ['id' => 'bar']]));
-    $this->moduleHandler->expects($this->at(0))
-      ->method('getImplementations')
-      ->with('entity_load')
-      ->will($this->returnValue([]));
-    $this->moduleHandler->expects($this->at(1))
-      ->method('getImplementations')
-      ->with('test_entity_type_load')
-      ->will($this->returnValue([]));
+      ->willReturn([['id' => 'foo'], ['id' => 'bar']]);
     $entities = $this->entityStorage->loadMultiple();
     foreach ($entities as $id => $entity) {
       $this->assertInstanceOf('Drupal\Core\Entity\EntityInterface', $entity);
@@ -588,26 +533,18 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::mapFromStorageRecords
    * @covers ::doLoadMultiple
    */
-  public function testLoadMultipleIds() {
-    $entity = $this->getMockEntity('Drupal\Core\Entity\EntityBase', [['id' => 'foo']]);
+  public function testLoadMultipleIds(): void {
+    $entity = $this->getMockEntity(EntityBaseTest::class, [['id' => 'foo']]);
     $this->entityType->expects($this->once())
       ->method('getClass')
-      ->will($this->returnValue(get_class($entity)));
+      ->willReturn(get_class($entity));
     $this->setUpKeyValueEntityStorage();
 
     $expected[] = $entity;
     $this->keyValueStore->expects($this->once())
       ->method('getMultiple')
       ->with(['foo'])
-      ->will($this->returnValue([['id' => 'foo']]));
-    $this->moduleHandler->expects($this->at(0))
-      ->method('getImplementations')
-      ->with('entity_load')
-      ->will($this->returnValue([]));
-    $this->moduleHandler->expects($this->at(1))
-      ->method('getImplementations')
-      ->with('test_entity_type_load')
-      ->will($this->returnValue([]));
+      ->willReturn([['id' => 'foo']]);
     $entities = $this->entityStorage->loadMultiple(['foo']);
     foreach ($entities as $id => $entity) {
       $this->assertInstanceOf('Drupal\Core\Entity\EntityInterface', $entity);
@@ -617,58 +554,50 @@ class KeyValueEntityStorageTest extends UnitTestCase {
 
   /**
    * @covers ::loadRevision
+   * @group legacy
    */
-  public function testLoadRevision() {
+  public function testLoadRevision(): void {
+    $this->expectDeprecation('Drupal\Core\Entity\KeyValueStore\KeyValueEntityStorage::loadRevision() is deprecated in drupal:10.1.0 and is removed from drupal:11.0.0. Use \Drupal\Core\Entity\RevisionableStorageInterface::loadRevision instead. See https://www.drupal.org/node/3294237');
     $this->setUpKeyValueEntityStorage();
 
-    $this->assertSame(NULL, $this->entityStorage->loadRevision(1));
+    $this->assertNull($this->entityStorage->loadRevision(1));
   }
 
   /**
    * @covers ::deleteRevision
+   * @group legacy
    */
-  public function testDeleteRevision() {
+  public function testDeleteRevision(): void {
+    $this->expectDeprecation('Drupal\Core\Entity\KeyValueStore\KeyValueEntityStorage::deleteRevision() is deprecated in drupal:10.1.0 and is removed from drupal:11.0.0. Use \Drupal\Core\Entity\RevisionableStorageInterface::deleteRevision instead. See https://www.drupal.org/node/3294237');
     $this->setUpKeyValueEntityStorage();
 
-    $this->assertSame(NULL, $this->entityStorage->deleteRevision(1));
+    $this->assertNull($this->entityStorage->deleteRevision(1));
   }
 
   /**
    * @covers ::delete
    * @covers ::doDelete
    */
-  public function testDelete() {
-    $entities['foo'] = $this->getMockEntity('Drupal\Core\Entity\EntityBase', [['id' => 'foo']]);
-    $entities['bar'] = $this->getMockEntity('Drupal\Core\Entity\EntityBase', [['id' => 'bar']]);
-    $this->entityType->expects($this->once())
-      ->method('getClass')
-      ->will($this->returnValue(get_class(reset($entities))));
+  public function testDelete(): void {
+    $entities['foo'] = $this->getMockEntity(EntityBaseTest::class, [['id' => 'foo']]);
+    $entities['bar'] = $this->getMockEntity(EntityBaseTest::class, [['id' => 'bar']]);
     $this->setUpKeyValueEntityStorage();
 
-    $this->moduleHandler->expects($this->at(0))
+    $hooks = [
+      'test_entity_type_predelete',
+      'entity_predelete',
+      'test_entity_type_predelete',
+      'entity_predelete',
+      'test_entity_type_delete',
+      'entity_delete',
+      'test_entity_type_delete',
+      'entity_delete',
+    ];
+    $this->moduleHandler->expects($this->exactly(count($hooks)))
       ->method('invokeAll')
-      ->with('test_entity_type_predelete');
-    $this->moduleHandler->expects($this->at(1))
-      ->method('invokeAll')
-      ->with('entity_predelete');
-    $this->moduleHandler->expects($this->at(2))
-      ->method('invokeAll')
-      ->with('test_entity_type_predelete');
-    $this->moduleHandler->expects($this->at(3))
-      ->method('invokeAll')
-      ->with('entity_predelete');
-    $this->moduleHandler->expects($this->at(4))
-      ->method('invokeAll')
-      ->with('test_entity_type_delete');
-    $this->moduleHandler->expects($this->at(5))
-      ->method('invokeAll')
-      ->with('entity_delete');
-    $this->moduleHandler->expects($this->at(6))
-      ->method('invokeAll')
-      ->with('test_entity_type_delete');
-    $this->moduleHandler->expects($this->at(7))
-      ->method('invokeAll')
-      ->with('entity_delete');
+      ->with($this->callback(function (string $hook) use (&$hooks): bool {
+        return array_shift($hooks) === $hook;
+      }));
 
     $this->keyValueStore->expects($this->once())
       ->method('deleteMultiple')
@@ -680,7 +609,7 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @covers ::delete
    * @covers ::doDelete
    */
-  public function testDeleteNothing() {
+  public function testDeleteNothing(): void {
     $this->setUpKeyValueEntityStorage();
 
     $this->moduleHandler->expects($this->never())
@@ -705,9 +634,9 @@ class KeyValueEntityStorageTest extends UnitTestCase {
    * @param array $methods
    *   (optional) The methods to mock.
    *
-   * @return \Drupal\Core\Entity\EntityInterface|\PHPUnit_Framework_MockObject_MockObject
+   * @return \Drupal\Core\Entity\EntityInterface|\PHPUnit\Framework\MockObject\MockObject
    */
-  public function getMockEntity($class = 'Drupal\Core\Entity\EntityBase', array $arguments = [], $methods = []) {
+  public function getMockEntity($class = EntityBaseTest::class, array $arguments = [], $methods = []) {
     // Ensure the entity is passed at least an array of values and an entity
     // type ID
     if (!isset($arguments[0])) {
@@ -721,6 +650,15 @@ class KeyValueEntityStorageTest extends UnitTestCase {
 
 }
 
+class EntityBaseTest extends EntityBase {
+  public $id;
+  public $langcode;
+  public $uuid;
+  public $label;
+  public $original;
+
+}
+
 namespace Drupal\Core\Entity\KeyValueStore;
 
 if (!defined('SAVED_NEW')) {
