@@ -1,9 +1,6 @@
 <?php
 
-/**
- * @file
- * Contains \Drupal\Tests\Core\Form\FormBuilderTest.
- */
+declare(strict_types=1);
 
 namespace Drupal\Tests\Core\Form;
 
@@ -25,6 +22,8 @@ use Symfony\Component\DependencyInjection\ContainerBuilder;
 use Symfony\Component\DependencyInjection\ContainerInterface;
 use Symfony\Component\HttpFoundation\Request;
 use Symfony\Component\HttpFoundation\RequestStack;
+use Symfony\Component\HttpFoundation\Session\Session;
+use Symfony\Component\HttpFoundation\Session\Storage\MockArraySessionStorage;
 
 /**
  * @coversDefaultClass \Drupal\Core\Form\FormBuilder
@@ -42,7 +41,7 @@ class FormBuilderTest extends FormTestBase {
   /**
    * {@inheritdoc}
    */
-  protected function setUp() {
+  protected function setUp(): void {
     parent::setUp();
 
     $this->container = new ContainerBuilder();
@@ -53,18 +52,32 @@ class FormBuilderTest extends FormTestBase {
 
   /**
    * Tests the getFormId() method with a string based form ID.
+   *
+   * @covers ::getFormId
    */
-  public function testGetFormIdWithString() {
+  public function testGetFormIdWithString(): void {
     $form_arg = 'foo';
     $form_state = new FormState();
-    $this->setExpectedException(\InvalidArgumentException::class, 'The form argument foo is not a valid form.');
+    $this->expectException(\InvalidArgumentException::class);
+    $this->expectExceptionMessage('The form class foo could not be found or loaded.');
+    $this->formBuilder->getFormId($form_arg, $form_state);
+  }
+
+  /**
+   * @covers ::getFormId
+   */
+  public function testGetFormIdWithNonFormClass(): void {
+    $form_arg = __CLASS__;
+    $form_state = new FormState();
+    $this->expectException(\InvalidArgumentException::class);
+    $this->expectExceptionMessage("The form argument $form_arg must be an instance of \Drupal\Core\Form\FormInterface.");
     $this->formBuilder->getFormId($form_arg, $form_state);
   }
 
   /**
    * Tests the getFormId() method with a class name form ID.
    */
-  public function testGetFormIdWithClassName() {
+  public function testGetFormIdWithClassName(): void {
     $form_arg = 'Drupal\Tests\Core\Form\TestForm';
 
     $form_state = new FormState();
@@ -77,8 +90,8 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the getFormId() method with an injected class name form ID.
    */
-  public function testGetFormIdWithInjectedClassName() {
-    $container = $this->getMock('Symfony\Component\DependencyInjection\ContainerInterface');
+  public function testGetFormIdWithInjectedClassName(): void {
+    $container = $this->createMock('Symfony\Component\DependencyInjection\ContainerInterface');
     \Drupal::setContainer($container);
 
     $form_arg = 'Drupal\Tests\Core\Form\TestFormInjected';
@@ -93,7 +106,7 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the getFormId() method with a form object.
    */
-  public function testGetFormIdWithObject() {
+  public function testGetFormIdWithObject(): void {
     $expected_form_id = 'my_module_form_id';
 
     $form_arg = $this->getMockForm($expected_form_id);
@@ -108,17 +121,17 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the getFormId() method with a base form object.
    */
-  public function testGetFormIdWithBaseForm() {
+  public function testGetFormIdWithBaseForm(): void {
     $expected_form_id = 'my_module_form_id';
     $base_form_id = 'my_module';
 
-    $form_arg = $this->getMock('Drupal\Core\Form\BaseFormIdInterface');
+    $form_arg = $this->createMock('Drupal\Core\Form\BaseFormIdInterface');
     $form_arg->expects($this->once())
       ->method('getFormId')
-      ->will($this->returnValue($expected_form_id));
+      ->willReturn($expected_form_id);
     $form_arg->expects($this->once())
       ->method('getBaseFormId')
-      ->will($this->returnValue($base_form_id));
+      ->willReturn($base_form_id);
 
     $form_state = new FormState();
     $form_id = $this->formBuilder->getFormId($form_arg, $form_state);
@@ -133,7 +146,7 @@ class FormBuilderTest extends FormTestBase {
    *
    * @dataProvider formStateResponseProvider
    */
-  public function testHandleFormStateResponse($class, $form_state_key) {
+  public function testHandleFormStateResponse($class, $form_state_key): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
 
@@ -144,9 +157,9 @@ class FormBuilderTest extends FormTestBase {
     $form_arg = $this->getMockForm($form_id, $expected_form);
     $form_arg->expects($this->any())
       ->method('submitForm')
-      ->will($this->returnCallback(function ($form, FormStateInterface $form_state) use ($response, $form_state_key) {
+      ->willReturnCallback(function ($form, FormStateInterface $form_state) use ($response, $form_state_key) {
         $form_state->setFormState([$form_state_key => $response]);
-      }));
+      });
 
     $form_state = new FormState();
     try {
@@ -164,7 +177,7 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Provides test data for testHandleFormStateResponse().
    */
-  public function formStateResponseProvider() {
+  public static function formStateResponseProvider() {
     return [
       ['Symfony\Component\HttpFoundation\Response', 'response'],
       ['Symfony\Component\HttpFoundation\RedirectResponse', 'redirect'],
@@ -174,7 +187,7 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the handling of a redirect when FormStateInterface::$response exists.
    */
-  public function testHandleRedirectWithResponse() {
+  public function testHandleRedirectWithResponse(): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
 
@@ -191,11 +204,11 @@ class FormBuilderTest extends FormTestBase {
     $form_arg = $this->getMockForm($form_id, $expected_form);
     $form_arg->expects($this->any())
       ->method('submitForm')
-      ->will($this->returnCallback(function ($form, FormStateInterface $form_state) use ($response, $redirect) {
+      ->willReturnCallback(function ($form, FormStateInterface $form_state) use ($response, $redirect) {
         // Set both the response and the redirect.
         $form_state->setResponse($response);
         $form_state->set('redirect', $redirect);
-      }));
+      });
 
     $form_state = new FormState();
     try {
@@ -213,16 +226,17 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the getForm() method with a string based form ID.
    */
-  public function testGetFormWithString() {
+  public function testGetFormWithString(): void {
     $form_id = 'test_form_id';
-    $this->setExpectedException(\InvalidArgumentException::class, 'The form argument test_form_id is not a valid form.');
+    $this->expectException(\InvalidArgumentException::class);
+    $this->expectExceptionMessage('The form class test_form_id could not be found or loaded.');
     $this->formBuilder->getForm($form_id);
   }
 
   /**
    * Tests the getForm() method with a form object.
    */
-  public function testGetFormWithObject() {
+  public function testGetFormWithObject(): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
 
@@ -236,7 +250,7 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the getForm() method with a class name based form ID.
    */
-  public function testGetFormWithClassString() {
+  public function testGetFormWithClassString(): void {
     $form_id = '\Drupal\Tests\Core\Form\TestForm';
     $object = new TestForm();
     $form = [];
@@ -251,16 +265,17 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the buildForm() method with a string based form ID.
    */
-  public function testBuildFormWithString() {
+  public function testBuildFormWithString(): void {
     $form_id = 'test_form_id';
-    $this->setExpectedException(\InvalidArgumentException::class, 'The form argument test_form_id is not a valid form.');
+    $this->expectException(\InvalidArgumentException::class);
+    $this->expectExceptionMessage('The form class test_form_id could not be found or loaded.');
     $this->formBuilder->getForm($form_id);
   }
 
   /**
    * Tests the buildForm() method with a class name based form ID.
    */
-  public function testBuildFormWithClassString() {
+  public function testBuildFormWithClassString(): void {
     $form_id = '\Drupal\Tests\Core\Form\TestForm';
     $object = new TestForm();
     $form = [];
@@ -275,7 +290,7 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the buildForm() method with a form object.
    */
-  public function testBuildFormWithObject() {
+  public function testBuildFormWithObject(): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
 
@@ -300,7 +315,7 @@ class FormBuilderTest extends FormTestBase {
    *
    * @dataProvider providerTestBuildFormWithTriggeringElement
    */
-  public function testBuildFormWithTriggeringElement($element_value, $input_value) {
+  public function testBuildFormWithTriggeringElement($element_value, $input_value): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
 
@@ -322,7 +337,7 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Data provider for ::testBuildFormWithTriggeringElement().
    */
-  public function providerTestBuildFormWithTriggeringElement() {
+  public static function providerTestBuildFormWithTriggeringElement() {
     $plain_text = 'Other submit value';
     $markup = 'Other submit <input> value';
     return [
@@ -338,18 +353,18 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the rebuildForm() method for a POST submission.
    */
-  public function testRebuildForm() {
+  public function testRebuildForm(): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
 
     // The form will be built four times.
-    $form_arg = $this->getMock('Drupal\Core\Form\FormInterface');
+    $form_arg = $this->createMock('Drupal\Core\Form\FormInterface');
     $form_arg->expects($this->exactly(2))
       ->method('getFormId')
-      ->will($this->returnValue($form_id));
+      ->willReturn($form_id);
     $form_arg->expects($this->exactly(4))
       ->method('buildForm')
-      ->will($this->returnValue($expected_form));
+      ->willReturn($expected_form);
 
     // Do an initial build of the form and track the build ID.
     $form_state = new FormState();
@@ -378,18 +393,18 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the rebuildForm() method for a GET submission.
    */
-  public function testRebuildFormOnGetRequest() {
+  public function testRebuildFormOnGetRequest(): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
 
     // The form will be built four times.
-    $form_arg = $this->getMock('Drupal\Core\Form\FormInterface');
+    $form_arg = $this->createMock('Drupal\Core\Form\FormInterface');
     $form_arg->expects($this->exactly(2))
       ->method('getFormId')
-      ->will($this->returnValue($form_id));
+      ->willReturn($form_id);
     $form_arg->expects($this->exactly(4))
       ->method('buildForm')
-      ->will($this->returnValue($expected_form));
+      ->willReturn($expected_form);
 
     // Do an initial build of the form and track the build ID.
     $form_state = new FormState();
@@ -416,20 +431,20 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests the getCache() method.
    */
-  public function testGetCache() {
+  public function testGetCache(): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
     $expected_form['#token'] = FALSE;
 
     // FormBuilder::buildForm() will be called twice, but the form object will
     // only be called once due to caching.
-    $form_arg = $this->getMock('Drupal\Core\Form\FormInterface');
+    $form_arg = $this->createMock('Drupal\Core\Form\FormInterface');
     $form_arg->expects($this->exactly(2))
       ->method('getFormId')
-      ->will($this->returnValue($form_id));
+      ->willReturn($form_id);
     $form_arg->expects($this->once())
       ->method('buildForm')
-      ->will($this->returnValue($expected_form));
+      ->willReturn($expected_form);
 
     // Do an initial build of the form and track the build ID.
     $form_state = (new FormState())
@@ -459,19 +474,19 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests that HTML IDs are unique when rebuilding a form with errors.
    */
-  public function testUniqueHtmlId() {
+  public function testUniqueHtmlId(): void {
     $form_id = 'test_form_id';
     $expected_form = $form_id();
     $expected_form['test']['#required'] = TRUE;
 
     // Mock a form object that will be built two times.
-    $form_arg = $this->getMock('Drupal\Core\Form\FormInterface');
+    $form_arg = $this->createMock('Drupal\Core\Form\FormInterface');
     $form_arg->expects($this->exactly(2))
       ->method('getFormId')
-      ->will($this->returnValue($form_id));
+      ->willReturn($form_id);
     $form_arg->expects($this->exactly(2))
       ->method('buildForm')
-      ->will($this->returnValue($expected_form));
+      ->willReturn($expected_form);
 
     $form_state = new FormState();
     $form = $this->simulateFormSubmission($form_id, $form_arg, $form_state);
@@ -482,10 +497,40 @@ class FormBuilderTest extends FormTestBase {
     $this->assertSame('test-form-id--2', $form['#id']);
   }
 
+  /**
+   * Tests that HTML IDs are unique between 2 forms with the same element names.
+   */
+  public function testUniqueElementHtmlId(): void {
+    $form_id_1 = 'test_form_id';
+    $form_id_2 = 'test_form_id_2';
+    $expected_form = $form_id_1();
+
+    // Mock 2 form objects that will be built once each.
+    $form_arg_1 = $this->createMock('Drupal\Core\Form\FormInterface');
+    $form_arg_1->expects($this->exactly(1))
+      ->method('getFormId')
+      ->willReturn($form_id_1);
+    $form_arg_1->expects($this->exactly(1))
+      ->method('buildForm')
+      ->willReturn($expected_form);
+    $form_arg_2 = $this->createMock('Drupal\Core\Form\FormInterface');
+    $form_arg_2->expects($this->exactly(1))
+      ->method('getFormId')
+      ->willReturn($form_id_2);
+    $form_arg_2->expects($this->exactly(1))
+      ->method('buildForm')
+      ->willReturn($expected_form);
+    $form_state = new FormState();
+    $form_1 = $this->simulateFormSubmission($form_id_1, $form_arg_1, $form_state);
+    $form_state = new FormState();
+    $form_2 = $this->simulateFormSubmission($form_id_2, $form_arg_2, $form_state);
+    $this->assertNotSame($form_1['actions']["#id"], $form_2['actions']["#id"]);
+  }
+
   /**
    * Tests that a cached form is deleted after submit.
    */
-  public function testFormCacheDeletionCached() {
+  public function testFormCacheDeletionCached(): void {
     $form_id = 'test_form_id';
     $form_build_id = $this->randomMachineName();
 
@@ -512,7 +557,7 @@ class FormBuilderTest extends FormTestBase {
   /**
    * Tests that an uncached form does not trigger cache set or delete.
    */
-  public function testFormCacheDeletionUncached() {
+  public function testFormCacheDeletionUncached(): void {
     $form_id = 'test_form_id';
     $form_build_id = $this->randomMachineName();
 
@@ -530,13 +575,14 @@ class FormBuilderTest extends FormTestBase {
   /**
    * @covers ::buildForm
    */
-  public function testExceededFileSize() {
+  public function testExceededFileSize(): void {
     $request = new Request([FormBuilderInterface::AJAX_FORM_REQUEST => TRUE]);
+    $request->setSession(new Session(new MockArraySessionStorage()));
     $request_stack = new RequestStack();
     $request_stack->push($request);
     $this->formBuilder = $this->getMockBuilder('\Drupal\Core\Form\FormBuilder')
       ->setConstructorArgs([$this->formValidator, $this->formSubmitter, $this->formCache, $this->moduleHandler, $this->eventDispatcher, $request_stack, $this->classResolver, $this->elementInfo, $this->themeManager, $this->csrfToken])
-      ->setMethods(['getFileUploadMaxSize'])
+      ->onlyMethods(['getFileUploadMaxSize'])
       ->getMock();
     $this->formBuilder->expects($this->once())
       ->method('getFileUploadMaxSize')
@@ -545,16 +591,42 @@ class FormBuilderTest extends FormTestBase {
     $form_arg = $this->getMockForm('test_form_id');
     $form_state = new FormState();
 
-    $this->setExpectedException(BrokenPostRequestException::class);
+    $this->expectException(BrokenPostRequestException::class);
     $this->formBuilder->buildForm($form_arg, $form_state);
   }
 
   /**
    * @covers ::buildForm
    */
-  public function testGetPostAjaxRequest() {
+  public function testPostAjaxRequest(): void {
     $request = new Request([FormBuilderInterface::AJAX_FORM_REQUEST => TRUE], ['form_id' => 'different_form_id']);
     $request->setMethod('POST');
+    $request->setSession(new Session(new MockArraySessionStorage()));
+    $this->requestStack->push($request);
+
+    $form_state = (new FormState())
+      ->setUserInput([FormBuilderInterface::AJAX_FORM_REQUEST => TRUE])
+      ->setMethod('get')
+      ->setAlwaysProcess()
+      ->disableRedirect()
+      ->set('ajax', TRUE);
+
+    $form_id = '\Drupal\Tests\Core\Form\TestForm';
+    $expected_form = (new TestForm())->buildForm([], $form_state);
+
+    $form = $this->formBuilder->buildForm($form_id, $form_state);
+    $this->assertFormElement($expected_form, $form, 'test');
+    $this->assertSame('test-form', $form['#id']);
+  }
+
+  /**
+   * @covers ::buildForm
+   */
+  public function testGetAjaxRequest(): void {
+    $request = new Request([FormBuilderInterface::AJAX_FORM_REQUEST => TRUE]);
+    $request->query->set('form_id', 'different_form_id');
+    $request->setMethod('GET');
+    $request->setSession(new Session(new MockArraySessionStorage()));
     $this->requestStack->push($request);
 
     $form_state = (new FormState())
@@ -577,7 +649,7 @@ class FormBuilderTest extends FormTestBase {
    *
    * @dataProvider providerTestChildAccessInheritance
    */
-  public function testChildAccessInheritance($element, $access_checks) {
+  public function testChildAccessInheritance($element, $access_checks): void {
     $form_arg = new TestFormWithPredefinedForm();
     $form_arg->setForm($element);
 
@@ -606,7 +678,7 @@ class FormBuilderTest extends FormTestBase {
    *
    * @return array
    */
-  public function providerTestChildAccessInheritance() {
+  public static function providerTestChildAccessInheritance() {
     $data = [];
 
     $element = [
@@ -740,14 +812,13 @@ class FormBuilderTest extends FormTestBase {
    *
    * @dataProvider providerTestValueCallableIsSafe
    */
-  public function testValueCallableIsSafe($callback, $expected) {
+  public function testValueCallableIsSafe($callback, $expected): void {
     $method = new \ReflectionMethod(FormBuilder::class, 'valueCallableIsSafe');
-    $method->setAccessible(TRUE);
     $is_safe = $method->invoke($this->formBuilder, $callback);
     $this->assertSame($expected, $is_safe);
   }
 
-  public function providerTestValueCallableIsSafe() {
+  public static function providerTestValueCallableIsSafe() {
     $data = [];
     $data['string_no_slash'] = [
       'Drupal\Core\Render\Element\Token::valueCallback',
@@ -777,7 +848,7 @@ class FormBuilderTest extends FormTestBase {
    *
    * @dataProvider providerTestInvalidToken
    */
-  public function testInvalidToken($expected, $valid_token, $user_is_authenticated) {
+  public function testInvalidToken($expected, $valid_token, $user_is_authenticated): void {
     $form_token = 'the_form_token';
     $form_id = 'test_form_id';
 
@@ -793,21 +864,38 @@ class FormBuilderTest extends FormTestBase {
     $current_user = $this->prophesize(AccountInterface::class);
     $current_user->isAuthenticated()->willReturn($user_is_authenticated);
     $property = new \ReflectionProperty(FormBuilder::class, 'currentUser');
-    $property->setAccessible(TRUE);
     $property->setValue($this->formBuilder, $current_user->reveal());
 
     $expected_form = $form_id();
     $form_arg = $this->getMockForm($form_id, $expected_form);
 
+    // Set up some request data so we can be sure it is removed when a token is
+    // invalid.
+    $this->request->request->set('foo', 'bar');
+    $_POST['foo'] = 'bar';
+
     $form_state = new FormState();
     $input['form_id'] = $form_id;
     $input['form_token'] = $form_token;
+    $input['test'] = 'example-value';
     $form_state->setUserInput($input);
-    $this->simulateFormSubmission($form_id, $form_arg, $form_state, FALSE);
+    $form = $this->simulateFormSubmission($form_id, $form_arg, $form_state, FALSE);
     $this->assertSame($expected, $form_state->hasInvalidToken());
+    if ($expected) {
+      $this->assertEmpty($form['test']['#value']);
+      $this->assertEmpty($form_state->getValue('test'));
+      $this->assertEmpty($_POST);
+      $this->assertEmpty(iterator_to_array($this->request->request->getIterator()));
+    }
+    else {
+      $this->assertEquals('example-value', $form['test']['#value']);
+      $this->assertEquals('example-value', $form_state->getValue('test'));
+      $this->assertEquals('bar', $_POST['foo']);
+      $this->assertEquals('bar', $this->request->request->get('foo'));
+    }
   }
 
-  public function providerTestInvalidToken() {
+  public static function providerTestInvalidToken() {
     $data = [];
     $data['authenticated_invalid'] = [TRUE, FALSE, TRUE];
     $data['authenticated_valid'] = [FALSE, TRUE, TRUE];
@@ -821,7 +909,7 @@ class FormBuilderTest extends FormTestBase {
    *
    * @dataProvider providerTestFormTokenCacheability
    */
-  public function testFormTokenCacheability($token, $is_authenticated, $expected_form_cacheability, $expected_token_cacheability, $method) {
+  public function testFormTokenCacheability($token, $is_authenticated, $expected_form_cacheability, $expected_token_cacheability, $method): void {
     $user = $this->prophesize(AccountProxyInterface::class);
     $user->isAuthenticated()
       ->willReturn($is_authenticated);
@@ -836,18 +924,18 @@ class FormBuilderTest extends FormTestBase {
       $form['#token'] = $token;
     }
 
-    $form_arg = $this->getMock('Drupal\Core\Form\FormInterface');
+    $form_arg = $this->createMock('Drupal\Core\Form\FormInterface');
     $form_arg->expects($this->once())
       ->method('getFormId')
-      ->will($this->returnValue($form_id));
+      ->willReturn($form_id);
     $form_arg->expects($this->once())
       ->method('buildForm')
-      ->will($this->returnValue($form));
+      ->willReturn($form);
 
     $form_state = new FormState();
     $built_form = $this->formBuilder->buildForm($form_arg, $form_state);
     if (!isset($expected_form_cacheability) || ($method == 'get' && !is_string($token))) {
-      $this->assertFalse(isset($built_form['#cache']));
+      $this->assertEquals($built_form['#cache'], ['tags' => ['CACHE_MISS_IF_UNCACHEABLE_HTTP_METHOD:form']]);
     }
     else {
       $this->assertTrue(isset($built_form['#cache']));
@@ -867,20 +955,21 @@ class FormBuilderTest extends FormTestBase {
    *
    * @return array
    */
-  public function providerTestFormTokenCacheability() {
+  public static function providerTestFormTokenCacheability() {
     return [
-      'token:none,authenticated:true' => [NULL, TRUE, ['contexts' => ['user.roles:authenticated']], ['max-age' => 0], 'post'],
-      'token:none,authenticated:false' => [NULL, FALSE, ['contexts' => ['user.roles:authenticated']], NULL, 'post'],
+      'token:none,authenticated:true' => [NULL, TRUE, ['contexts' => ['user.roles:authenticated'], 'tags' => ['CACHE_MISS_IF_UNCACHEABLE_HTTP_METHOD:form']], ['max-age' => 0], 'post'],
+      'token:none,authenticated:false' => [NULL, FALSE, ['contexts' => ['user.roles:authenticated'], 'tags' => ['CACHE_MISS_IF_UNCACHEABLE_HTTP_METHOD:form']], NULL, 'post'],
       'token:false,authenticated:false' => [FALSE, FALSE, NULL, NULL, 'post'],
       'token:false,authenticated:true' => [FALSE, TRUE, NULL, NULL, 'post'],
-      'token:none,authenticated:false,method:get' => [NULL, FALSE, ['contexts' => ['user.roles:authenticated']], NULL, 'get'],
-      'token:test_form_id,authenticated:false,method:get' => ['test_form_id', TRUE, ['contexts' => ['user.roles:authenticated']], ['max-age' => 0], 'get'],
+      'token:none,authenticated:false,method:get' => [NULL, FALSE, ['contexts' => ['user.roles:authenticated'], 'tags' => ['CACHE_MISS_IF_UNCACHEABLE_HTTP_METHOD:form']], NULL, 'get'],
+      'token:test_form_id,authenticated:false,method:get' => ['test_form_id', TRUE, ['contexts' => ['user.roles:authenticated'], 'tags' => ['CACHE_MISS_IF_UNCACHEABLE_HTTP_METHOD:form']], ['max-age' => 0], 'get'],
     ];
   }
 
 }
 
 class TestForm implements FormInterface {
+
   public function getFormId() {
     return 'test_form';
   }
@@ -888,11 +977,14 @@ class TestForm implements FormInterface {
   public function buildForm(array $form, FormStateInterface $form_state) {
     return test_form_id();
   }
+
   public function validateForm(array &$form, FormStateInterface $form_state) {}
+
   public function submitForm(array &$form, FormStateInterface $form_state) {}
 
 }
 class TestFormInjected extends TestForm implements ContainerInjectionInterface {
+
   public static function create(ContainerInterface $container) {
     return new static();
   }
